// {{{ copyright

/********************************************************************
 *
 * $Id: Options.java,v 1.9 2001/04/24 16:06:48 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 2000 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

// {{{ imports

import java.awt.Component;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.FileInputStream;
import java.io.FileOutputStream;

import javax.swing.JButton;

import de.qfs.lib.config.Configuration;
import de.qfs.lib.gui.FocusFixer;
import de.qfs.lib.gui.Message;
import de.qfs.lib.gui.SwingUtil;
import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;
import de.qfs.lib.option.BooleanOption;
import de.qfs.lib.option.ChoiceOption;
import de.qfs.lib.option.EditorOption;
import de.qfs.lib.option.IntegerOption;
import de.qfs.lib.option.KeyOption;
import de.qfs.lib.option.NestedOption;
import de.qfs.lib.option.Option;
import de.qfs.lib.option.OptionDialog;
import de.qfs.lib.option.OptionSet;
import de.qfs.lib.option.OptionValidator;
import de.qfs.lib.option.StringOption;
import de.qfs.lib.option.TableOption;
import de.qfs.lib.option.TextOption;
import de.qfs.lib.util.ArgsParser;
import de.qfs.lib.util.LogSetup;
import de.qfs.lib.util.MapResourceBundle;

// }}}

/**
 * This program demonstrates the use of the qflib option package and the
 * configuration package. Please keep in mind that this is just an example and
 * not suitable for a real application, since it lacks proper structure and
 * error checking, among other things.
 *
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.9 $
 */
public class Options
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger = new Logger (Options.class);

    /**
     * Name of the configuration file.
     */
    private final static String CFG = "options.cfg";

    public static OptionDialog dialog;
    // }}}

    // {{{ main

    /**
     * Main entry point for the Optin demo.
     *
     * @param   args    Command line arguments. All standard logging args
     *                  from LogSetup are supported.
     */
    public static void main(String[] args)
    {
        // Initialization
        //---------------

        // Set up the logging system
        Log.setOutputLevel(Log.ERRDETAIL);
        ArgsParser ap = new ArgsParser ();
        LogSetup.instance().addOptions(ap);
        try {
            ap.parse(args);
            LogSetup.instance().setupLogging("Options", ap);
        } catch (Exception ex) {
        }

        // Miscellaneous preparations
        SwingUtil.prepareKeymaps();

        // Initialize the resources for the options
        MapResourceBundle rb = new MapResourceBundle ();
        rb.fetchProperties("options", Options.class);
        Option.addResources(rb);
        // Use these resources for error messages also
        Message.addResources(rb);

        // Create and set up the OptionGroups
        //-----------------------------------

        // The main option group that contains everything else
        final OptionSet main = new OptionSet ("main");

        // The table data
        TableOption table = new TableOption
            ("table", new Object[][] {
                new Object[] {"Alpha", new Integer (10), Boolean.TRUE, null},
                new Object[] {"Bravo", new Integer (20), Boolean.FALSE,
                              "Multi\nLine\nComment"},
                new Object[] {"Charlie", new Integer (30), Boolean.TRUE, null}
            });
        main.add(table);

        // The options for the note tab
        String text = rb.getString("note.text", "Missing: note.text");
        EditorOption note = new EditorOption ("note", text) {
            // This value should neither be saved nor restored
            // Note: If there were more options that mustn't be saved, it
            // would be simpler to use a Configurator to collect the
            // OptionSets that contain saveable options and register that with
            // the Configuration instead of the main group.
            public String getConfigurableValue() {return "";}
            public void setConfigurableValue(String value) {}
        };
        main.add(note);

        // The options for the simple tab
        main.add(new BooleanOption ("boolean", true));
        main.add(new IntegerOption ("integer", 0, true, true));
        main.add(new StringOption ("string", "Some string", true));
        main.add(new TextOption ("text", "This is some\nmulti-line text.",
                                 true));
        main.add(new KeyOption ("key", null));
        main.add(new KeyOption ("key2", null));

        // The integers tab uses a group of type nested, so we put the
        // Options in a NestedOption
        OptionSet integers = new OptionSet ("integers");
        integers.add(new IntegerOption ("integer1", 0));
        integers.add(new IntegerOption ("integer2", -1, false, true));
        integers.add(new IntegerOption ("integer3", 3, true, true));
        IntegerOption special = new IntegerOption ("integer4", 4, true, true);

        special.addValidator(new OptionValidator () {
            public boolean checkValue(Component component, Option option,
                                      Object value)
            {
                if (value != null &&
                    ((Integer) value).intValue() % 2 != 0) {
                    Message.showMessage(component, "mustBeEven");
                    return false;
                }
                return true;
            }
        });
        integers.add(special);
        main.add(new NestedOption ("integers", integers));

        // The choices tab uses the '/' syntax for nested options
        OptionSet choices = new OptionSet ("choices");
        choices.add(new IntegerOption ("radio", 1));
        choices.add(new ChoiceOption ("combo", "Different choice"));
        choices.add(new ChoiceOption ("combo2", "First choice"));
        main.add(new NestedOption ("choices", choices));

        // Setup and try to restore the configuration
        //---------------------------------
        Configuration.instance().register(main);
        try {
            Configuration.instance().restore
                (new FileInputStream (CFG));
            System.err.println("Configuration restored");
        } catch (Exception ex) {
            System.err.println("Configuration not read");
        }

        // Create the dialog
        //------------------

        // From here on stuff should better run in the AWT event loop
        SwingUtil.invokeAndWait(new Runnable () {
            public void run() {
                dialog =
                    new OptionDialog("demo", (Frame) null, main, "tree");

                // Not necessary but nice to have
                FocusFixer.instance().addWindow(dialog);

                // Setup the action for the save button
                JButton save = dialog.getButton("save");
                if (save == null) {
                    if (logger.level >= Log.ERR) {
                        logger.log(Log.ERR, "main(String[])",
                                   "No save button");
                    }
                } else {
                    save.addActionListener(new ActionListener () {
                        public void actionPerformed(ActionEvent e) {
                            if (dialog.readValues()) {
                                try {
                                    Configuration.instance().save
                                        (new FileOutputStream (CFG));
                                    System.err.println
                                        ("Configuration saved.");
                                } catch (Exception ex) {
                                    System.err.println
                                        ("Configuration not saved.");
                                }
                            }
                        }
                    });
                }
                // SwingUtil.centerWindow(dialog);
                dialog.doModal(null, true);
            }
        });

        // Cleanup and exit
        //-----------------
        LogSetup.instance().stopLogging();
        System.exit(0);
    }

    // }}}
}
