// {{{ copyright

/********************************************************************
 *
 * $Id: SortedTable.java,v 1.5 2000/12/27 18:28:28 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 2000 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

// {{{ imports

import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.util.Enumeration;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import de.qfs.lib.gui.SortedTableHelper;
import de.qfs.lib.gui.TableModelFilter;
import de.qfs.lib.gui.TableModelFilterEvent;
import de.qfs.lib.gui.TableModelFilterListener;
import de.qfs.lib.log.Log;
import de.qfs.lib.util.LogSetup;
import de.qfs.lib.util.ArgsParser;

// }}}

/**
 * This is a little demonstration showing how to create and manipulate a
 * filtered and sorted JTable. Please keep in mind that this is just an
 * example and not suitable for a real application, since it lacks good
 * structure and error checking, among other things.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.5 $
 */
public class SortedTable
{
    // {{{ variables

    /**
     * The data for the table.
     */
    private static Object[][] tableData = new Object[][] {
        {new Integer (0), "A", new Double (Math.random()),
         new Boolean (true)},
        {new Integer (1), "C", new Double (Math.random()),
         new Boolean (false)},
        {new Integer (2), "D", new Double (Math.random()),
         new Boolean (false)},
        {new Integer (3), "B", new Double (Math.random()),
         new Boolean (true)}
        };

    /**
     * The column names for the table.
     */
    private static String[] tableHeaders = new String[] {
        "Number", "Letter", "Random", "Flag"
    };

    /**
     * Possible filter settings.
     */
    private static String[] filterNames = new String[] {
        "No Filter",
        "Even Numbers",
        "Only C or D",
        "Random > 0.5",
        "True Flag"
    };

    /**
     * The filter for the table data.
     */
    private static Filter filter = new Filter ();

    /**
     * The SortedTableHelper used to set up the sorting and filtering.
     */
    private static SortedTableHelper helper;

    // }}}

    // {{{ main

    /**
     * The main entry point for the SortedTable.
     *
     * @param   args    The command line arguments - ignored.
     */
    public static void main(String[] args)
    {
        // Set up the logging system
        Log.setOutputLevel(Log.ERRDETAIL);
        ArgsParser ap = new ArgsParser ();
        LogSetup.instance().addOptions(ap);
        try {
            ap.parse(args);
            LogSetup.instance().setupLogging("Options", ap);
        } catch (Exception ex) {
        }

        // Create the frame
        JFrame frame = new JFrame ("Sorted table demo");
        frame.setName("Main");
        frame.addWindowListener(new WindowAdapter () {
            public void windowClosing(WindowEvent e) {
                // Closing the window will terminate the demo
                System.exit(0);
            }
        });

        JPanel pane = new JPanel ();
        pane.setLayout(new BorderLayout ());
        pane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        frame.setContentPane(pane);

        // Create the table model
        DefaultTableModel model = new DefaultTableModel
            (tableData, tableHeaders) {

            // Set up the column classes
            public Class getColumnClass(int column) {
                switch (column) {
                    case 0:
                        return Integer.class;
                    case 1:
                        return String.class;
                    case 2:
                        return Double.class;
                    case 3:
                        return Boolean.class;
                }
                return Object.class;
            }

            // Only the String and the Checkbox columns are editable
            public boolean isCellEditable(int row, int column)
            {
                return column == 1 || column == 3;
            }
        };

        // Create the table
        final JTable table = new JTable(model);
        table.setName("Table");
        table.getTableHeader().setName("TableHeader");
        // Set it up for sorting and filtering using the SortedTableHelper
        helper = new SortedTableHelper (table);
        helper.setTableModelFilter(filter);
        helper.prepareTable();

        pane.add(new JScrollPane (table));

        // Create the combo box for the filter
        JComboBox combo = new JComboBox (filterNames);
        combo.setName("Filters");
        combo.setSelectedIndex(0);
        pane.add(combo, "North");

        combo.addItemListener(new ItemListener () {
            public void itemStateChanged(ItemEvent e) {
                // Notify the filter when the selection of the combo changes
                filter.setFilterType(((JComboBox) e.getSource())
                                     .getSelectedIndex());
            }
        });

        // Create the "Dump" button
        JButton dump = new JButton ("Dump");
        dump.setName("Dump");
        dump.addActionListener(new ActionListener () {
            public void actionPerformed(ActionEvent e) {
                // Display all selected row numbers in table coordinates and
                // in original model coordinates
                int[] rows = table.getSelectedRows();
                if (rows != null && rows.length > 0) {
                    System.err.print("Selected:");
                    for (int i = 0; i < rows.length; i++) {
                        System.err.print(" " + rows[i]);
                    }
                    System.err.println();
                    System.err.print(" Maps to:");
                    for (int i = 0; i < rows.length; i++) {
                        System.err.print(" " + helper.getSortedTableModel()
                                               .getMappedRow(rows[i]));
                    }
                    System.err.println();
                }
            }
        });

        // Create the exit button
        JButton exit = new JButton ("Exit");
        exit.setName("Exit");
        exit.addActionListener(new ActionListener () {
            public void actionPerformed(ActionEvent e) {
                LogSetup.instance().stopLogging();
                System.exit(0);
            }
        });

        // Add the buttons
        JPanel panel = new JPanel ();
        panel.add(dump);
        panel.add(exit);
        frame.getContentPane().add(panel, "South");

        // Off we go
        frame.setSize(300, 220);
        frame.setLocation(300, 100);
        frame.show();
    }

    // }}}

    //----------------------------------------------------------------------
    // The class that implements the TableModelFilter inteface
    //----------------------------------------------------------------------
    // {{{ class Filter

    /**
     * An Object of this class will be used to filter the rows of the table.
     */
    private static class Filter
        implements TableModelFilter
    {
        // {{{ variables

        /**
         * Selector that decides what rows to filter.
         */
        private int filterType;

        /**
         * The registered TableModelFilterListeners.
         */
        private Vector listeners = new Vector ();

        // }}}

        // {{{ constructor

        /**
         * Create a new Filter.
         */
        public Filter ()
        {
        }

        // }}}

        // {{{ setFilterType

        /**
         * Select which rows to filter.
         *
         * @param       filterType      The selector that determines the
         *                              filtered rows.
         */
        public void setFilterType(int filterType)
        {
            this.filterType = filterType;
            fireFilterChanged();
        }

        // }}}
        // {{{ filter

        /**
         * Decide whether a row should be displayed or not. The result depends
         * on the selector set with {@link #setFilterType setFilterType}.
         *
         * @param       model   The original table model.
         * @param       row     The row to filter.
         *
         * @return      True to display the row, false to hide it.
         */
        public boolean filter (TableModel model, int row)
        {
            switch(filterType) {
            case 0:
                // none
                return true;
            case 1:
                // even
                return ((Integer) model.getValueAt(row, 0)).intValue() % 2
                    == 0;
            case 2:
                // C or D
                String val = (String) model.getValueAt(row, 1);
                return val.equals("C") || val.equals("D");
            case 3:
                // > 0.5
                return ((Double) model.getValueAt(row, 2)).doubleValue() > 0.5;
            case 4:
                // true flag
                return ((Boolean) model.getValueAt(row, 3)).booleanValue();
            }
            return true;
        }

        // }}}
        // {{{ addTableModelFilterListener

        /**
         * Add a TableModelFilterListener to the TableModelFilter.
         *
         * @param   listener        The TableModelFilterListener to add.
         */
        public synchronized void addTableModelFilterListener
            (TableModelFilterListener listener)
        {
            if (! listeners.contains(listener)) {
                listeners.addElement(listener);
            }
        }

        // }}}
        // {{{ removeTableModelFilterListener

        /**
         * Remove a TableModelFilterListener from the TableModelFilter.
         *
         * @param   listener        The TableModelFilterListener to remove.
         */
        public synchronized void removeTableModelFilterListener
            (TableModelFilterListener listener)
        {
            listeners.removeElement(listener);
        }

        // }}}
        // {{{ fireFilterChanged

        /**
         * Notify the registered listeners of a change in the filter.
         */
        private void fireFilterChanged()
        {
            Vector tmp;
            synchronized(this) {
                tmp = (Vector) listeners.clone();
            }

            TableModelFilterEvent event = new TableModelFilterEvent (this);
            for (Enumeration enum = tmp.elements();
                 enum.hasMoreElements(); ) {
                ((TableModelFilterListener) enum.nextElement())
                    .filterChanged(event);
            }
        }

        // }}}
    }

    // }}}
}
