// {{{ copyright

/********************************************************************
 *
 * $Id: Version.jpp,v 1.14 2001/02/27 10:04:30 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib;

// {{{ imports

import java.text.NumberFormat;

// }}}

/**
 * This class provides access to the qflib version number. The version
 * is composed of a major a minor and a release part. <p>
 *
 * Releases of qflib with different major version are generally considered
 * incompatible. Differences in minor version are also considered incompatible,
 * but may not be fatal. <p>
 *
 * The release number is increased for bug fixes and extensions that do not
 * change any visible interfaces.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.14 $
 */
public class Version
{
    // {{{ Variables

    /**
     * The major version.
     */
    public final static int MAJOR = 0;

    /**
     * The minor version.
     */
    public final static int MINOR = 98;

    /**
     * The release number.
     */
    public final static int RELEASE = 1;

    /**
     * The JDK version to which this qflib version is compatible.
     */

    public final static String JDK_VERSION = "1.2";


    // }}}

    // {{{ getVersion

    /**
     * Get the full qflib version as a String.
     *
     * @return  The qflib version in the format MAJOR.MINOR.RELEASE
     */
    public static String getVersion()
    {
        NumberFormat nf = NumberFormat.getInstance();
        nf.setMinimumIntegerDigits(2);

        return MAJOR + "." + nf.format(MINOR) + "." + RELEASE;
    }

    // }}}
    // {{{ require

    /**
     * Check, whether this release of qflib is compatible with the required
     * version.
     *
     * @param   major   The required major version. Must be equal to
     *                  {@link #MAJOR MAJOR}.
     * @param   minor   The required minor version. Must be equal to
     *                  {@link #MINOR MINOR}.
     * @param   release The required major version. Must be less than or equal
     *                  to {@link #RELEASE RELEASE}.
     *
     * @return  True if this release of qflib is compatible with the requested
     *          one, false otherwise.
     */
    public final static boolean require(int major, int minor, int release)
    {
        return major == MAJOR && minor == MINOR && release <= RELEASE;
    }

    // }}}

    // {{{ main

    /**
     * Dump the version numbers for this release to System.out.<br>
     * With argument -version print the whole version number.<br>
     * With argument -major print only the major number.<br>
     * With argument -minor print only the minor number.<br>
     * With argument -release print only the release number.<br>
     * With argument -jdk print only the JDK version for which this qflib
     * version was compiled.<br>
     * Otherwise print a message containing the full version.
     */
    public static void main(String[] argv)
    {
        if (argv.length > 0 && argv[0].equals("-version")) {
            System.out.println("" + getVersion());
        } else if (argv.length > 0 && argv[0].equals("-major")) {
            System.out.println("" + MAJOR);
        } else if (argv.length > 0 && argv[0].equals("-minor")) {
            System.out.println("" + MINOR);
        } else if (argv.length > 0 && argv[0].equals("-release")) {
            System.out.println("" + RELEASE);
        } else if (argv.length > 0 && argv[0].equals("-jdk")) {
            System.out.println("" + 12 );
        } else {
            System.out.println("This is qflib release " + getVersion()+
                               ", compatible with JDK " + JDK_VERSION);
        }
        System.exit(0);
    }

    // }}}
}
