// {{{ copyright

/********************************************************************
 *
 * $Id: ConfigurationEvent.java,v 1.6 2000/06/06 16:41:25 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.config;

// {{{ imports

import java.util.EventObject;
import java.util.Properties;

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

/**
 * This event encapsulates the details of a state change in a {@link
 * Configurable Configurable} registered with the {@link Configuration
 * Configuration}.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.6 $
 */
public class ConfigurationEvent
    extends EventObject
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private static Logger logger = new Logger (ConfigurationEvent.class);

    /**
     * The name under which the changed Configurable is registered.
     */
    private String configurableName;

    /**
     * The new state.
     */
    private Properties state;

    // }}}

    // {{{ ConfigurationEvent

    /**
     * Create a new ConfigurationEvent.
     *
     * @param   source  The Configurable that changed state.
     * @param   name    The name under which the Configurable is registered.
     * @param   state   The new state of the Configurable.
     */
    public ConfigurationEvent(Configurable source, String name,
                              Properties state)
    {
        super (source);
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD,
                       "ConfigurationEvent(Configurable,String,Properties)",
                       "name: " + name);
        }
        configurableName = name;
        this.state = state;
    }

    // }}}

    // {{{ getConfigurableName

    /**
     * Get the name under which the Configurable is registered.
     *
     * @return  The Configurable's registration name.
     */
    public final String getConfigurableName()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getConfigurableName()", "");
        }
        return configurableName;
    }

    // }}}
    // {{{ getConfigurable

    /**
     * Get the changed Configurable.
     *
     * @return  The Configurable that changed.
     */
    public final Configurable getConfigurable()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getConfigurable()", "");
        }
        return (Configurable) getSource();
    }

    // }}}
    // {{{ state

    /**
     * Get the new state of the Configurable.
     *
     * @return  The Configurable's new state.
     */
    public final Properties state()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "state()", "");
        }
        return state;
    }

    // }}}

    // {{{ toString

    /**
     * Get a String representation of the ConfigurationEvent
     *
     * @return  The String representation.
     */
    public String toString()
    {
        return "ConfigurationEvent[configurable=" + getSource() +
            ",name=" + configurableName +
            ",state=" + state + "]";
    }

    // }}}
}
