// {{{ copyright

/********************************************************************
 *
 * $Id: Configurator.java,v 1.10 2001/01/19 19:11:56 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.config;

// {{{ imports

import java.util.Enumeration;
import java.util.Vector;

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

// {{{ doc

/**
 * A Configurator is a composite, holding {@link Configurable Configurables}
 * and other Configurators, thus building a hierarchy which is used to
 * determine the names under which the Configurables are registered with the
 * {@link Configuration Configuration}.  This way it is possible to register
 * similar Configurables under different names, without them having to know
 * anything about their place in the hierarchy. <p>
 *
 * A Configurator is especially useful when building a reusable GUI component
 * that has various subcomponents requiring confguration. All the nested
 * Configurables can be collected under one Configurator hierarchy, that can
 * be made accessible through the components interface. The user of the
 * component can the register all Configurables at once.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.10 $
 */

// }}}
public class Configurator
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger = new Logger (Configurator.class);

    /**
     * The Configurations the Configurator is registered with.
     */
    private Vector registered;

    /**
     * The name of the Configurator.
     */
    private String name;

    /**
     * The full name of the Configurator, including its parent's fullname.
     */
    private String fullname;

    /**
     * The parent of the Configurator.
     */
    private Configurator parent;

    /**
     * The children of the Configurator.
     */
    private Vector children = new Vector();

    // }}}

    // {{{ constructor

    /**
     * Create a new Configurator.
     *
     * @param   name    The name of the Configurator.
     */
    public Configurator(String name)
    {
        this.name = name;
        registered = new Vector ();
    }

    // }}}

    // {{{ add

    /**
     * Add a Configurable to the Configurator's children.
     *
     * If the receiver was already registered with the Configuration,
     * the Configurable is registered as well.
     *
     * @param   configurable    The Configurable to add.
     */
    public void add(Configurable configurable)
    {
        if (children.contains(configurable)) {
            // avoid double registration
            if (logger.level >= Log.WRN) {
                logger.log(Log.WRN, "add(Configurable)",
                           "Already added: " + configurable);
            }
            return;
        }
        children.addElement(configurable);
        for (Enumeration e = registered.elements();
             e.hasMoreElements(); ) {
            ((Configuration) e.nextElement())
                .registerAs(configurable,
                            getName() + "." +
                            configurable.getConfigurableName());
        }
    }

    /**
     * Add a Configurator to the Configurator's children.
     *
     * If the receiver was already registered with the Configuration,
     * the new Configurator is registered as well.
     *
     * @param   configurator    The Configurator to add.
     */
    public void add(Configurator configurator)
    {
        if (children.contains(configurator)) {
            // avoid double registration
            if (logger.level >= Log.WRN) {
                logger.log(Log.WRN, "add(Configurator)",
                           "Already added: " + configurator);
            }
            return;
        }
        children.addElement(configurator);
        configurator.setParent(this);
        for (Enumeration e = registered.elements();
             e.hasMoreElements(); ) {
            configurator.register((Configuration) e.nextElement());
        }
    }

    // }}}
    // {{{ remove

    /**
     * Remove a Configurable from the Configurator's children.
     *
     * If the receiver was already registered with the Configuration,
     * the Configurable is removed.
     *
     * @param   configurable    The Configurable to remove.
     * @param   keepValues      If true, the Configurable's state is kept
     *                          in the Configuration.
     */
    public void remove(Configurable configurable, boolean keepValues)
    {
        if (children.removeElement(configurable)) {
            for (Enumeration e = registered.elements();
                 e.hasMoreElements(); ) {
                ((Configuration) e.nextElement())
                    .unregisterAs(configurable,
                                  getName() + "." +
                                  configurable.getConfigurableName(),
                                  keepValues);
            }
        } else {
            if (logger.level >= Log.WRN) {
                logger.log(Log.WRN, "remove(Configurable,boolean)",
                           "Unknown configurable: " + configurable);
            }
        }
    }

    /**
     * Remove a Configurator from the Configurator's children.
     *
     * If the receiver was already registered with the Configuration,
     * the Configurator is removed.
     *
     * @param   configurator    The Configurator to remove.
     * @param   keepValues      If true, the Configurator's state is kept
     *                          in the Configuration.
     */
    public void remove(Configurator configurator, boolean keepValues)
    {
        if (children.contains(configurator)) {
            for (Enumeration e = registered.elements();
                 e.hasMoreElements(); ) {
                configurator.unregister((Configuration) e.nextElement(),
                                        keepValues);
            }
            children.removeElement(configurator);
            configurator.setParent(null);
        } else {
            if (logger.level >= Log.WRN) {
                logger.log(Log.WRN, "remove(Configurator,boolean)",
                           "Unknown configurator: " + configurator);
            }
        }
    }

    // }}}
    // {{{ isRegistered

    /**
     * Query whether the Configurator is registered with the Configuration.
     *
     * @return  True is the Configurator is registered.
     */
    public boolean isRegistered()
    {
        return isRegistered(Configuration.instance());
    }

    /**
     * Query whether the Configurator is registered with a special
     * Configuration.
     *
     * @param   configuration   The Configuration to test for.
     *
     * @return  True is the Configurator is registered.
     */
    public boolean isRegistered(Configuration configuration)
    {
        return registered.contains(configuration);
    }

    // }}}
    // {{{ register

    /**
     * Register the children of the Configurator with the Configuration.
     *
     * The names used are constructed from the Configurator's parent's name,
     * the Configurator's name and the children's own names.
     */
    public void register()
    {
        register(Configuration.instance());
    }

    /**
     * Register the children of the Configurator with a special Configuration.
     *
     * The names used are constructed from the Configurator's parent's name,
     * the Configurator's name and the children's own names.
     *
     * @param   configuration   The Configuration to register with.
     */
    public void register(Configuration configuration)
    {
        if (registered.contains(configuration)) {
            if (logger.level >= Log.WRN) {
                logger.log(Log.WRN, "register(Configuration)",
                           "Already registered.");
            }
            return;
        }
        fullname = getName();
        for (int i = 0; i < children.size(); i++) {
            Object child = children.elementAt(i);
            if (child instanceof Configurator) {
                ((Configurator)child).register(configuration);
            } else {
                configuration.registerAs
                    ((Configurable)child,
                     getName() + "." +
                     ((Configurable)child).getConfigurableName());
            }
        }
        registered.addElement(configuration);
    }

    // }}}
    // {{{ unregister

    /**
     * Unregister the Configurator's children from the Configuration.
     *
     * @param   keepValues      If true, keep the state of the Configurables
     *                          in the Configuration.
     */
    public void unregister(boolean keepValues)
    {
        unregister(Configuration.instance(), keepValues);
    }

    /**
     * Unregister the Configurator's children from a special Configuration.
     *
     * @param   configuration   The Configuration to unregister from.
     * @param   keepValues      If true, keep the state of the Configurables
     *                          in the Configuration.
     */
    public void unregister(Configuration configuration, boolean keepValues)
    {
        if (!registered.contains(configuration)) {
            if (logger.level >= Log.ERR) {
                logger.log(Log.ERR, "unregister(Configuration,boolean)",
                           "Not registered.");
            }
            return;
        }
        for (int i = 0; i < children.size(); i++) {
            Object child = children.elementAt(i);
            if (child instanceof Configurator) {
                ((Configurator)child).unregister(configuration, keepValues);
            } else {
                configuration.unregisterAs
                    ((Configurable)child,
                     getName() + "." +
                     ((Configurable)child).getConfigurableName(),
                     keepValues);
            }
        }
        registered.removeElement(configuration);
    }

    // }}}
    // {{{ getName

    /**
     * Get the name of the Configurator. This includes the Configurator's
     * parent's name.
     *
     * When the Configurator is registered with the Configuration, its
     * name will be frozen, even if it is moved to a different parent.
     *
     * @return  The name of the Configurator.
     */
    public String getName()
    {
        if (registered.size() > 0) {
            return fullname;
        }
        if (parent != null) {
            return parent.getName() + "." + name;
        }
        return name;
    }

    // }}}
    // {{{ copy

    /**
     * Create a deep copy of the Configurator hierarchy rooted at this
     * Configurator. Configurator children are copied themselves, while
     * Configurable children are added identically to the result. <p>
     *
     * This method is useful to save parts of a Configuration in different
     * files, while still keeping the whole Configuration in one big hierarchy.
     *
     * @return  A deep copy of the Configurator.
     */
    public Configurator copy()
    {
        Configurator result = new Configurator (name);

        for (int i = 0; i < children.size(); i++) {
            Object child = children.elementAt(i);
            if (child instanceof Configurator) {
                result.add(((Configurator) child).copy());
            } else {
                result.add((Configurable) child);
            }
        }
        return result;
    }

    // }}}

    // {{{ setParent

    /**
     * Set the Configurator's parent.
     *
     * @param   parent  The parent to set.
     */
    protected void setParent (Configurator parent)
    {
        this.parent = parent;
    }

    // }}}
}
