// {{{ copyright

/********************************************************************
 *
 * $Id: FrameConfigWrapper.java,v 1.11 2000/06/06 17:00:02 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.config;

// {{{ imports

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Point;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.io.Serializable;

import java.util.Properties;

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

/**
 * A ConfigWrapper for {@link java.awt.Frame Frame} objects. The state saved
 * is the frame's location and size.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.11 $
 */
public class FrameConfigWrapper
    extends ConfigWrapper
    implements ComponentListener
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private static Logger logger = new Logger (FrameConfigWrapper.class);

    /**
     * The Logger used for logging ForceSize methods.
     */
    private final static Logger fsLogger = new Logger (ForceSize.class);

    /**
     * The wrapped Frame.
     */
    protected Frame frame;

    /**
     * Helper for registration callbacks.
     */
    private RegistrationHelper regHelper = new RegistrationHelper ();

    // }}}

    // {{{ constructor

    /**
     * Create a new FrameConfigWrapper with the default name "frame".
     */
    public FrameConfigWrapper()
    {
        setConfigurableName("frame");
    }

    // }}}

    // {{{ setClient

    /**
     * Set the client object for the ConfigWrapper.
     *
     * @param   client  The client object to set.
     */
    protected void setClient(Object client)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "setClient(Object)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "client: " + client);
        }
        frame = (Frame) client;
    }

    // }}}

    //----------------------------------------------------------------------
    // Configurable methods
    //----------------------------------------------------------------------
    // {{{ getConfigurableState

    /**
     * Get the Configurable's state.
     */
    public Properties getConfigurableState()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getConfigurableState()", "");
        }
        Properties state = new Properties ();
        Point pos = frame.getLocation();
        Dimension dim = frame.getSize();
        state.put("x", "" + pos.x);
        state.put("y", "" + pos.y);
        state.put("width", "" + dim.width);
        state.put("height", "" + dim.height);
        if (logger.level >= Log.DBG) {
            logger.log(Log.DBG, "getConfigurableState()",
                       "state: " + state);
        }
        return state;
    }

    // }}}
    // {{{ setConfigurableState

    /**
     * Set the Configurable's state.
     */
    public void setConfigurableState(Properties state)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "setConfigurableState(Properties)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "state: " + state);
        }

        String x = state.getProperty("x");
        String y = state.getProperty("y");
        String width = state.getProperty("width");
        String height = state.getProperty("height");
        if (logger.level >= Log.DBG) {
            logger.log(Log.DBG, "setConfigurableState(Properties)",
                       "x: " + x + ", y: " + y +
                       ", width: " + width + ", height: " + height);
        }
        if (x == null || y == null || width == null || height == null) {
            if (logger.level >= Log.WRN) {
                logger.log(Log.WRN, "setConfigurableState(Properties)",
                           "Invalid state: " +
                           "x: " + x + ", y: " + y +
                           ", width: " + width + ", height: " + height);
            }
            return;
        }

        try {
            Dimension size = new Dimension (Integer.parseInt(width),
                                            Integer.parseInt(height));
            Point pos = new Point (Integer.parseInt(x),
                                   Integer.parseInt(y));
            frame.addWindowListener(new ForceSize (pos, size));
            restore(size, pos);
        } catch (NumberFormatException ex) {
            logger.log(Log.WRN, "setConfigurableState(Properties)", ex);
        }
    }

    // }}}
    // {{{ registrationChanged

    /**
     * This callback is used to notify the Configurable whenever it is
     * registered with or unregistered from the Configuration, so it can
     * decide whether to notify the Configuration when its state changes.
     *
     * @param   config  The affected Configuration.
     * @param   name    The registration name of the Configurable which may
     *                  be different from the Configurable's name.
     * @param   registered True if the Configurable was just registered,
     *                  false if it was unregistered.
     * @param   valuesKept Whether the Configurable's values were kept, if
     *                  it was just unregistered.
     */
    public void registrationChanged(Configuration config, String name,
                                    boolean registered, boolean valuesKept)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD,
                       "registrationChanged(Configuration,String," +
                       "boolean,boolean)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "name: " + name + ", " +
                       "registered: " + registered);
        }
        int ret = regHelper.registrationChanged(config, registered);

        if (frame != null) {
            if (ret == 1) {
                frame.addComponentListener(this);
            } else if (ret == -1) {
                frame.removeComponentListener(this);
            }
        }
    }

    // }}}

    //----------------------------------------------------------------------
    // ComponentListener interface
    //----------------------------------------------------------------------
    // {{{ componentMoved

    /**
     * Notify the Configuration that the state has changed.
     *
     * @param   e       Ignored.
     */
    public void componentMoved(ComponentEvent e)
    {
            if (logger.level >= Log.MTD) {
                    logger.log(Log.MTD, "componentMoved(ComponentEvent)",
                               logger.level < Log.MTDDETAIL ? "" :
                               "e: " + e);
            }
            Configuration.instance().stateChanged(this);
    }

    // }}}
    // {{{ componentResized

    /**
     * Notify the Configuration that the state has changed.
     *
     * @param   e       Ignored.
     */
    public void componentResized(ComponentEvent e)
    {
            if (logger.level >= Log.MTD) {
                    logger.log(Log.MTD, "componentResized(ComponentEvent)",
                               logger.level < Log.MTDDETAIL ? "" :
                               "e: " + e);
            }
            Configuration.instance().stateChanged(this);
    }

    // }}}
    // {{{ componentHidden

    /**
     * Ignored.
     *
     * @param   e       Ignored.
     */
    public void componentHidden(ComponentEvent e)
    {
    }

    // }}}
    // {{{ componentShown

    /**
     * Ignored.
     *
     * @param   e       Ignored.
     */
    public void componentShown(ComponentEvent e)
    {
    }

    // }}}

    //----------------------------------------------------------------------
    // Helper methods
    //----------------------------------------------------------------------
    // {{{ restore

    /**
     * Restore the frame's size and location.
     *
     * @param   size            The size to set.
     * @param   location        The location to set.
     */
    protected void restore (final Dimension size, final Point location)
    {
        frame.setLocation(location);
        frame.pack();
        frame.setSize(size);
        frame.validate();
    }

    // }}}

    //----------------------------------------------------------------------
    // Workaround for weird window manager / AWT Toolkit combinations
    //----------------------------------------------------------------------
    // {{{ class ForceSize

    /**
     * Little helper to fix problems that occur at least with some X11 window
     * magers and cause the size/location of the frame to be set incorrectly.
     */
    private class ForceSize
        extends WindowAdapter
    {
        // {{{ variables

        /**
         * The location to enforce.
         */
        private Point location;

        /**
         * The size to enforce.
         */
        private Dimension size;

        // }}}

        // {{{ constructor

        /**
         * Create a new ForceSize.
         *
         * @param       location        The location to enforce.
         * @param       size            The size to enforce.
         */
        public ForceSize(Point location, Dimension size)
        {
            if (fsLogger.level >= Log.MTD) {
                fsLogger.log(Log.MTD, "ForceSize(Point,Dimension)",
                           fsLogger.level < Log.MTDDETAIL ? "" :
                           "location: " + location + ", " +
                           "size: " + size);
            }

            this.location = location;
            this.size = size;
        }

        // }}}
        // {{{ windowOpened

        /**
         * Notify the listener that the window has been opened.
         *
         * @param       e       Ignored.
         */
        public void windowOpened(WindowEvent e)
        {
            if (fsLogger.level >= Log.MTD) {
                fsLogger.log(Log.MTD, "windowOpened(WindowEvent)", "");
            }
            Dimension dim = frame.getSize();
            Point loc = frame.getLocation();

            if (!dim.equals(size)) {
                if (fsLogger.level >= Log.DBG) {
                    fsLogger.log(Log.DBG, "windowOpened(WindowEvent)",
                               "Size mismatch: " + dim);
                }
                // first try: set it again
                frame.setSize(size);
                dim = frame.getSize();
                if (!dim.equals(size)) {
                    if (fsLogger.level >= Log.DBG) {
                        fsLogger.log(Log.DBG, "windowOpened(WindowEvent)",
                                   "Still size mismatch: " + dim);
                    }
                    // second try - adapt values
                    frame.setSize(2*size.width - dim.width,
                                  2*size.height - dim.height);
                }
            }

            if (!loc.equals(location)) {
                if (fsLogger.level >= Log.DBG) {
                    fsLogger.log(Log.DBG, "windowOpened(WindowEvent)",
                               "Position mismatch: " + loc);
                }
                // first try: set it again
                frame.setLocation(location);
                loc = frame.getLocation();
                if (!loc.equals(location)) {
                    if (fsLogger.level >= Log.DBG) {
                        fsLogger.log(Log.DBG, "windowOpened(WindowEvent)",
                                   "Still position mismatch: " + loc);
                    }
                    // second try - adapt values
                    frame.setLocation(2*location.x - loc.x,
                                      2*location.y - loc.y);
                }
            }

            frame.removeWindowListener(this);
        }

        // }}}
    }

    // }}}
}
