// {{{ copyright

/********************************************************************
 *
 * $Id: JSplitPaneConfigWrapper.java,v 1.13 2000/11/14 12:49:23 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.config;

// {{{ imports

import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import java.io.Serializable;

import java.util.Properties;

import javax.swing.SwingUtilities;
import javax.swing.JSplitPane;

import de.qfs.lib.gui.SwingUtil;
import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

/**
 * A ConfigWrapper for {@link javax.swing.JSplitPane JSplitPane} objects. The
 * state saved is the location of the divider. <p>
 *
 * The JSplitPaneConfigWrapper has special constructors that configure it to
 * force an initial setting of the divider which can otherwise be difficult to
 * achieve due to timing problems.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.13 $
 */
public class JSplitPaneConfigWrapper
    extends ConfigWrapper
    implements PropertyChangeListener
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private static Logger logger = new Logger (JSplitPaneConfigWrapper.class);

    /**
     * The wrapped JSplitPane.
     */
    protected JSplitPane pane;

    /**
     * Whether the wrapper is registered with the Configuration.
     */
    protected boolean registered;

    /**
     * The targetted divider location for the split pane.
     */
    private int targetLocation = -1;

    /**
     * The targetted divider location for the split pane.
     */
    private double relTargetLocation = -1.0;

    /**
     * Flag to avoid multiple override of divider settings.
     */
    private boolean override = true;

    /**
     * Helper for registration callbacks.
     */
    private RegistrationHelper regHelper = new RegistrationHelper ();

    // }}}

    // {{{ constructors

    /**
     * Create a new JSplitPaneConfigWrapper with the default name "splitpane".
     */
    public JSplitPaneConfigWrapper ()
    {
        setConfigurableName("splitpane");
    }

    /**
     * Create a new JSplitPaneConfigWrapper that will enforce the initial
     * setting of the divider location.
     *
     * @param   client  The JSplitPane to wrap.
     * @param   name    The configurable name.
     * @param   defaultLocation The default divider location for the split pane
     *                  if no state has been saved.
     */
    public JSplitPaneConfigWrapper (JSplitPane client, String name,
                                    int defaultLocation)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD,
                       "JSplitPaneConfigWrapper(JSplitPane,String,int)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "name: " + name + ", " +
                       "defaultLocation: " + defaultLocation);
        }
        setConfigurableName(name);
        targetLocation = defaultLocation;
        if (logger.level >= Log.DBG) {
            logger.log(Log.DBG,
                       "JSplitPaneConfigWrapper(JSplitPane,String,int)",
                       "Setting defaultLocation: " + defaultLocation +
                       ", myName: " + name);
        }
        client.setDividerLocation(defaultLocation);
        client.addComponentListener(new SizeListener ());
        setClient(client);
    }

    /**
     * Create a new JSplitPaneConfigWrapper that will enforce the initial
     * setting of the divider location.
     *
     * @param   client  The JSplitPane to wrap.
     * @param   name    The configurable name.
     * @param   defaultLocation The default divider location for the split pane
     *                  if no state has been saved.
     */
    public JSplitPaneConfigWrapper (JSplitPane client, String name,
                                    double defaultLocation)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD,
                       "JSplitPaneConfigWrapper(JSplitPane,String,double)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "client: " + client + ", " +
                       "name: " + name + ", " +
                       "defaultLocation: " + defaultLocation);
        }
        setConfigurableName(name);
        relTargetLocation = defaultLocation;
        if (logger.level >= Log.DBG) {
            logger.log(Log.DBG,
                       "JSplitPaneConfigWrapper(JSplitPane,String,double)",
                       "Setting defaultLocation: " + defaultLocation +
                       ", myName: " + name);
        }
        client.setDividerLocation(defaultLocation);
        client.addComponentListener(new SizeListener ());
        setClient(client);
    }

    // }}}

    // {{{ setClient

    /**
     * Set the client object for the ConfigWrapper.
     *
     * @param   client  The client object to set.
     */
    protected void setClient(Object client)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "setClient(Object)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "myName: " + getConfigurableName());
        }
        pane = (JSplitPane)client;
        pane.addPropertyChangeListener(this);
    }

    // }}}

    //----------------------------------------------------------------------
    // Configurable methods
    //----------------------------------------------------------------------
    // {{{ getConfigurableState

    /**
     * Get the Configurable's state.
     */
    public Properties getConfigurableState()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getConfigurableState()",
                       logger.level < Log.MTDDETAIL ? "" :
                       "myName: " + getConfigurableName());
        }
        Properties state = new Properties ();
        if (targetLocation < 0 && relTargetLocation >= 0) {
            state.put("divider", "" + relTargetLocation);
        } else if (targetLocation >= 0) {
            state.put("divider", "" + targetLocation);
        } else {
            state.put("divider", "" + pane.getDividerLocation());
        }
        return state;
    }

    // }}}
    // {{{ setConfigurableState

    /**
     * Set the Configurable's state.
     */
    public void setConfigurableState(Properties state)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "setConfigurableState(Properties)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "myName: " + getConfigurableName() +
                       ", state: " + state);
        }

        String pos = state.getProperty("divider");
        if (logger.level >= Log.DBG) {
            logger.log(Log.DBG, "setConfigurableState(Properties)",
                       "divider: " + pos);
        }
        if (pos == null) {
            if (logger.level >= Log.WRN) {
                logger.log(Log.WRN, "setConfigurableState(Properties)",
                           "Invalid state: " +
                           "divider: " + pos);
            }
            return;
        }
        try {
            targetLocation = Integer.parseInt(pos);
            SwingUtil.invokeAndWait(new Runnable() {
                public void run () {
                    if (logger.level >= Log.DBG) {
                        logger.log(Log.DBG, "run()",
                                   "Setting targetLocation: " +
                                   targetLocation +
                                   ", myName: " + getConfigurableName());
                    }
                    pane.setDividerLocation(targetLocation);
                }
            });
        } catch (NumberFormatException ex) {
            try {
                relTargetLocation = new Double (pos).doubleValue();
                SwingUtil.invokeAndWait(new Runnable() {
                    public void run () {
                        if (logger.level >= Log.DBG) {
                            logger.log(Log.DBG, "run()",
                                       "Setting relTargetLocation: " +
                                       relTargetLocation +
                                       ", myName: " + getConfigurableName());
                        }
                        pane.setDividerLocation(relTargetLocation);
                    }
                });
            } catch (NumberFormatException ex2) {
                logger.log(Log.WRN, "setConfigurableState(Properties)", ex2);
            }
        }
    }

    // }}}

    // {{{ registrationChanged

    /**
     * This callback is used to notify the Configurable whenever it is
     * registered with or unregistered from the Configuration, so it can
     * decide whether to notify the Configuration when its state changes.
     *
     * @param   config  The affected Configuration.
     * @param   name    The registration name of the Configurable which may
     *                  be different from the Configurable's name.
     * @param   registered True if the Configurable was just registered,
     *                  false if it was unregistered.
     * @param   valuesKept Whether the Configurable's values were kept, if
     *                  it was just unregistered.
     */
    public void registrationChanged(Configuration config, String name,
                                    boolean registered, boolean valuesKept)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD,
                       "registrationChanged(Configuration,String," +
                       "boolean,boolean)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "name: " + name + ", " +
                       "registered: " + registered);
        }
        regHelper.registrationChanged(config, registered);
    }


    // }}}

    //----------------------------------------------------------------------
    // PropertyChangeListener interface
    //----------------------------------------------------------------------
    // {{{ propertyChanged

    /**
     * Notify the JSplitPaneConfigWrapper of a change in the JSplitPane.
     *
     * @param   e       The event containing the details of the change.
     */
    public void propertyChange(PropertyChangeEvent e)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "propertyChanged(PropertyChangeEvent)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "myName: " + getConfigurableName() +
                       ", propertyName: " + e.getPropertyName());
        }
        if (e.getPropertyName().equals
            (JSplitPane.LAST_DIVIDER_LOCATION_PROPERTY )) {
            if (! pane.isShowing() &&
                (targetLocation >= 0 || relTargetLocation >= 0)) {
                if (targetLocation >= 0) {
                    if (logger.level >= Log.DBG) {
                        logger.log(Log.DBG,
                                   "propertyChange(PropertyChangeEvent)",
                                   "Setting targetLocation: " +
                                   targetLocation +
                                   ", myName: " + getConfigurableName());
                    }
                    if (override) {
                        pane.setDividerLocation(targetLocation);
                        override = false;
                    }
                } else {
                    if (logger.level >= Log.DBG) {
                        logger.log(Log.DBG,
                                   "propertyChange(PropertyChangeEvent)",
                                   "Setting relTargetLocation: " +
                                   relTargetLocation +
                                   ", myName: " + getConfigurableName());
                    }
                    if (override) {
                        pane.setDividerLocation(relTargetLocation);
                        override = false;
                    }
                }
            } else {
                targetLocation = pane.getDividerLocation();
                if (logger.level >= Log.DBG) {
                    logger.log(Log.DBG, "propertyChange(PropertyChangeEvent)",
                               "divider: " + targetLocation);
                }
                if (registered) {
                    regHelper.stateChanged(this);
                }
            }
        }
    }

    // }}}

    //----------------------------------------------------------------------
    // SizeListener
    //----------------------------------------------------------------------
    // {{{ class SizeListener

    /**
     * Listener for size events on the SplitPane.
     */
    private class SizeListener
        extends ComponentAdapter
    {
        // {{{ componentResized

        /**
         * Enforce the setting of the divider location.
         *
         * @param       e       Ignored.
         */
        public void componentResized(ComponentEvent e)
        {
            if (logger.level >= Log.MTD) {
                logger.log(Log.MTD, "componentResized(ComponentEvent)",
                           logger.level < Log.MTDDETAIL ? "" :
                           "myName: " + getConfigurableName() +
                           ", targetLocation: " + targetLocation);
            }
            if (targetLocation >= 0) {
                if (pane.getDividerLocation() != targetLocation) {
                    if (logger.level >= Log.DBG) {
                        logger.log(Log.DBG, "componentResized(ComponentEvent)",
                                   "setting targetLocation: " +
                                   targetLocation +
                                   ", myName: " + getConfigurableName());
                    }
                    pane.setDividerLocation(targetLocation);
                }
            } else {
                if (logger.level >= Log.DBG) {
                    logger.log(Log.DBG, "componentResized(ComponentEvent)",
                               "setting relTargetLocation: " +
                               relTargetLocation +
                               ", myName: " + getConfigurableName());
                }
                pane.setDividerLocation(relTargetLocation);
            }
        }

        // }}}
    }

    // }}}
}
