// {{{ copyright

/********************************************************************
 *
 * $Id: AdapterTreeModel.java,v 1.1 2001/04/20 16:07:29 gs Exp $
 *
 * Copyright (C) 2000 Gregor Schmid, Quality First Software
 * All rights reserved
 *
 *******************************************************************/

// }}}

package de.qfs.lib.gui;

// {{{ imports

import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import java.util.Vector;

import de.qfs.lib.gui.SwingUtil;
import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;
import de.qfs.lib.tree.TreeAdapter;

// }}}

/**
 * A generalized tree model that implements the Swing TreeModel interface with
 * the help of a {@link de.qfs.lib.tree.TreeAdapter TreeAdapter} and has a
 * number of extra features.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.1 $
 * @since       0.98.1
 */
public class AdapterTreeModel
    implements TreeModel
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger = new Logger (AdapterTreeModel.class);

    /**
     * The TreeAdapter for the tree.
     */
    protected TreeAdapter adapter;

    /**
     * The root of the model.
     */
    protected Object root;

    /**
     * The TreeModelListeners of the model.
     */
    protected Vector listeners = new Vector ();

    // }}}

    // {{{ constructor

    /**
     * Create a new AdapterTreeModel.
     *
     * @param   adapter The adapter for the tree hierarchy.
     * @param   root    The root node of the model.
     */
    public AdapterTreeModel (TreeAdapter adapter, Object root)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "Model(TreeAdapter,Object)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "adapter: " + adapter + ", " +
                       "root: " + root);
        }
        this.adapter = adapter;
        this.root = root;
    }

    // }}}

    // {{{ getAdapter

    /**
     * Get the TreeAdapter that implements the tree hierarchy for this model.
     *
     * @return  The TreeAdapter of the model.
     */
    public final TreeAdapter getAdapter()
    {
        return adapter;
    }


    // }}}

    //----------------------------------------------------------------------
    // The TreeModel interface
    //----------------------------------------------------------------------
    // {{{ addTreeModelListener

    /**
     * Add a listener for the TreeModelEvent posted after the tree changes.
     *
     * @param   listener        The listener to add.
     */
    public synchronized void addTreeModelListener(TreeModelListener listener)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "addTreeModelListener(TreeModelListener)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "listener: " + listener);
        }
        // avoid double registration
        listeners.removeElement(listener);
        // last added, first called
        listeners.insertElementAt(listener, 0);
    }

    // }}}
    // {{{ removeTreeModelListener

    /**
     * Remove a listener previously added with addTreeModelListener.
     *
     * @param   listener        The listener to remove.
     */
    public synchronized void
    removeTreeModelListener(TreeModelListener listener)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "removeTreeModelListener(TreeModelListener)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "listener: " + listener);
        }
        listeners.removeElement(listener);
    }

    // }}}
    // {{{ getChild

    /**
     * Return the child of parent at index index in the parent's child array.
     *
     * @param   parent  The parent whose child to get.
     * @param   index   The index to look at.
     *
     * @return  The child object.
     */
    public Object getChild(Object parent, int index)
    {
        return adapter.getChildNode(parent, index);
    }

    // }}}
    // {{{ getChildCount

    /**
     * Return the number of children of parent.
     *
     * @param   parent  The parent.
     *
     * @return  The number of children.
     */
    public int getChildCount(Object parent)
    {
        return adapter.getChildCount(parent);
    }

    // }}}
    // {{{ getIndexOfChild

    /**
     * Return the index of child in parent.
     *
     * @param   parent  The parent.
     * @param   child   The child whose index to get.
     *
     * @return  The index of the child.
     */
    public int getIndexOfChild(Object parent, Object child)
    {
        return adapter.getIndexOfChildNode(parent, child);
    }

    // }}}
    // {{{ getRoot

    /**
     * Get the root of the model.
     *
     * @return  The root of the model.
     */
    public Object getRoot()
    {
        return root;
    }

    // }}}
    // {{{ isLeaf

    /**
     * Test whether a node is a leaf node.
     *
     * @param   node    The node to test.
     *
     * @return  True, if the node is a leaf node.
     */
    public boolean isLeaf(Object node)
    {
        return adapter.isLeaf(node);
    }

    // }}}
    // {{{ valueForPathChanged

    /**
     * Notify the TreeModelListeners that a node has changed.
     *
     * @param   path    The path that has changed.
     * @param   newValue The new value of the node.
     */
    public void valueForPathChanged(final TreePath path, final Object newValue)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "valueForPathChanged(TreePath,Object)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "path: " + path + ", " +
                       "newValue: " + newValue);
        }
        final TreeModelEvent event = new TreeModelEvent (this, path);
        final Vector tmp;
        synchronized (this) {
            tmp = (Vector) listeners.clone();
        }
        SwingUtil.invokeAndWait(new Runnable () {
            public void run() {
                for (int i = 0; i < tmp.size(); i++) {
                    ((TreeModelListener) tmp.elementAt(i))
                        .treeNodesChanged(event);
                }
            }
        });
    }

    // }}}

    //----------------------------------------------------------------------
    // Additional notification methods
    //----------------------------------------------------------------------
    // {{{ nodeInserted

    /**
     * Notify the model that a node was inserted into the tree.
     *
     * @param   parentPath      The parent of the inserted node.
     * @param   index   The index at which the node was inserted.
     */
    public void nodeInserted(final Object[] parentPath, final int index)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "nodeInserted(Object[],int)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "parentPath: " + parentPath + ", " +
                       "index: " + index);
        }
        final TreeModelEvent event = new TreeModelEvent
            (this,  parentPath, new int[] {index},
            new Object[] {getChild(parentPath[parentPath.length - 1], index)});
        final Vector tmp;
        synchronized (this) {
            tmp = (Vector) listeners.clone();
        }
        SwingUtil.invokeAndWait(new Runnable () {
            public void run() {
                for (int i = 0; i < tmp.size(); i++) {
                    ((TreeModelListener) tmp.elementAt(i))
                        .treeNodesInserted(event);
                }
            }
        });
    }

    // }}}
    // {{{ nodeRemoved

    /**
     * Notify the model that a node was removed from the tree.
     *
     * @param   parentPath      The parent of the removed node.
     * @param   index   The index from which the node was removed.
     */
    public void nodeRemoved(final Object[] parentPath, final  int index)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "nodeRemoved(Object[],int)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "parentPath: " + parentPath + ", " +
                       "index: " + index);
        }
        final TreeModelEvent event = new TreeModelEvent
            (this,  parentPath, new int[] {index}, null);
        final Vector tmp;
        synchronized (this) {
            tmp = (Vector) listeners.clone();
        }
        SwingUtil.invokeAndWait(new Runnable () {
            public void run() {
                for (int i = 0; i < tmp.size(); i++) {
                    ((TreeModelListener) tmp.elementAt(i))
                        .treeNodesRemoved(event);
                }
            }
        });
    }

    // }}}
}
