// {{{ copyright

/********************************************************************
 *
 * $Id: Borders.java,v 1.10 2001/02/17 09:41:24 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.gui;

// {{{ imports

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;

import javax.swing.ImageIcon;
import javax.swing.BorderFactory;
import javax.swing.border.Border;

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

/**
 * A Collection of useful borders.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.10 $
 */
public class Borders
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private static Logger logger = new Logger (Borders.class);

    /**
     * A Border depicting a rectangular Button protruding from its setting. It
     * takes up 4 pixels on each side and is location compatible with {@link
     * #SUNKEN SUNKEN} and {@link #SLIGHTLY_SUNKEN SLIGHTLY_SUNKEN}.
     */
    public final static Border PROTRUDING = new Protruding ();

    /**
     * A Border depicting a rectangular Button slightly protruding from its
     * setting. It takes up 4 pixels on each side and is location compatible
     * with {@link #SUNKEN SUNKEN} and {@link #SLIGHTLY_SUNKEN
     * SLIGHTLY_SUNKEN}.
     */
    public final static Border SLIGHTLY_PROTRUDING = new SlightlyProtruding ();

    /**
     * A Border depicting a rectangular Button sunken into its setting. It
     * takes up 4 pixels on each side and is location compatible with {@link
     * #PROTRUDING PROTRUDING} and {@link #SLIGHTLY_PROTRUDING
     * SLIGHTLY_PROTRUDING}.
     */
    public final static Border SUNKEN = new Sunken ();

    /**
     * A Border depicting a rectangular Button slightly sunken into its
     * setting. It takes up 4 pixels on each side and is location compatible
     * with {@link #PROTRUDING PROTRUDING} and {@link #SLIGHTLY_PROTRUDING
     * SLIGHTLY_PROTRUDING}.
     */
    public final static Border SLIGHTLY_SUNKEN = new SlightlySunken ();

    /**
     * A slightly lowered border that is less conspicuous than Swing's
     * lowered bevel border. It takes up 2 pixels on each side and is location
     * compatible with Swing's raised, lowered and etched borders.
     *
     * @since   0.98.0
     */
    public final static Border SLIGHTLY_LOWERED = new SlightlyLowered ();

    /**
     * The icon for the stipple border.
     */
    private final static ImageIcon stippleIcon =  new ImageIcon
        (Borders.class.getResource
         ("/de/qfs/lib/resources/icons/stipple.gif"));

    /**
     * A stipple border similar to the one Windows uses for indicating the
     * focus owner.
     */
    public final static Border STIPPLE =
        BorderFactory.createMatteBorder(1, 1, 1, 1, stippleIcon);

    // }}}

    // {{{ class Protruding

    /**
     * A Border depicting a rectangular Button protruding from its setting.
     */
    private static class Protruding
        implements Border
    {
        /**
         * The border's insets.
         */
        private final static Insets insets = new Insets (4, 4, 4, 4);

        /**
         * Get the border's insets for a component.
         *
         * @param       c       The component to get the insets for.
         *
         * @return      This border takes up 4 pixels on each side.
         */
        public Insets getBorderInsets(Component c)
        {
            return insets;
        }

        /**
         * Query whether the border is opaque.
         *
         * @return      False.
         */
        public boolean isBorderOpaque()
        {
            return false;
        }

        /**
         * Paint the border.
         *
         * @param       c       The component to paint the border for.
         * @param       g       The Graphics to paint on.
         * @param       x       The X coordinate of the border.
         * @param       y       The Y coordinate of the border.
         * @param       w       The width of the border.
         * @param       h       The height of the border.
         */
        public void paintBorder(Component c, Graphics g,
                                int x, int y, int w, int h)
        {
            Color medium = c.getBackground().darker();

            g.setColor(Color.white);
            g.drawLine(x+2, y+2, x+w-4, y+2);
            g.drawLine(x+2, y+3, x+2, y+h-4);
            g.drawLine(x+w-2, y+1, x+w-2, y+4);
            g.drawLine(x+1, y+h-2, x+4, y+h-2);

            g.setColor(Color.black);
            g.drawLine(x+1, y+1, x+w-3, y+1);
            g.drawLine(x+1, y+2, x+1, y+h-3);
            g.drawLine(x+w-3, y+2, x+w-3, y+h-3);
            g.drawLine(x+2, y+h-3, x+w-4, y+h-3);

            g.setColor(medium);
            g.drawLine(x, y, x+w-3, y);
            g.drawLine(x, y+1, x, y+h-3);
            g.drawLine(x+w-2, y+5, x+w-2, y+h-1);
            g.drawLine(x+w-1, y+6, x+w-1, y+h-1);
            g.drawLine(x+5, y+h-2, x+w-3, y+h-2);
            g.drawLine(x+6, y+h-1, x+w-3, y+h-1);
            g.drawLine(x+w-4, y+3, x+w-4, y+h-4);
            g.drawLine(x+3, y+h-4, x+w-5, y+h-4);
        }
    }

    // }}}
    // {{{ class SlightlyProtruding

    /**
     * A Border depicting a rectangular Button slightly protruding from its
     * setting.
     */
    private static class SlightlyProtruding
        implements Border
    {
        /**
         * The border's insets.
         */
        private final static Insets insets = new Insets (4, 4, 4, 4);

        /**
         * Get the border's insets for a component.
         *
         * @param       c       The component to get the insets for.
         *
         * @return      This border takes up 4 pixels on each side.
         */
        public Insets getBorderInsets(Component c)
        {
            return insets;
        }

        /**
         * Query whether the border is opaque.
         *
         * @return      False.
         */
        public boolean isBorderOpaque()
        {
            return false;
        }

        /**
         * Paint the border.
         *
         * @param       c       The component to paint the border for.
         * @param       g       The Graphics to paint on.
         * @param       x       The X coordinate of the border.
         * @param       y       The Y coordinate of the border.
         * @param       w       The width of the border.
         * @param       h       The height of the border.
         */
        public void paintBorder(Component c, Graphics g,
                                int x, int y, int w, int h)
        {
            Color medium = c.getBackground().darker();

            g.setColor(Color.white);
            g.drawLine(x+2, y+2, x+w-4, y+2);
            g.drawLine(x+2, y+3, x+2, y+h-4);
            g.drawLine(x+w-2, y+1, x+w-2, y+4);
            g.drawLine(x+1, y+h-2, x+4, y+h-2);

            g.setColor(Color.black);
            g.drawLine(x+1, y+1, x+w-3, y+1);
            g.drawLine(x+1, y+2, x+1, y+h-3);
            g.drawLine(x+w-3, y+2, x+w-3, y+h-3);
            g.drawLine(x+2, y+h-3, x+w-4, y+h-3);

            g.setColor(medium);
            g.drawLine(x, y, x+w-3, y);
            g.drawLine(x, y+1, x, y+h-3);
            g.drawLine(x+w-2, y+5, x+w-2, y+h-2);
            g.drawLine(x+5, y+h-2, x+w-3, y+h-2);
            g.drawLine(x+w-4, y+3, x+w-4, y+h-4);
            g.drawLine(x+3, y+h-4, x+w-5, y+h-4);
        }
    }

    // }}}
    // {{{ class Sunken

    /**
     * A Border depicting a rectangular Button sunken into its setting.
     */
    private static class Sunken
        implements Border
    {
        /**
         * The border's insets.
         */
        private final static Insets insets = new Insets (4, 4, 4, 4);

        /**
         * Get the border's insets for a component.
         *
         * @param       c       The component to get the insets for.
         *
         * @return      This border takes up 4 pixels on each side.
         */
        public Insets getBorderInsets(Component c)
        {
            return insets;
        }

        /**
         * Query whether the border is opaque.
         *
         * @return      False.
         */
        public boolean isBorderOpaque()
        {
            return false;
        }

        /**
         * Paint the border.
         *
         * @param       c       The component to paint the border for.
         * @param       g       The Graphics to paint on.
         * @param       x       The X coordinate of the border.
         * @param       y       The Y coordinate of the border.
         * @param       w       The width of the border.
         * @param       h       The height of the border.
         */
        public void paintBorder(Component c, Graphics g,
                                int x, int y, int w, int h)
        {
            Color medium = c.getBackground().darker();

            g.setColor(Color.white);
            g.drawLine(x+w-2, y+1, x+w-2, y+h-2);
            g.drawLine(x+1, y+h-2, x+w-3, y+h-2);

            g.setColor(Color.black);
            g.drawLine(x+1, y+1, x+w-3, y+1);
            g.drawLine(x+1, y+2, x+1, y+h-3);
            g.drawLine(x+w-3, y+2, x+w-3, y+h-3);
            g.drawLine(x+2, y+h-3, x+w-4, y+h-3);

            g.setColor(medium);
            g.drawLine(x, y, x+w-3, y);
            g.drawLine(x, y+1, x, y+h-3);
            g.drawLine(x+2, y+2, x+w-4, y+2);
            g.drawLine(x+2, y+3, x+w-4, y+3);
            g.drawLine(x+2, y+4, x+2, y+h-4);
            g.drawLine(x+3, y+4, x+3, y+h-4);
            g.drawLine(x+w-4, y+4, x+w-4, y+h-4);
            g.drawLine(x+4, y+h-4, x+w-5, y+h-4);
        }
    }

    // }}}
    // {{{ class SlightlySunken

    /**
     * A Border depicting a rectangular Button slightly sunken into its
     * setting.
     */
    private static class SlightlySunken
        implements Border
    {
        /**
         * The border's insets.
         */
        private final static Insets insets = new Insets (4, 4, 4, 4);

        /**
         * Get the border's insets for a component.
         *
         * @param       c       The component to get the insets for.
         *
         * @return      This border takes up 4 pixels on each side.
         */
        public Insets getBorderInsets(Component c)
        {
            return insets;
        }

        /**
         * Query whether the border is opaque.
         *
         * @return      False.
         */
        public boolean isBorderOpaque()
        {
            return false;
        }

        /**
         * Paint the border.
         *
         * @param       c       The component to paint the border for.
         * @param       g       The Graphics to paint on.
         * @param       x       The X coordinate of the border.
         * @param       y       The Y coordinate of the border.
         * @param       w       The width of the border.
         * @param       h       The height of the border.
         */
        public void paintBorder(Component c, Graphics g,
                                int x, int y, int w, int h)
        {
            Color medium = c.getBackground().darker();

            g.setColor(Color.white);
            g.drawLine(x+w-2, y+1, x+w-2, y+h-2);
            g.drawLine(x+1, y+h-2, x+w-3, y+h-2);

            g.setColor(Color.black);
            g.drawLine(x+1, y+1, x+w-3, y+1);
            g.drawLine(x+1, y+2, x+1, y+h-3);
            g.drawLine(x+w-3, y+2, x+w-3, y+h-3);
            g.drawLine(x+2, y+h-3, x+w-4, y+h-3);

            g.setColor(medium);
            g.drawLine(x, y, x+w-3, y);
            g.drawLine(x, y+1, x, y+h-3);
            g.drawLine(x+2, y+2, x+w-4, y+2);
            g.drawLine(x+2, y+4, x+2, y+h-4);
            g.drawLine(x+w-4, y+4, x+w-4, y+h-4);
            g.drawLine(x+4, y+h-4, x+w-5, y+h-4);
        }
    }

    // }}}
    // {{{ class SlightlyLowered

    /**
     * A Border depicting a rectangular Button slightly lowered into its
     * setting. Layout compatible with Swing's etched, lowered and raised
     * borders.
     *
     * @since   0.98.0
     */
    private static class SlightlyLowered
        implements Border
    {
        /**
         * The border's insets.
         */
        private final static Insets insets = new Insets (2, 2, 2, 2);

        /**
         * Get the border's insets for a component.
         *
         * @param       c       The component to get the insets for.
         *
         * @return      This border takes up 4 pixels on each side.
         */
        public Insets getBorderInsets(Component c)
        {
            return insets;
        }

        /**
         * Query whether the border is opaque.
         *
         * @return      False.
         */
        public boolean isBorderOpaque()
        {
            return false;
        }

        /**
         * Paint the border.
         *
         * @param       c       The component to paint the border for.
         * @param       g       The Graphics to paint on.
         * @param       x       The X coordinate of the border.
         * @param       y       The Y coordinate of the border.
         * @param       w       The width of the border.
         * @param       h       The height of the border.
         */
        public void paintBorder(Component c, Graphics g,
                                int x, int y, int w, int h)
        {
            Color dark = c.getBackground().darker().darker();

            g.setColor(dark);
            g.drawLine(x, y, x+w-1, y);
            g.drawLine(x, y, x, y+h-1);

            g.setColor(Color.white);
            g.drawLine(x+w-1, y+1, x+w-1, y+h-1);
            g.drawLine(x+1, y+h-1, x+w-1, y+h-1);
        }
    }

    // }}}
}
