// {{{ copyright

/********************************************************************
 *
 * $Id: DelayedListSelectionListener.java,v 1.7 2000/06/06 17:00:02 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.gui;

// {{{ imports

import java.awt.event.ActionListener;

import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

import de.qfs.lib.util.DelayedAction;

// }}}

/**
 * This class implements a delayed version of the ListSelectionListener
 * interface with the help of a {@link de.qfs.lib.util.DelayedAction
 * DelayedAction}. <p>
 *
 * A delayed reaction to selection events is useful, when the selection
 * causes a complicated display to be updated. Without the delay, keyboard
 * traversal through the list will cause a redisplay for every item passed. By
 * using a DelayedListSelectionListener, only the last selection will actually
 * cause a redisplay, if the delay is set correctly. <p>
 *
 * When the delayed action is actually performed, the {@link
 * #delayedValueChanged delayedValueChanged} method will be called. This
 * abstract method must be implemented by a subclass.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.7 $
 */
public abstract class DelayedListSelectionListener
    implements ListSelectionListener
{
    // {{{ variables

    /**
     * The default delay for a DelayedListSelectionListener.
     */
    public final static int DEFAULT_DELAY = 300;

    /**
     * The delayed action.
     */
    private DelayedAction delayedAction;

    // }}}

    // {{{ constructors

    /**
     * Create a new DelayedListSelectionListener with the default delay
     * of 300 milliseconds.
     */
    public DelayedListSelectionListener ()
    {
        this (DEFAULT_DELAY);
    }

    /**
     * Create a new DelayedListSelectionListener.
     *
     * @param   delay   The delay in milliseconds until a
     *          delayedValueChanged is fired
     */
    public DelayedListSelectionListener (int delay)
    {
        delayedAction = new DelayedAction (delay);
    }

    // }}}

    // {{{ valueChanged

    /**
     * Restart the timer for the delayed selection changed.
     *
     * @param   event   The event that caused the change.
     */
    public void valueChanged (ListSelectionEvent event)
    {
        delayedAction.perform(new RunHelper (event));
    }

    // }}}
    // {{{ delayedValueChanged

    /**
     * Handle a ListSelectionEvent similar to {@link
     * ListSelectionListener#valueChanged ListSelectionListener.valueChanged}.
     *
     * @param event     The event that caused the change.
     */
    protected abstract void delayedValueChanged (ListSelectionEvent event);

    // }}}

    // {{{ class RunHelper

    /**
     * Helper class for the delayed Action.
     */
    private class RunHelper
        implements Runnable
    {
        /**
         * The event that caused the delayed action to fire.
         */
        private ListSelectionEvent event;

        // {{{ constructor

        /**
         * Create a new RunHelper.
         *
         * @param       event   The event that caused the delayed action to
         *                      fire.
         */
        public RunHelper(ListSelectionEvent event)
        {
            this.event = event;
        }

        // }}}

        // {{{ run

        /**
         * Execute the delayedValueChanged method.
         */
        public void run()
        {
            delayedValueChanged(event);
        }

        // }}}
    }

    // }}}
}
