// {{{ copyright

/********************************************************************
 *
 * $Id: EventQueue.jpp,v 1.16 2001/04/25 07:26:05 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.gui;

// {{{ imports

import java.awt.AWTEvent;
import java.awt.Component;
import java.awt.MenuComponent;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;

import java.awt.ActiveEvent;

import java.awt.event.KeyEvent;

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;
import de.qfs.lib.util.ExceptionHandler;

// }}}

// {{{ doc

/**
 * This EventQueue enables exception handling for exceptions raised in the AWT
 * event loop thread. Since version 0.98.0 this works even with JDK 1.1. <p>
 *
 * Additionally, a workaround is provided for a problem with some JDKs on
 * Motif systems (linux for example), where no KEY_PRESSED and KEY_TYPED
 * events are generated for the Shift_Tab combination under certain
 * circumstances. This hack generates the neccessary events so tab traversal
 * works both forwards and backwards. <p>
 *
 * New in qflib 0.98.0 is another hack needed for some Motif systems like the
 * IBM JDKs on linux: They generate Alt-Key events with the modifier bits for
 * ALT and META set, which causes keyboard access of menus to fail (among
 * other Alt-... bindings). If you enable the meta hack, the META modifier
 * will be removed from these key events. <p>
 *
 * Yet another hack is needed for the blackdown JDK 1.3 on linux: It generates
 * a KEY_TYPED event for the delete key, causing weird symbols to appear in
 * text components. These events are filtered if the delete hack is
 * enabled. <p>
 *
 * Different ways are used to install an EventQueue as the system event queue
 * for JDKs 1.1 and 1.2. With JDK 1.1, make sure that the class is in your
 * CLASSPATH and add the line <br>
 * <code>AWT.EventQueueClass=de.qfs.lib.gui.EventQueue</code> <br> to the file
 * <code>awt.properties</code> in the lib directory of your JDK. Make sure you
 * don't put any extra SPACE characters on the line, as some JDKs choke on
 * them. In any case you must call {@link #install install}. <p>
 *
 * <strong>Warning:</strong> If you use this EventQueue with JDK 1.1 and
 * {@link #install install} it, {@link java.awt.EventQueue#getNextEvent
 * EventQueue.getNextEvent} and {@link java.awt.EventQueue#peekEvent()
 * EventQueue.peekEvent()} will return {@link WrapperEvent WrapperEvents}
 * instead of the normal events and {@link java.awt.EventQueue#peekEvent(int)
 * EventQueue.peekEvent(int)} will cease to work. If you want to use these
 * methods, either check for WrapperEvents and call {@link
 * WrapperEvent#getWrappedEvent WrapperEvent.getWrappedEvent} to access the
 * original event (but dispatch the event through the WrapperEvent's {@link
 * WrapperEvent#dispatch dispatch} method), or temporarily disable the
 * EventQueue with {@link #uninstall uninstall}. <p>
 *
 * <strong>Warning:</strong> If you use this EventQueue with JDK 1.2 or above,
 * you must use the EventQueue returned by the {@link #instance instance}
 * method instead of {@link java.awt.Toolkit#getSystemEventQueue
 * Toolkit.getSystemEventQueue} to poll events with {@link
 * java.awt.EventQueue#getNextEvent EventQueue.getNextEvent} etc. You can
 * {@link java.awt.EventQueue#postEvent EventQueue.postEvent} to the system
 * event queue as usual.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.16 $
 */

// }}}
public class EventQueue extends java.awt.EventQueue
{
    // {{{ variables

    /**
     * Id for WrapperEvents.
     */
    public final static int WRAPPER_ID = AWTEvent.RESERVED_ID_MAX + 10770;

    /**
     * The Logger used for logging.
     */
    private static Logger logger = new Logger (EventQueue.class);

    /**
     * The Logger used for logging.
     */
    private static Logger weLogger = new Logger (WrapperEvent.class);

    /**
     * The installed Queue or null.
     */
    protected static EventQueue theQueue;

    /**
     * The ExceptionHandler for Events dispatched through the Queue.
     */
    private static ExceptionHandler handler;

    /**
     * Whether the queue is currently installed.
     */
    protected boolean installed;

    /**
     * Flag for the tab hack.
     */
    private boolean tabPressed;

    /**
     * The keyCode of the last KEY_PRESSED event.
     */
    private int lastCodePressed;

    /**
     * Whether the inital settings for the hacks has already been guessed.
     */
    private static boolean hacksGuessed = false;

    /**
     * Whether the tab hack is enabled.
     */
    private static boolean tabHackEnabled = true;

    /**
     * Whether the meta hack is enabled.
     */
    private static boolean metaHackEnabled = true;

    /**
     * Whether the delete hack is enabled.
     */
    private static boolean deleteHackEnabled = true;

    // }}}

    // {{{ constructor

    /**
     * Create a new EventQueue. This constructor must be public so the
     * AWT Toolkit can create an EventQueue. It should not be called directly.
     */
    public EventQueue()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "EventQueue()", "");
        }
    }

    // }}}

    // {{{ instance

    /**
     * Get the EventQueue instance, if it has been installed. Otherwise
     * get the SystemEventQueue.
     *
     * @return  The EventQueue instance.
     */
    public static java.awt.EventQueue instance()
    {
        if (theQueue != null) {
            return theQueue;
        }
        return Toolkit.getDefaultToolkit().getSystemEventQueue();
    }

    // }}}
    // {{{ install

    /**
     * Install the EventQueue instance as the System EventQueue. With
     * JDK 1.1 additional support is needed (see class doc).
     *
     * @return  True, if the EventQueue has been installed.
     */
    public static boolean install()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "install()", "");
        }

        if (! hacksGuessed) {
            // This should be done only once and cannot be placed in the
            // constructor or a static initializer, since accessing the
            // Toolkit can lead to recursion.
            deleteHackEnabled =
                (System.getProperty("java.vendor").indexOf("Blackdown") >= 0
                 || System.getProperty("java.vendor").indexOf("Sun") >= 0)
                && System.getProperty("java.version").startsWith("1.3.");

            tabHackEnabled = Toolkit.getDefaultToolkit().getClass().getName()
                .indexOf("motif") >= 0;

            metaHackEnabled =
                (System.getProperty("java.vendor").indexOf("IBM") >= 0
                 || System.getProperty("java.vendor")
                    .indexOf("Hewlett Packard") >= 0)
                &&  Toolkit.getDefaultToolkit().getClass().getName()
                    .indexOf("motif") >= 0;

            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG, "EventQueue()",
                           "deleteHackEnabled: " + deleteHackEnabled +
                           ", tabHackEnabled: " + tabHackEnabled +
                           ", metaHackEnabled: " + metaHackEnabled);
            }
            hacksGuessed = true;
        }

        if (theQueue != null) {
            if (! theQueue.installed) {
                theQueue.installed = true;

                Toolkit.getDefaultToolkit().getSystemEventQueue()
                    .push(theQueue);

            }
            return true;
        }
        java.awt.EventQueue original =
            Toolkit.getDefaultToolkit().getSystemEventQueue();
        // Has the EventQueue been installed automatically ?
        if (original instanceof EventQueue) {
            theQueue = (EventQueue) original;
            theQueue.installed = true;
            return true;
        }

        theQueue = new EventQueue ();
        original.push(theQueue);
        theQueue.installed = true;
        return true;

    }

    // }}}
    // {{{ uninstall

    /**
     * Uninstall the EventQueue. With JDK 1.2 the EventQueue is truly
     * uninstalled. With JDK 1.1 this is not possible, so the EventQueue acts
     * as if it weren't there.
     */
    public static void uninstall()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "uninstall()", "");
        }
        if (theQueue != null) {
            theQueue.installed = false;

            theQueue.pop();
            theQueue = null;

        }
    }

    // }}}
    // {{{ isInstalled

    /**
     * Check, whether the EventQueue has been installed.
     *
     * @return  True, if the EventQueue is installed.
     */
    public static boolean isInstalled()
    {
        return theQueue != null && theQueue.installed;
    }

    // }}}

    // {{{ dispatchEvent

    /**
     * Dispatch an event. This is the method of choice to access events on the
     * event loop for JDK 1.2. This method is called from inside the event
     * loop thread, so exception ahndling is possible. <p>
     *
     * This method is public for the sake of {@link SwingUtil#withEvents
     * SwingUtil.withEvents}. Don't use it unless you really know what you are
     * doing.
     *
     * @param   event   The event to dispatch.
     */
    public void dispatchEvent (AWTEvent event)
    {
        // better don't log this one, too many events
        // if (logger.level >= Log.DBG) {
            // logger.log(Log.DBG, "dispatchEvent(AWTEvent)",
                       // logger.level < Log.DBGDETAIL ? "" :
                       // "event: " + event);
        // }
        if (installed && tabHackEnabled && tabHack(event)) {
            return;
        }
        if (installed && metaHackEnabled
            && event.getID() >= KeyEvent.KEY_FIRST
            && event.getID() <= KeyEvent.KEY_LAST
            && (((KeyEvent) event).getModifiers() &
                (KeyEvent.ALT_MASK | KeyEvent.META_MASK))
               == (KeyEvent.ALT_MASK | KeyEvent.META_MASK)) {
            // deprecated, but we need it
            KeyEvent ke = (KeyEvent) event;
            event = new KeyEvent ((Component) ke.getSource(), ke.getID(),
                                  ke.getWhen(),
                                  (ke.getModifiers() & ~KeyEvent.META_MASK),
                                  ke.getKeyCode(), ke.getKeyChar());
        }
        if (installed && deleteHackEnabled) {
            if (event.getID() == KeyEvent.KEY_PRESSED) {
                lastCodePressed = ((KeyEvent) event).getKeyCode();
            } else if (event.getID() == KeyEvent.KEY_TYPED
                       && lastCodePressed == KeyEvent.VK_DELETE
                       && ((KeyEvent) event).getKeyChar() == 255 /* Delete */) {
                return;
            }
        }

        if (installed && handler != null) {
            try {
                doDispatch(event);
            } catch (Throwable ex) {
                if (! handler.handleException(ex)) {
                    if (ex instanceof RuntimeException) {
                        throw ((RuntimeException) ex);
                    } else {
                        throw ((Error) ex);
                    }
                }
            }
        } else {
            doDispatch(event);
        }
    }

    // }}}
    // {{{ doDispatch

    /**
     * Really dispatch an event.
     *
     * @param   event   The event to dispatch.
     */
    private void doDispatch(AWTEvent event)
    {
        // better don't log this one, too many events
        // if (logger.level >= Log.MTD) {
            // logger.log(Log.MTD, "doDispatch(AWTEvent)",
                       // logger.level < Log.MTDDETAIL ? "" :
                       // "event.id: " + event.getID());
        // }

        super.dispatchEvent(event);

    }

    // }}}

    // {{{ pop

    /**
     * Public version of pop.
     */
    public void pop()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "pop()", "");
        }
        super.pop();
    }

    // }}}

    // {{{ setExceptionHandler

    /**
     * Set a handler for Exceptions that occur in the AWT event dispatch
     * thread. Only works with JDK 1.2 and only if the EventQueue is
     * installed.
     *
     * @param   handler The ExceptionHandler to set.
     */
    public static void setExceptionHandler (ExceptionHandler handler)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "setExceptionHandler(ExceptionHandler)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "handler: " + handler);
        }
        EventQueue.handler = handler;
    }

    // }}}

    // {{{ isTabHackEnabled

    /**
     * Query whether the tab hack is enabled.
     *
     * @return  True if it is enabled.
     */
    public final static boolean isTabHackEnabled()
    {
        return tabHackEnabled;
    }

    // }}}
    // {{{ setTabHackEnabled

    /**
     * Set whether the tab hack should be enabled.
     *
     * @param   enabled True to enable the hack.
     */
    public final static void setTabHackEnabled(boolean enabled)
    {
        tabHackEnabled = enabled;
    }


    // }}}
    // {{{ isMetaHackEnabled

    /**
     * Query whether the meta hack is enabled.
     *
     * @return  True if it is enabled.
     * @since   0.98.0
     */
    public final static boolean isMetaHackEnabled()
    {
        return metaHackEnabled;
    }

    // }}}
    // {{{ setMetaHackEnabled

    /**
     * Set whether the meta hack should be enabled.
     *
     * @param   enabled True to enable the hack.
     * @since   0.98.0
     */
    public final static void setMetaHackEnabled(boolean enabled)
    {
        metaHackEnabled = enabled;
    }


    // }}}
    // {{{ isDeleteHackEnabled

    /**
     * Query whether the delete hack is enabled.
     *
     * @return  True if it is enabled.
     * @since   0.98.0
     */
    public final static boolean isDeleteHackEnabled()
    {
        return deleteHackEnabled;
    }

    // }}}
    // {{{ setDeleteHackEnabled

    /**
     * Set whether the delete hack should be enabled.
     *
     * @param   enabled True to enable the hack.
     * @since   0.98.0
     */
    public final static void setDeleteHackEnabled(boolean enabled)
    {
        deleteHackEnabled = enabled;
    }


    // }}}
    // {{{ tabHack

    /**
     * At least on Motif systems (linux for example), no KEY_PRESSED and
     * KEY_TYPED events are generated for the Shift_Tab combination, but only
     * under certain circumstances. This hack generates the neccessary events
     * when it comes across a KEY_RELEASED event without a preceding
     * KEY_PRESSED event, so tab traversal works both ways again.
     *
     * @param   e       The event to check.
     *
     * @return  True, if the hack was executed. In this case the original event
     *          should be ignored, since it will be reposted after the
     *          KEY_PRESSED and KEY_TYPED events.
     */
    protected boolean tabHack(AWTEvent e)
    {
        if (e.getID() >= KeyEvent.KEY_FIRST
            && e.getID() <= KeyEvent.KEY_LAST) {
            KeyEvent event = (KeyEvent) e;
            if (event.getKeyCode() == event.VK_TAB) {
                if (event.getID() == event.KEY_PRESSED
                    && (event.getModifiers() & event.SHIFT_MASK)
                       == event.SHIFT_MASK) {
                    tabPressed = true;
                } else if (event.getID() == event.KEY_RELEASED
                           && (event.getModifiers() & event.SHIFT_MASK)
                              == event.SHIFT_MASK) {
                    if (tabPressed) {
                        tabPressed = false;
                    } else {
                        postEvent(new KeyEvent ((Component)
                                                event.getSource(),
                                                event.KEY_PRESSED,
                                                event.getWhen(),
                                                event.getModifiers(),
                                                event.getKeyCode(),
                                                event.getKeyChar()));
                        postEvent(new KeyEvent ((Component)
                                                event.getSource(),
                                                event.KEY_TYPED,
                                                event.getWhen(),
                                                event.getModifiers(),
                                                0, (char) 9));
                        postEvent(e);
                        return true;
                    }
                }
            }
        }
        return false;
    }

    // }}}

    // {{{ class WrapperEvent

    /**
     * ActiveEvent that wraps itself around an AWTEvent and dispatches it in
     * JDK 1.2 style through the EventQueue's {@link #dispatchEvent
     * dispatchEvent} method. This way it is possible to get inside the AWT
     * event dispatch thread even with JDK 1.1.
     * @since   0.98.0
     */
    public class WrapperEvent
        extends AWTEvent
        implements ActiveEvent
    {
        // {{{ variables

        /**
         * The wrapped event.
         */
        AWTEvent event;

        // }}}

        //------------------------------------------------------------------
        // Constructor
        //------------------------------------------------------------------
        // {{{ WrapperEvent

        /**
         * Create a new WrapperEvent.
         *
         * @param       event   The event to wrap.
         */
        public WrapperEvent (AWTEvent event)
        {
            super(event.getSource(), WRAPPER_ID);
            // better don't log this one, too many events
            // if (weLogger.level >= Log.MTD) {
                // weLogger.log(Log.MTD, "WrapperEvent(AWTEvent)",
                             // weLogger.level < Log.MTDDETAIL ? "" :
                             // "event.id: " + event.getID());
            // }
            this.event = event;
        }

        // }}}

        // {{{ getWrappedEvent

        /**
         * Get the wrapped event.
         *
         * @return      The wrapped event.
         */
        public final AWTEvent getWrappedEvent()
        {
            return event;
        }

        // }}}

        //------------------------------------------------------------------
        // The ActiveEvent interface
        //------------------------------------------------------------------
        // {{{ dispatch

        /**
         * Dispatch the wrapped event through the EventQueue's {@link
         * #dispatchEvent dispatchEvent} method.
         */
        public void dispatch()
        {
            // better don't log this one, too many events
            // if (weLogger.level >= Log.MTD) {
                // weLogger.log(Log.MTD, "dispatch()",
                             // weLogger.level < Log.MTDDETAIL ? "" :
                             // "event.id: " + event.getID());
            // }
            dispatchEvent(event);
        }

        // }}}
    }

    // }}}
}
