// {{{ copyright

/********************************************************************
 *
 * $Id: LevelTreeSelectionModel.java,v 1.7 2000/09/29 18:05:41 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.gui;

// {{{ imports

import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.TreePath;

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

/**
 * This class enables the selection of multiple nodes of a JTree, as long as
 * the nodes are siblings.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.7 $
 */
public class LevelTreeSelectionModel extends DefaultTreeSelectionModel
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger =
        new Logger (LevelTreeSelectionModel.class);

    // }}}

    // {{{ constructor

    /**
     * Create a new LevelTreeSelectionModel.
     */
    public LevelTreeSelectionModel ()
    {
    }


    // }}}

    // {{{ canPathsBeAdded

    /**
     * Decide wheter some paths can be added to the selection.
     *
     * @param   paths   The paths to add.
     *
     * @return  True if the paths can be added.
     */
    protected boolean canPathsBeAdded (TreePath[] paths)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "canPathsBeAdded(TreePath[])",
                       logger.level < Log.MTDDETAIL ? "" :
                       "paths: " + paths);
        }
        if (paths.length == 0) {
            return true;
        }
        TreePath cmp = getLeadSelectionPath();
        if (logger.level >= Log.DBG) {
            logger.log(Log.DBG, "canPathsBeAdded(TreePath[])",
                       "lead selection: " + cmp);
            for (int i = 0; i < paths.length; i++) {
                logger.log(Log.DBG, "canPathsBeAdded(TreePath[])",
                           "path[" + i + "]: " + paths[i]);
            }

        }

        int start = 0;
        if (cmp == null) {
            cmp = paths[0];
            start = 1;
        }

        TreePath ppath = cmp.getParentPath();
        Object parent = ppath == null ? null : ppath.getLastPathComponent();
        for (int i = start; i < paths.length; i++) {
            ppath = paths[i].getParentPath();
            if ((ppath == null ? null : ppath.getLastPathComponent())
                 != parent) {
                return false;
            }
        }
        return true;
    }

    // }}}
    // {{{ canPathsBeRemoved

    /**
     * Paths can always be removed.
     *
     * @param   paths   The paths to remove.
     *
     * @return  Always true.
     */
    protected boolean canPathsBeRemoved (TreePath[] paths)
    {
        return true;
    }

    // }}}
    // {{{ addSelectionPath

    /**
     * Add a path to the selection.
     *
     * @param   path    The path to add.
     */
    public void addSelectionPath (TreePath path)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "addSelectionPath(TreePath)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "path: " + path);
        }
        if (canPathsBeAdded (new TreePath[] {path})) {
            super.addSelectionPath(path);
        } else {
            super.setSelectionPath(path);
        }
    }

    // }}}
    // {{{ addSelectionPaths

    /**
     * Add some paths to the selection.
     *
     * @param   paths   The paths to add.
     */
    public void addSelectionPaths (TreePath[] paths)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "addSelectionPaths(TreePath[])",
                       logger.level < Log.MTDDETAIL ? "" :
                       "paths: " + paths);
        }
        if (canPathsBeAdded (paths)) {
            super.addSelectionPaths(paths);
        } else if (paths.length == 1) {
            super.setSelectionPaths(paths);
        }
    }

    // }}}
    // {{{ setSelectionPaths

    /**
     * Set the paths of the selection.
     *
     * @param   paths   The paths to set.
     */
    public void setSelectionPaths (TreePath[] paths)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "setSelectionPaths(TreePath[])",
                       logger.level < Log.MTDDETAIL ? "" :
                       "paths: " + paths);
        }
        if (canPathsBeAdded (paths)) {
            super.setSelectionPaths(paths);
        } else if (paths.length == 1) {
            super.setSelectionPaths(paths);
        }
    }

    // }}}
}
