// {{{ copyright

/********************************************************************
 *
 * $Id: ModalDialog.jpp,v 1.12 2001/04/21 19:55:01 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.gui;

// {{{ imports

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import java.util.StringTokenizer;

import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.FocusManager;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import de.qfs.lib.config.Configuration;
import de.qfs.lib.config.ConfigWrapper;
import de.qfs.lib.gui.Grid;
import de.qfs.lib.gui.SwingUtil;
import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;
import de.qfs.lib.util.MapResourceBundle;

// }}}

// {{{ doc

/**
 * This is a very useful base class for modal dialogs that display one large
 * component - or several components grouped into a large one - above a row of
 * buttons. ModalDialog reads the number and labels of the buttons as well as
 * some of their characteristics from a set of resources, simplifying
 * localization together with layout, default actions, and return value
 * definition. <p>
 *
 * The following resources are used by default. These may be extended or
 * ignored by derived classes. <p>
 *
 * <TABLE BORDER="1" CELLPADDING="3" CELLSPACING="0" WIDTH="100%">
 * <TR>
 * <TH WIDTH="40%">Resource name</TH>
 * <TH>Usage</TH>
 * </TR>
 * <TR>
 * <TH WIDTH="100%" COLSPAN="2">Dialog resources</TH>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>dialog.<em>name</em>.title</code></TD>
 * <TD>The title of the dialog (optional, no default).</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>dialog.<em>name</em>.name</code></TD>
 * <TD>The name of the dialog (optional, default
 * <code>ModalDialog-<em>name</em></code>).</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>dialog.<em>name</em>.width</code></TD>
 * <TD>The minimum width of the dialog (optional, no default).</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>dialog.<em>name</em>.height</code></TD>
 * <TD>The minimum height of the dialog (optional, no default).</TD>
 * </TR>
 * <TR>
 * <TR>
 * <TD WIDTH="40%"><code>dialog.<em>name</em>.configure</code></TD>
 * <TD> If true, create a {@link de.qfs.lib.config.ConfigWrapper
 * ConfigWrapper} for the dialog and register it under the name
 * <code><em>name</em>.dialog</code> (optional, default false).</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>dialog.<em>name</em>.buttons</code></TD>
 * <TD>The resource names of the buttons for the dialog (optional, default
 * <code>"ok cancel"</code> ).</TD>
 * </TR>
 * <TR>
 * <TH WIDTH="100%" COLSPAN="2">Button resources</TH>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>dialog.button.<em>name</em>.label</code></TD>
 * <TD>The label of the button (required).</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>dialog.button.<em>name</em>.mnemonic</code></TD>
 * <TD>Keyboard accelerator (optional, default none).</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>dialog.button.<em>name</em>.name</code></TD>
 * <TD>The name of the button set with {@link java.awt.Component#setName
 * setName} (optional, default <code>"Button-<em>name</em>"</code>).</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>dialog.button.<em>name</em>.close</code></TD>
 * <TD>Whether to close the dialog when the button is pressed (optional,
 * default true).</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>dialog.button.<em>name</em>.retval</code></TD>
 * <TD>The return value for {@link #doModal doModal} when the dialog is closed
 * via this button. Ignored unless <code>close</code> is true (optional,
 * default is the index of the button, starting from 0) .</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>dialog.button.<em>name</em>.default</code></TD>
 * <TD>Whether the button is default capable. The first button with
 * <code>default</code> set to true will be the initial default button
 * (optional, default false) .</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>dialog.button.<em>name</em>.disabled</code></TD>
 * <TD>If true, the button will be disabled (optional, default false) .</TD>
 * </TR>
 * </TABLE> <p>
 *
 * Default values for the buttons <code>ok</code>, <code>yes</code>,
 * <code>no</code> and <code>cancel</code> are defined in the resource bundle
 * <code>/de/qfs/lib/resources/properties/qflib</code> in
 * <code>qflib.jar</code>. You should include these in the resources you pass
 * to {@link #init init}.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.12 $
 * @since       0.98.0
 */

// }}}
public class ModalDialog
    extends JDialog
    implements ActionListener, WindowListener
{
    // {{{ constants

    /**
     * Retun value when the dialog is closed.
     */
    public static final int CLOSED_OPTION = -1;

    /**
     * Return value for the ok button.
     */
    public static final int OK_OPTION = 0;

    /**
     * Return value for the yes button.
     */
    public static final int YES_OPTION = 0;

    /**
     * Return value for the no button.
     */
    public static final int NO_OPTION = 1;

    /**
     * Return value for the cancel button.
     */
    public static final int CANCEL_OPTION = 2;

    /**
     * The default button names.
     */
    protected final static String[] defaultButtons = {"ok", "cancel"};

    // }}}
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private static Logger logger = new Logger (ModalDialog.class);

    /**
     * Helper to build strings.
     */
    private static StringBuffer sb = new StringBuffer ();

    /**
     * The resource name of the dialog.
     */
    protected String name;

    /**
     * The return code from the last call to {@link #doModal doModal}.
     */
    protected int retval;

    /**
     * The main component of the dialog.
     */
    protected Component mainComponent;

    /**
     * The panel for the main component.
     */
    protected JPanel mainPanel;

    /**
     * Data for the buttons of the dialog.
     */
    protected ButtonData[] buttons;

    /**
     * The layout for the dialog.
     */
    protected GridBagLayout layout;

    /**
     * The content pane of the dialog.
     */
    protected JPanel panel;

    /**
     * The minimum width of the dialog.
     */
    protected int minWidth;

    /**
     * The minimum height of the dialog.
     */
    protected int minHeight;

    // }}}

    //----------------------------------------------------------------------
    // Constructors
    //----------------------------------------------------------------------
    // {{{ ModalDialog(String,Frame)

    /**
     * Create a new ModalDialog. Don't forget to call {@link #init init}
     * afterwards.
     *
     * @param   name    The resource name of the dialog.
     * @param   parent  The dialog's owner.
     */
    public ModalDialog (String name, Frame parent)
    {
        super(parent);
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "ModalDialog(String,Frame)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "parent: " + parent + ", " +
                       "name: " + name);
        }
        this.name = name;
        setModal(true);
    }

    // }}}
    // {{{ ModalDialog(String,Frame,Component)

    /**
     * Create a new ModalDialog. Don't forget to call {@link #init init}
     * afterwards.
     *
     * @param   name    The resource name of the dialog.
     * @param   parent  The dialog's owner.
     * @param   com     The main component for the dialog.
     */
    public ModalDialog (String name, Frame parent, Component com)
    {
        super(parent);
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "ModalDialog(String,Frame,Component)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "parent: " + parent + ", " +
                       "name: " + name + ", " +
                       "com: " + com);
        }
        this.name = name;
        mainComponent = com;
        setModal(true);
    }

    // }}}

    // {{{ ModalDialog(String,Dialog)

    /**
     * Create a new ModalDialog. This constructor is only available for
     * JDK 1.2 and above. Don't forget to call {@link #init init} afterwards.
     *
     * @param   name    The resource name of the dialog.
     * @param   parent  The dialog's owner.
     */
    public ModalDialog (String name, Dialog parent)
    {
        super(parent);
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "ModalDialog(String,Dialog)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "parent: " + parent + ", " +
                       "name: " + name);
        }
        this.name = name;
        setModal(true);
    }

    // }}}
    // {{{ ModalDialog(String,Dialog,Component)

    /**
     * Create a new ModalDialog. This constructor is only available for
     * JDK 1.2 and above. Don't forget to call {@link #init init} afterwards.
     *
     * @param   name    The resource name of the dialog.
     * @param   parent  The dialog's owner.
     * @param   com     The main component for the dialog.
     */
    public ModalDialog (String name, Dialog parent, Component com)
    {
        super(parent);
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "ModalDialog(String,Dialog,Component)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "parent: " + parent + ", " +
                       "name: " + name + ", " +
                       "com: " + com);
        }
        this.name = name;
        mainComponent = com;
        setModal(true);
    }

    // }}}


    //----------------------------------------------------------------------
    // Public methods
    //----------------------------------------------------------------------
    // {{{ init

    /**
     * Initialize the dialog and the buttons. This method is not called
     * directly by the constructor to give the constructor of a derived class
     * the chance to run first.
     *
     * @param   rb      The resources to use.
     */
    public void init(MapResourceBundle rb)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "init(MapResourceBundle)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "rb: " + rb);
        }

        String title = rb.getString(resName("title"), null);
        if (title != null) {
            setTitle(title);
        }

        String dName = rb.getString(resName("name"), null);
        if (dName != null) {
            setName(dName);
        } else {
            setName("ModalDialog-" + name);
        }

        String[] buttonNames = initButtonNames(rb);
        int bnum = buttonNames == null ? 0 : buttonNames.length;
        buttons = bnum == 0 ? null : new ButtonData [bnum];

        Component[] buttonComs;
        buttonComs = new Component[2 * bnum + 1];
        buttonComs[0] = Box.createHorizontalGlue();
        buttonComs[buttonComs.length - 1] = Box.createHorizontalGlue();
        int maxwidth = 0;
        JButton defaultButton = null;
        for (int i = 0; i < bnum; i++) {
            buttons[i] = initButton(buttonNames[i], rb);
            buttons[i].button.addActionListener(this);
            buttonComs[2*i + 1] = buttons[i].button;
            if (i < buttons.length - 1) {
                buttonComs[2*i + 2] = Box.createHorizontalStrut(10);
            }
            maxwidth = Math.max(maxwidth,
                                buttons[i].button.getPreferredSize().width);
            if (defaultButton == null &&
                buttons[i].deflt) {
                defaultButton = buttons[i].button;
            }
        }
        if (buttons != null) {
            int height = buttons[0].button.getPreferredSize().height;
            for (int i = 0; i < buttons.length; i++) {
                buttons[i].button.setPreferredSize
                    (new Dimension (maxwidth, height));
            }
        }

        panel = new Grid ();

        mainPanel = new JPanel ();
        mainPanel.setLayout(new BorderLayout ());
        initComponent();
        if (mainComponent != null) {
            mainPanel.add(mainComponent, BorderLayout.CENTER);
        }
        ((Grid) panel).addLine(new Component[] {mainPanel}, 1.0);
        ((Grid) panel).addLine(new Component[] {Box.createVerticalStrut(10)});
        ((Grid) panel).addLine(buttonComs);

        panel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        panel.setMinimumSize(panel.getPreferredSize());

        setContentPane(panel);
        if (defaultButton != null) {
            getRootPane().setDefaultButton(defaultButton);
        }
        addWindowListener (this);
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);

        minWidth = rb.getInt(resName("width"), -1);
        minHeight = rb.getInt(resName("height"), -1);
        fixSize();
        // Only for the initial position - may be overridden in doModal
        SwingUtil.centerWindow(this);

        if (rb.getBoolean(resName("configure"), false)) {
            Configuration.instance().register
                (ConfigWrapper.makeWrapper(this, name + ".dialog"));
        }
    }

    // }}}
    // {{{ setMainComponent

    /**
     * Set the main component of the dialog.
     *
     * @param   com     The new main component.
     */
    public void setMainComponent(Component com)
    {
        mainComponent = com;
        if (mainPanel != null) {
            mainPanel.removeAll();
            mainPanel.add(com, BorderLayout.CENTER);
            panel.revalidate();
            fixSize();
        }
    }

    // }}}
    // {{{ doModal

    /**
     * Bring up the dialog and wait for the user to finish editing. If the
     * user selects a button with the close property set, the dialog will be
     * closed.
     *
     * @param   parent     Place the dialog relative to this component if
     *                     non-null.
     * @param   resetFocus Whether to reset the focus to the first child
     *                     component of the dialog.
     *
     * @return -1 if the dialog was canceled, 0 for the first button, 1 for
     * the second button and so on. If the selected button's return code is
     * specified directly by a resource, that value is returned instead.
     */
    public int doModal(Component parent, boolean resetFocus)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "doModal()", "");
        }
        if (parent != null) {
            setLocationRelativeTo(parent);
        }
        if (resetFocus && buttons != null) {
            FocusManager.getCurrentManager().focusNextComponent
                (buttons[buttons.length - 1].button);
            // This is quite a hack - but it appears to be necessary
            addWindowListener(new WindowAdapter() {
                public void windowActivated(WindowEvent e) {
                    SwingUtilities.invokeLater(new Runnable () {
                        public void run() {
                            requestFocus();
                        }
                    });
                    removeWindowListener(this);
                }
            });
        }
        setVisible(true);

        return retval;
    }

    // }}}
    // {{{ getButtonCount

    /**
     * Get the number of buttons of the dialog.
     *
     * @return  The number of buttons.
     */
    public int getButtonCount()
    {
        return buttons == null ? 0 : buttons.length;
    }

    // }}}
    // {{{ getButton(int)

    /**
     * Get a button of the dialog.
     *
     * @param   index   The index of the button.
     *
     * @return  The button at the specified index.
     *
     * @throws IndexOutOfBoundsException If there is no button for that index.
     */
    public JButton getButton(int index)
        throws IndexOutOfBoundsException
    {
        if (buttons == null) {
            throw new IndexOutOfBoundsException ("no buttons");
        }
        return buttons[index].button;
    }

    // }}}
    // {{{ getButton(String)

    /**
     * Get a button of the dialog.
     *
     * @param   name    The resource name of the button.
     *
     * @return  The button with the given name or null if none is found.
     */
    public JButton getButton(String name)
    {
        if (buttons == null) {
            return null;
        }
        for (int i = 0; i < buttons.length; i++) {
            if (buttons[i].name.equals(name)) {
                return buttons[i].button;
            }
        }
        return null;
    }

    // }}}

    //----------------------------------------------------------------------
    // Initialization
    //----------------------------------------------------------------------
    // {{{ initButtonNames

    /**
     * Get the names of the dialog's buttons. The default action is to read
     * the names from the resources.
     *
     * @param   rb      The resources to use.
     *
     * @return  The button names.
     */
    protected String[] initButtonNames(MapResourceBundle rb)
    {
        String buttonRes = rb.getString(resName("buttons"), null);
        if (buttonRes == null) {
            return defaultButtons;
        } else {
            StringTokenizer st = new StringTokenizer (buttonRes, " \t\r\n");
            Vector tmp = new Vector ();
            while (st.hasMoreTokens()) {
                tmp.addElement(st.nextToken());
            }
            String[] names = new String[tmp.size()];
            tmp.copyInto(names);
            return names;
        }
    }

    // }}}
    // {{{ initButton

    /**
     * Initialize one button.
     *
     * @param   bname   The resource name of the button.
     * @param   rb      The resources to use.
     *
     * @return  The initialized button.
     */
    protected ButtonData initButton(String bname, MapResourceBundle rb)
    {
        return new ButtonData (bname, rb);
    }

    // }}}
    // {{{ initComponent

    /**
     * This method can be overridden in a derived class to initialize the
     * {@link #mainComponent mainComponent} member of the dialog. It will be
     * placed a small distance above the buttons, filling all the remaining
     * space.
     */
    protected void initComponent() {}

    // }}}

    //----------------------------------------------------------------------
    // The ActionListener interface
    //----------------------------------------------------------------------
    // {{{ actionPerformed

    /**
     * Handler for a button press. If the button's close property is set, its
     * return value is determined and the dialog is closed.
     *
     * @param   e       The event containing the details.
     */
    public void actionPerformed(ActionEvent e)
    {
        if (e.getSource() instanceof JButton) {
            int idx = findButton ((JButton) e.getSource());
            if (idx >= 0) {
                if (buttons[idx].close) {
                    if (buttons[idx].hasRetval) {
                        retval = buttons[idx].retval;
                    } else {
                        retval = idx;
                    }
                    setVisible(false);
                }
            }
        }
    }

    // }}}

    //----------------------------------------------------------------------
    // The WindowListener interface
    //----------------------------------------------------------------------
    // {{{ windowClosing

    /**
     * Cancellation handler. The return value is set to -1 and the dialog is
     * closed.
     *
     * @param   e       The event containing the details.
     */
    public void windowClosing(WindowEvent e)
    {
        retval = -1;
        setVisible(false);
    }

    // }}}
    // {{{ windowOpened

    /**
     * Noop.
     */
    public void windowOpened(WindowEvent e) {}

    // }}}
    // {{{ windowClosed

    /**
     * Noop.
     */
    public void windowClosed(WindowEvent e) {}

    // }}}
    // {{{ windowActivated

    /**
     * Noop.
     */
    public void windowActivated(WindowEvent e) {}

    // }}}
    // {{{ windowDeactivated

    /**
     * Noop.
     */
    public void windowDeactivated(WindowEvent e) {}

    // }}}
    // {{{ windowIconified

    /**
     * Noop.
     */
    public void windowIconified(WindowEvent e) {}

    // }}}
    // {{{ windowDeiconified

    /**
     * Noop.
     */
    public void windowDeiconified(WindowEvent e) {}

    // }}}

    //----------------------------------------------------------------------
    // Helper methods
    //----------------------------------------------------------------------
    // {{{ fixSize

    /**
     * Pack the dialog and enlarge it if necessary for the minimum width and
     * height.
     */
    protected void fixSize()
    {
        pack();
        Dimension size = getSize();
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "fixSize()",
                       logger.level < Log.MTDDETAIL ? "" :
                       "size: " + size +
                       ", minWidth: " + minWidth +
                       ", minHeight: " + minHeight);
        }
        int w = Math.max(minWidth, size.width);
        int h = Math.max(minHeight, size.height);
        setSize(w, h);
        // let's do a little voodoo magic
        pack();
        size = getSize();
        w = Math.max(minWidth, size.width);
        h = Math.max(minHeight, size.height);
        setSize(w, h);
    }

    // }}}
    // {{{ resName

    /**
     * Helper method that uses a StringBuffer to build a resource name for the
     * dialog. This method should be protected but is public due to a compiler
     * bug in javac.
     *
     * @param   res     The resource needed.
     *
     * @return  The whole resource name: "dialog.<em>name</em>.<em>res</em>".
     */
    public String resName(String res)
    {
        synchronized(sb) {
            sb.setLength(0);
            sb.append("dialog.");
            sb.append(name);
            sb.append('.');
            sb.append(res);
            return sb.toString();
        }
    }

    // }}}
    // {{{ buttonResName

    /**
     * Helper method that uses a StringBuffer to build a resource name for one
     * of the dialog's buttons. This method should be protected but is public
     * due to a compiler bug in javac.
     *
     * @param   button  The resource name of the button.
     * @param   res     The resource needed.
     *
     * @return  The whole resource name:
     *          "dialog.button.<em>button</em>.<em>res</em>".
     */
    public static String
        buttonResName(String button, String res)
    {
        synchronized(sb) {
            sb.setLength(0);
            sb.append("dialog.button.");
            sb.append(button);
            sb.append('.');
            sb.append(res);
            return sb.toString();
        }
    }

    // }}}
    // {{{ findButton

    /**
     * Get the index of a button.
     *
     * @param   button  The button to look for.
     *
     * @return  The index of the button or -1 if not found.
     */
    protected int findButton(JButton button)
    {
        if (buttons == null) {
            return -1;
        }
        for (int i = 0; i < buttons.length; i++) {
            if (buttons[i].button == button) {
                return i;
            }
        }
        return -1;
    }

    // }}}

    //----------------------------------------------------------------------
    // The button data
    //----------------------------------------------------------------------
    // {{{ class ButtonData

    /**
     * A structure that encapsulates the values for a button. <p>
     *
     * This class is public only to work around a javac BUG.
     */
    public static class ButtonData
    {
        // {{{ variables

        /**
         * The resource name of the button.
         */
        public String name;

        /**
         * The button itself.
         */
        public JButton button;

        /**
         * The buttons return value.
         */
        public int retval;

        /**
         * Whether the return value is explicitely defined.
         */
        public boolean hasRetval;

        /**
         * Whether the button should close the dialog.
         */
        public boolean close;

        /**
         * Whether the button is a default button.
         */
        public boolean deflt;

        // }}}

        //------------------------------------------------------------------
        // Constructors
        //------------------------------------------------------------------
        // {{{ ButtonData()

        /**
         * Create a new ButtonData.
         */
        public ButtonData ()
        {
        }

        // }}}
        // {{{ ButtonData(String,MapResourceBundle)

        /**
         * Create a new ButtonData, reading its value from the resources.
         *
         * @param       name    The resource name of the button.
         * @param       rb      The resources to use.
         */
        public ButtonData (String name, MapResourceBundle rb)
        {
            this.name = name;
            String label = rb.getString(buttonResName(name, "label"),
                                        "MISSING: " + name);
            button = new JButton (label);
            button.setName(rb.getString(buttonResName(name, "name"),
                                        "Button-" + name));
            String mnemo = rb.getString(buttonResName(name, "mnemonic"), null);
            if (mnemo != null && mnemo.length() > 0) {
                button.setMnemonic(mnemo.charAt(0));
            }
            if (rb.getString(buttonResName(name, "retval"), null) != null) {
                hasRetval = true;
                retval = rb.getInt(buttonResName(name, "retval"), -1);
            }
            deflt = rb.getBoolean(buttonResName(name, "default"), false);
            if (! deflt) {
                button.setDefaultCapable(false);
            }
            close = rb.getBoolean(buttonResName(name, "close"), true);
            button.setEnabled(! rb.getBoolean
                              (buttonResName(name, "disabled"), false));
        }

        // }}}
    }

    // }}}
}
