// {{{ copyright

/********************************************************************
 *
 * $Id: SortedTableHeaderCellRenderer.java,v 1.6 2000/06/06 16:41:25 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.gui;

// {{{ imports

import java.awt.Component;

import java.net.URL;

import java.util.Enumeration;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.LookAndFeel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.TableColumnModelEvent;
import javax.swing.event.TableColumnModelListener;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.JTableHeader;

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;
import de.qfs.lib.util.MapResourceBundle;

// }}}

/**
 * This CellRenderer draws a little arrow on the column determining the
 * sort order of a table.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.6 $
 */
public class SortedTableHeaderCellRenderer
    extends JLabel
    implements TableCellRenderer, TableModelSorterListener,
               TableColumnModelListener
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private static Logger logger =
        new Logger (SortedTableHeaderCellRenderer.class);

    /**
     * Whether icons are already initialized.
     */
    private static boolean initialized;

    /**
     * The up icon.
     */
    private static Icon upArrow;

    /**
     * The down icon
     */
    private static Icon downArrow;

    /**
     * The empty icon
     */
    private static Icon noArrow;

    /**
     * The table being sorted.
     */
    private JTable table;

    /**
     * The table's ColumnModel.
     */
    private TableColumnModel columnModel;

    /**
     * The column used for sorting
     */
    private int sortColumn = -1;

    /**
     * Whether sorting is ascending or descending
     */
    private boolean ascending = true;

    // }}}

    // {{{ constructor

    /**
     * Create a new SortedTableHeaderCellRenderer.
     *
     * @param   table   The table being rendered.
     */
    public SortedTableHeaderCellRenderer (JTable table)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "SortedTableHeaderCellRenderer(JTable)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "table: " + table);
        }
        initIcons();
        this.table = table;
        JTableHeader header = table.getTableHeader();
        if (header != null) {
            setForeground(header.getForeground());
            setBackground(header.getBackground());
            setFont(header.getFont());
        } else {
            LookAndFeel.installColorsAndFont(this,
                                             "TableHeader.background",
                                             "TableHeader.foreground",
                                             "TableHeader.font");
        }
        LookAndFeel.installBorder(this, "TableHeader.cellBorder");
        setOpaque(true);
        setIconTextGap(2);

        initColumns();
    }

    // }}}
    // {{{ initIcons

    /**
     * Fetch the icons.
     */
    private static synchronized void initIcons()
    {
        if (!initialized) {
            if (logger.level >= Log.MTD) {
                logger.log(Log.MTD, "initIcons()", "");
            }

            URL res = SortedTableHeaderCellRenderer.class.getResource
                ("/de/qfs/lib/resources/icons/uparrow.gif");
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG, "initIcons()",
                           "URL for up arrow: " + res);
            }
            if (res != null) {
                upArrow = new ImageIcon (res);
                if (upArrow == null) {
                    logger.log(Log.WRN, "initIcons()",
                               "Couldn't load Icon for URL " + res);
                }
            }

            res = SortedTableHeaderCellRenderer.class.getResource
                ("/de/qfs/lib/resources/icons/downarrow.gif");
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG, "initIcons()",
                           "URL for down arrow: " + res);
            }
            if (res != null) {
                downArrow = new ImageIcon (res);
                if (downArrow == null) {
                    logger.log(Log.WRN, "initIcons()",
                               "Couldn't load Icon for URL " + res);
                }
            }

            res = SortedTableHeaderCellRenderer.class.getResource
                ("/de/qfs/lib/resources/icons/noarrow.gif");
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG, "initIcons()",
                           "URL for no arrow: " + res);
            }
            if (res != null) {
                noArrow = new ImageIcon (res);
                if (noArrow == null) {
                    logger.log(Log.WRN, "initIcons()",
                               "Couldn't load Icon for URL " + res);
                }
            }

            initialized = true;
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG, "initIcons()",
                           "up: " + upArrow + ", down: " + downArrow);
            }
        }
    }

    // }}}
    // {{{ reinit

    /**
     * This method must be called after changing the TableModel or the
     * TableColumnModel of the table.
     */
    public void reinit()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "reinit()", "");
        }
        if (columnModel != null) {
            columnModel.removeColumnModelListener(this);
        }
        initColumns();
    }


    // }}}
    // {{{ initColumns

    /**
     * Set this Renderer as HeaderRenderer for the columns of the table.
     */
    private void initColumns()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "initColumns()", "");
        }
        columnModel = table.getColumnModel();
        for (Enumeration e = columnModel.getColumns(); e.hasMoreElements(); ) {
            TableColumn col = (TableColumn) e.nextElement();
            col.setHeaderRenderer(this);
        }
        columnModel.addColumnModelListener(this);
    }

    // }}}

    //----------------------------------------------------------------------
    // TableCellRenderer
    //----------------------------------------------------------------------

    // {{{ getTableCellRendererComponent

    /**
     * Get the Component to render the cell.
     */
    public Component getTableCellRendererComponent
            (JTable table, Object value, boolean isSelected,
             boolean hasFocus, int row, int column)
    {
        // called too often
        //  if (Flags.LOG && TRACE) {
            //  logger.log(Log.MTD,
                       //  "getTableCellRendererComponent" +
                       //  "(JTable,Object,boolean,boolean,int,int)",
                       //  "table: " + table + ", " +
                       //  "value: " + value + ", " +
                       //  "isSelected: " + isSelected + ", " +
                       //  "hasFocus: " + hasFocus + ", " +
                       //  "row: " + row + ", " +
                       //  "column: " + column);
        //  }

        setText(value.toString());
        int idx = table.convertColumnIndexToModel(column);
        if (idx == sortColumn) {
            setIcon(ascending ? downArrow : upArrow);
        } else {
            setIcon(noArrow);
        }
        return this;
    }

    // }}}

    //----------------------------------------------------------------------
    // TableModelSorterListener
    //----------------------------------------------------------------------

    // {{{ sortOrderChanged

    /**
     * Set the column used for sorting.
     *
     * @param   e       The event holding the details.
     */
    public void sortOrderChanged(TableModelSorterEvent e)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "sortOrderChanged(TableModelSorterEvent)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "e: " + e +
                       ", column: " + e.getColumn() +
                       ", ascending: " + e.isAscending());
        }
        sortColumn = e.getColumn();
        ascending = e.isAscending();
        table.getTableHeader().repaint();
    }

    // }}}

    //----------------------------------------------------------------------
    // TableColumnModelListener
    //----------------------------------------------------------------------

    // {{{ columnAdded

    /**
     * Set renderer for new columns.
     */
    public void columnAdded(TableColumnModelEvent e)
    {
        ((TableColumnModel) e.getSource()).getColumn(e.getToIndex())
            .setHeaderRenderer(this);
    }

    // }}}
    // {{{ columnMarginChanged

    /**
     * Ignored
     */
    public void columnMarginChanged(ChangeEvent e)
    {
    }

    // }}}
    // {{{ columnMoved

    /**
     * Ignored
     */
    public void columnMoved(TableColumnModelEvent e)
    {
    }

    // }}}
    // {{{ columnRemoved

    /**
     * Ignored
     */
    public void columnRemoved(TableColumnModelEvent e)
    {
    }

    // }}}
    // {{{ columnSelectionChanged

    /**
     * Ignored
     */
    public void columnSelectionChanged(ListSelectionEvent e)
    {
    }

    // }}}
}
