// {{{ copyright

/********************************************************************
 *
 * $Id: SortedTableHelper.java,v 1.15 2000/11/29 14:38:36 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.gui;

// {{{ imports

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;
import de.qfs.lib.util.DelayedAction;

// }}}

/**
 * This little helper class is very useful in creating a sorted JTable.
 * It creates and connects all the necessary parts, while still leaving
 * everything configurable.<p>
 *
 * The fastest way to get a standard sorted table is
 * <pre>
 * JTable table;
 * SortedTableHelper helper = new SortedTableHelper (table);
 * helper.prepareTable();
 * </pre>
 *
 * This will create a table with a {@link FilteredAndSortedTableModel
 * FilteredAndSortedTableModel} using a {@link DefaultTableModelSorter
 * DefaultTableModelSorter} and a {@link SortedTableHeaderCellRenderer
 * SortedTableHeaderCellRenderer} using the default arrow icons.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.15 $
 */
public class SortedTableHelper
    implements TableModelFilterListener, TableModelSorterListener
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private static Logger logger = new Logger (SortedTableHelper.class);

    /**
     * The logger for the HeaderListener.
     */
    private final static Logger hlLogger = new Logger (HeaderListener.class);

    /**
     * The table to sort.
     */
    private JTable table;

    /**
     * The original model of the table.
     */
    private TableModel model;

    /**
     * The sorted model of the table.
     */
    private FilteredAndSortedTableModel sortedModel;

    /**
     * The sorter for the table.
     */
    private TableModelSorter sorter;

    /**
     * The filter for the table.
     */
    private TableModelFilter filter;

    /**
     * The renderer for the TableHeader.
     */
    private TableCellRenderer renderer;

    // }}}

    // {{{ constructors

    /**
     * Create a new SortedTableHelper. The helper will sort the table using a
     * {@link DefaultTableModelSorter DefaultTableModelSorter}, no filter and
     * a standard {@link SortedTableHeaderCellRenderer
     * SortedTableHeaderCellRenderer}.
     *
     * @param   table   The table that is to be sorted.
     */
    public SortedTableHelper(JTable table)
    {
        this(table, null, new DefaultTableModelSorter ());
        sorter.setSortColumn(0);
    }

    /**
     * Create a new SortedTableHelper. The helper will sort the table using
     * the given sorter and filter and a standard {@link
     * SortedTableHeaderCellRenderer SortedTableHeaderCellRenderer}.
     *
     * @param   table   The table that is to be sorted.
     * @param   filter  The filter for the table. If null, the table will not
     *          be filtered.
     * @param   sorter  The sorter for the table. If null, the table will not
     *                  be sorted.
     */
    public SortedTableHelper(JTable table, TableModelFilter filter,
                             TableModelSorter sorter)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD,
                       "SortedTableHelper(JTable,TableModelSorter," +
                       "TableModelFilter)",
                       "table: " + table + ", " +
                       "sorter: " + sorter + ", " +
                       "filter: " + filter);
        }
        this.table = table;
        model = table.getModel();

        this.filter = filter;
        this.sorter = sorter;

        sortedModel =
            new FilteredAndSortedTableModel (model, this.filter, this.sorter);
        this.renderer = new SortedTableHeaderCellRenderer (table);
    }

    /**
     * Create a new SortedTableHelper from a FilteredAndSortedTableModel.
     *
     * @param   table   The table that is to be sorted.
     * @param   sortedModel The FilteredAndSortedTableModel to use.
     */
    public SortedTableHelper(JTable table,
                             FilteredAndSortedTableModel sortedModel)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD,
                       "SortedTableHelper(JTable,FilteredAndSortedTableModel)",
                       "table: " + table + ", " +
                       "sortedModel: " + sortedModel);
        }

        this.table = table;

        this.sortedModel = sortedModel;

        model = sortedModel.getModel();
        filter = sortedModel.getFilter();
        sorter = sortedModel.getSorter();

        this.renderer = new SortedTableHeaderCellRenderer (table);
    }

    // }}}

    // {{{ prepareTable

    /**
     * Prepare the table for sorting. This sets up everything and installs
     * a MouseListener on the Table's header for sorting.
     */
    public void prepareTable()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "prepareTable()", "");
        }
        table.setModel(sortedModel);
        if (renderer instanceof SortedTableHeaderCellRenderer) {
            ((SortedTableHeaderCellRenderer) renderer).reinit();
        }
        if (sorter != null) {
            sorter.addTableModelSorterListener(this);
        }
        if (filter != null) {
            filter.addTableModelFilterListener(this);
        }
        if (sorter != null
            && (renderer instanceof TableModelSorterListener)) {
            sorter.addTableModelSorterListener
                ((TableModelSorterListener) renderer);
            ((TableModelSorterListener) renderer).sortOrderChanged
                (new TableModelSorterEvent (sorter,
                                            sorter.getSortColumn(),
                                            sorter.isSortAscending()));
        }
        if (sorter != null || filter != null) {
            saveSelection(true);
        }
        table.getTableHeader().addMouseListener(new HeaderListener ());
    }

    // }}}
    // {{{ saveSelection

    /**
     * Set whether the FilteredAndSortedTableModel will save the selected rows
     * of the table when sorting, filtering or updating. <p>
     *
     * This method is automtically called from {@link #prepareTable
     * prepareTable}, but it must be called again whenever the row selection
     * model of the table is changed.
     *
     * @param   save    True to save, false to ignore the selection.
     */
    public void saveSelection(boolean save)
    {
        if (save) {
            if (sortedModel != null
                && table != null) {
                sortedModel.setSavedSelectionModel(table.getSelectionModel());
            }
        } else {
            if (sortedModel != null) {
                sortedModel.setSavedSelectionModel(null);
            }
        }
    }

    // }}}
    // {{{ updateUI

    /**
     * This method should be called if you change the UI of the table, even if
     * you use {@link javax.swing.SwingUtilities#updateComponentTreeUI
     * SwingUtilities.updateComponentTreeUI}, since otherwise the header
     * renderer of the table doesn't get updated.
     *
     * @since   0.97.1
     */
    public void updateUI()
    {
        if (renderer instanceof SortedTableHeaderCellRenderer) {
            setHeaderCellRenderer(new SortedTableHeaderCellRenderer (table));
        }
    }

    // }}}

    //----------------------------------------------------------------------
    // Access to the variables
    //----------------------------------------------------------------------
    // {{{ getTableModel

    /**
     * Get the original TableModel of the table.
     *
     * @return  The original TableModel.
     */
    public final TableModel getTableModel()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getTableModel()", "");
        }
        return model;
    }

    // }}}
    // {{{ setTableModel

    /**
     * Call this method instead of {@link JTable#setModel JTable.setModel}
     * to change the model of the table. It will take care of updating all
     * other components necessary for sorting.
     *
     * @param   model   The new TableModel.
     */
    public void setTableModel(TableModel model)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "setTableModel(TableModel)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "model: " + model);
        }

        //  if (sortedModel != null && this.model != null) {
            //  this.model.removeTableModelListener(sortedModel);
        //  }
        //  if (sortedModel != null && sorter != null) {
            //  sorter.removeTableModelSorterListener(sortedModel);
        //  }

        this.model = model;

        sortedModel.setModel(model);

        if (renderer instanceof SortedTableHeaderCellRenderer) {
            ((SortedTableHeaderCellRenderer) renderer).reinit();
        }
    }

    // }}}
    // {{{ getSortedTableModel

    /**
     * Get the FilteredAndSortedTableModel used to sort the table.
     *
     * @return  The TableModel used for filtering and sorting.
     */
    public final FilteredAndSortedTableModel getSortedTableModel()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getSortedTableModel()", "");
        }
        return sortedModel;
    }

    // }}}
    // {{{ setSortedTableModel

    /**
     * Call this method instead of {@link JTable#setModel JTable.setModel}
     * to change the model of the table. It will take care of updating all
     * other components necessary for sorting.
     *
     * @param   model   The new TableModel.
     */
    public void setSortedTableModel(FilteredAndSortedTableModel model)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD,
                       "setSortedTableModel(FilteredAndSortedTableModel)",
                       "model: " + model);
        }

        if (sorter != null) {
            sorter.removeTableModelSorterListener(this);
            if (sortedModel != null) {
                sorter.removeTableModelSorterListener(sortedModel);
            }
            if (renderer instanceof TableModelSorterListener) {
                sorter.removeTableModelSorterListener
                    ((TableModelSorterListener) renderer);
            }
        }

        if (filter != null) {
            filter.removeTableModelFilterListener(this);
        }

        sortedModel = model;
        this.model = sortedModel.getModel();

        sorter = sortedModel.getSorter();
        if (sorter != null) {
            sorter.addTableModelSorterListener(this);
        }
        filter = sortedModel.getFilter();
        if (filter != null) {
            filter.addTableModelFilterListener(this);
        }

        if (table.getCellEditor() != null) {
            table.getCellEditor().cancelCellEditing();
        }
        table.setModel(sortedModel);

        if (renderer instanceof SortedTableHeaderCellRenderer) {
            ((SortedTableHeaderCellRenderer) renderer).reinit();
        }
        if (sorter != null
            && (renderer instanceof TableModelSorterListener)) {
                sorter.addTableModelSorterListener
                    ((TableModelSorterListener) renderer);
                ((TableModelSorterListener) renderer).sortOrderChanged
                    (new TableModelSorterEvent (sorter,
                                                sorter.getSortColumn(),
                                                sorter.isSortAscending()));
        }
    }

    // }}}
    // {{{ getTableModelSorter

    /**
     * Get the TableModelSorter implementing the sort order.
     *
     * @return  The table's sorter.
     */
    public final TableModelSorter getTableModelSorter()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getTableModelSorter()", "");
        }
        return sorter;
    }


    // }}}
    // {{{ setTableModelSorter

    /**
     * Set the TableModelSorter implementing the sort order. Will cause a full
     * redraw of the table.
     *
     * @param   sorter  The sorter to set.
     */
    public final void setTableModelSorter(TableModelSorter sorter)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "setTableModelSorter(TableModelSorter)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "sorter: " + sorter);
        }
        if (this.sorter != null) {
            sorter.removeTableModelSorterListener(this);
            if (renderer instanceof TableModelSorterListener) {
                this.sorter.removeTableModelSorterListener
                    ((TableModelSorterListener) renderer);
            }
        }
        this.sorter = sorter;
        if (table.getCellEditor() != null) {
            table.getCellEditor().cancelCellEditing();
        }
        sortedModel.setSorter(sorter);
        if (sorter != null) {
            sorter.addTableModelSorterListener(this);
            if (renderer instanceof TableModelSorterListener) {
                sorter.addTableModelSorterListener
                    ((TableModelSorterListener) renderer);
                ((TableModelSorterListener) renderer).sortOrderChanged
                    (new TableModelSorterEvent (sorter,
                                                sorter.getSortColumn(),
                                                sorter.isSortAscending()));
            }
        }
    }


    // }}}
    // {{{ getTableModelFilter

    /**
     * Get the TableModelFilter for the table.
     *
     * @return  The table's filter.
     */
    public final TableModelFilter getTableModelFilter()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getTableModelFilter()", "");
        }
        return filter;
    }


    // }}}
    // {{{ setTableModelFilter

    /**
     * Set the TableModelFilter for the table. Will cause a full redraw of the
     * table.
     *
     * @param   filter  The filter to set.
     */
    public final void setTableModelFilter(TableModelFilter filter)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "setTableModelFilter(TableModelFilter)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "filter: " + filter);
        }
        if (this.filter != null) {
            filter.removeTableModelFilterListener(this);
        }
        this.filter = filter;
        if (filter != null) {
            filter.addTableModelFilterListener(this);
        }
        if (table.getCellEditor() != null) {
            table.getCellEditor().cancelCellEditing();
        }
        sortedModel.setFilter(filter);
    }


    // }}}
    // {{{ getHeaderCellRenderer

    /**
     * Get the renderer for the TableHeader.
     *
     * @return  The table's header renderer.
     */
    public final TableCellRenderer getHeaderCellRenderer()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getHeaderCellRenderer()", "");
        }
        return renderer;
    }

    // }}}
    // {{{ setHeaderCellRenderer

    /**
     * Set the renderer for the TableHeader.
     *
     * @param   renderer        The header renderer to set.
     */
    public final void setHeaderCellRenderer(TableCellRenderer renderer)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "setHeaderCellRenderer(TableCellRenderer)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "renderer: " + renderer);
        }
        if (sorter != null
            && (this.renderer instanceof TableModelSorterListener)) {
            sorter.removeTableModelSorterListener
                ((TableModelSorterListener) this.renderer);
        }
        this.renderer = renderer;
        if (renderer instanceof SortedTableHeaderCellRenderer) {
            ((SortedTableHeaderCellRenderer) renderer).reinit();
        }
        if (sorter != null
            && (renderer instanceof TableModelSorterListener)) {
                sorter.addTableModelSorterListener
                    ((TableModelSorterListener) renderer);
                ((TableModelSorterListener) renderer).sortOrderChanged
                    (new TableModelSorterEvent (sorter,
                                                sorter.getSortColumn(),
                                                sorter.isSortAscending()));
        }
    }

    // }}}

    //----------------------------------------------------------------------
    // Listener interfaces
    //----------------------------------------------------------------------
    // {{{ filterChanged

    /**
     * Notify the listener of a change in the TableModelFilter.
     *
     * @param   event   Details about the change.
     *
     * @since   0.98.0
     */
    public void filterChanged(TableModelFilterEvent e)
    {
        if (table.getCellEditor() != null) {
            table.getCellEditor().cancelCellEditing();
        }
    }

    // }}}
    // {{{ sortOrderChanged

    /**
     * Notify the listener of a change in the sort order.
     *
     * @param   event   Details about the change.
     *
     * @since   0.98.0
     */
    public void sortOrderChanged(TableModelSorterEvent e)
    {
        if (table.getCellEditor() != null) {
            table.getCellEditor().cancelCellEditing();
        }
    }

    // }}}

    //----------------------------------------------------------------------
    // The MouseListener for the TableHeader
    //----------------------------------------------------------------------
    // {{{ class HeaderListener

    /**
     * Listener for mouse clicks on the Table's header.
     */
    private class HeaderListener extends MouseAdapter
    {
        /**
         * Delay changes so multiple clicks can be detected.
         */
        private DelayedAction delay = new DelayedAction (300);

        /**
         * Handle a mouse click.
         *
         * @param       e       The event contining the details.
         */
        public void mouseClicked(MouseEvent e)
        {
            if (hlLogger.level >= Log.MTD) {
                hlLogger.log(Log.MTD, "mouseClicked(MouseEvent)",
                           hlLogger.level < Log.MTDDETAIL ? "" :
                           "e: " + e);
            }
            final int column = table.getColumnModel()
                .getColumnIndexAtX(e.getX());
            final int ccol = table.convertColumnIndexToModel(column);
            if (column != -1) {
                if(e.getClickCount() == 1) {
                    if (sorter != null) {
                        delay.perform(new Runnable () {
                            public void run() {
                                // if (table.getCellEditor() != null) {
                                    // table.getCellEditor().cancelCellEditing();
                                // }
                                sorter.setSortColumn(ccol);
                            }
                        });
                    }
                } else {
                    delay.cancel();
                    if (e.getClickCount() == 2) {
                        SwingUtil.autoSizeTableColumn(table, column);
                    }
                }
            }
        }
    }

    // }}}

}
