// {{{ copyright

/********************************************************************
 *
 * $Id: SplashWindow.java,v 1.6 2000/07/10 13:30:29 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.gui;

// {{{ imports

import java.awt.Window;
import java.awt.Frame;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.JProgressBar;

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

/**
 * The SplashWindow is a special window intended to be displayed while an
 * application initializes itself, to inform the user about what is going on.
 *
 * A SplashWindow includes a ProgressBar at its bottom. It should be subclassed
 * and customized for additional display.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.6 $
 */
public class SplashWindow extends Window
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger = new Logger (SplashWindow.class);

    /**
     * The last SplashWindow that was displayed.
     */
    private static SplashWindow lastWindow;

    /**
     * The ProgressBar at the bottom of the window. See {@link #setProgress
     * setProgress} for how to customize it.
     */
    protected JProgressBar bar;

    /**
     * This panel holds the contents of the window. It has got a raised border.
     */
    protected JPanel panel;

    // }}}

    // {{{ constructor

    /**
     * Create a new SplashWindow. It will be centered on the screen but not
     * shown yet.
     *
     * @param   width   The width of the SplashWindow.
     * @param   height  The height of the SplashWindow
     */
    public SplashWindow (int width, int height)
    {
        super (new Frame ());
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "SplashWindow(int,int)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "width: " + width + ", " +
                       "height: " + height);
        }
        setSize(width, height);
        SwingUtil.centerWindow(this);
        setBackground(Color.lightGray);
        lastWindow = this;

        panel = new JPanel ();
        panel.setLayout(new BorderLayout ());
        add(panel, "Center");

        bar = new JProgressBar ();
        bar.setString("");
        bar.setStringPainted(true);
        panel.add(bar, "South");
    }

    // }}}

    // {{{ getLastSplashWindow

    /**
     * Get the last SplashWindow that was created. This is like a floating
     * singleton. Typically there is only one SplashWindow per application
     * and this method simplifies access to it from different places.
     *
     * @return  The last SplashWindow created.
     */
    public static SplashWindow getLastSplashWindow()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getLastSplashWindow()", "");
        }
        return lastWindow;
    }

    // }}}

    // {{{ remove

    /**
     * Remove the SplashWindow from the screen and dispose it.
     */
    public synchronized void remove()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "remove()", "");
        }
        setVisible(false);
        SwingUtil.cleanup(this);
        dispose();
        if (lastWindow == this) {
            lastWindow = null;
        }
    }

    // }}}
    // {{{ setProgress

    /**
     * Set information for the ProgressBar of the SplashWindow.
     *
     * @param   text    The text information to display.
     * @param   current The current progress value.
     * @param   max     The maximum progress value.
     */
    public void setProgress(String text, int current, int max)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "setProgress(String,int,int)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "text: " + text + ", " +
                       "current: " + current + ", " +
                       "max: " + max);
        }
        bar.setString(text);
        bar.setMaximum(max);
        bar.setValue(current);
    }

    // }}}
}
