// {{{ copyright

/********************************************************************
 *
 * $Id: TreeNotification.jpp,v 1.8 2000/06/14 16:50:55 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.gui;

// {{{ imports


import java.util.Arrays;
import java.util.ArrayList;
import java.util.List;


import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

// {{{ doc

/**
 * A TreeNotification supports a mechanism similar to TreeEvents, but with a
 * different philosophy. <p>
 *
 * Often a user has access to a TreeNode that has changed in some way, but
 * doesn't know anything about the TreeModel or the affected tree. Similarly,
 * the node itself might like to notify its model of some change, but it may
 * not be wise to have every node in the tree know about its model. <p>
 *
 * In a typical tree structure a node knows its parent though, so a
 * notification mechanism can be implemented by passing the notification up
 * through the hierarchy to the root node, which must decide what to do with
 * it. As a side effect the path to the affected node can be constructed on
 * the way. <p>
 *
 * This is especially useful when there is a large hierarchy which is
 * displayed in several trees, possibly rooted at different nodes. In this
 * case, structural changes have to be forwarded to all relevant tree models,
 * which may reside in totally unrelated places. Instead of having a central
 * logic that registers all of the models, it suffices if the root nodes of
 * the models know to which model they belong and forward the notification to
 * them. <p>
 *
 * Things get more difficult when the type of the notification is visual
 * rather than structural, e.g. a node should be selected and made visible. In
 * this case typically only one tree is the designated recipient of the
 * notification. Having a root node know about its tree view would violate the
 * Model/View pattern as implemented by TreeModel/JTree anyway. <p>
 *
 * One possible solution is to carry the intended receiver of the notification
 * in the notification itself. A root node can then check, if the receiver is
 * a tree to which the node belongs and act accordingly. <p>
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.8 $
 */

// }}}
public class TreeNotification
{
    // {{{ constants

    /**
     * Indicates that a node has been modified
     */
    public final static int MODIFIED = 0;

    /**
     * Indicates that a node has been inserted
     */
    public final static int INSERTED = 1;

    /**
     * Indicates that a node has been removed
     */
    public final static int REMOVED = 2;

    /**
     * Indicates that the node structure has changed
     */
    public final static int STRUCTURE_CHANGED = 3;

    /**
     * Indicates that a node should be selected
     */
    public final static int ADD_TO_SELECTION = 4;

    /**
     * Indicates that a node should be selected
     */
    public final static int SELECT = 5;

    /**
     * Indicates that a node should be made visible
     */
    public final static int SHOW = 6;

    /**
     * Indicates that a node should be selected and shown
     */
    public final static int SELECT_AND_SHOW = 7;

    /**
     * Indicates that a node should be expanded
     */
    public final static int EXPAND = 8;

    /**
     * Indicates that a node should be collapsed
     */
    public final static int COLLAPSE = 9;

    // }}}
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger = new Logger (TreeNotification.class);

    /**
     * The path of the TreeNotification.
     */
    private List path;

    /**
     * The operation of the TreeNotification.
     */
    private int operation;

    /**
     * The index of the TreeNotification.
     */
    private int index;

    /**
     * The designated receiver of the TreeNotification.
     */
    private Object receiver;

    // }}}

    // {{{ constructors

    /**
     * Create a new TreeNotification.
     *
     * @param   operation       The type of the notification.
     * @param   index           An optional index for INSERTED or REMOVED
     *                          changes.
     */
    public TreeNotification (int operation, int index)
    {
        path = new ArrayList();
        this.operation = operation;
        this.index = index;
    }

    /**
     * Create a new TreeNotification.
     *
     * @param   receiver        The designated receiver of the notification.
     * @param   operation       The type of the notification.
     * @param   index           An optional index for INSERTED or REMOVED
     *                          changes.
     */
    public TreeNotification (Object receiver, int operation, int index)
    {
        path = new ArrayList();
        this.receiver = receiver;
        this.operation = operation;
        this.index = index;
    }

    // }}}

    // {{{ prependNode

    /**
     * Prepend a node to the path for the TreeNotification.
     *
     * @param   node    The node to prepend.
     */
    public void prependNode(Object node)
    {
        path.add(0, node);
    }

    // }}}
    // {{{ getReceiver

    /**
     * Get the designated receiver of the TreeNotification.
     *
     * @return  The receiver of the TreeNotification.
     */
    public final Object getReceiver()
    {
        return receiver;
    }

    // }}}
    // {{{ setReceiver

    /**
     * Set the designated receiver of the TreeNotification.
     *
     * @param   receiver        The receiver to set.
     */
    public final void setReceiver(Object receiver)
    {
        this.receiver = receiver;
    }

    // }}}
    // {{{ getPath

    /**
     * Get the path of the TreeNotification.
     *
     * @return  The path of the TreeNotification.
     */
    public final Object[] getPath()
    {
        return path.toArray();
    }

    // }}}
    // {{{ setPath

    /**
     * Set the path of the TreeNotification.
     *
     * @param   path    The path to set.
     */
    public final void setPath(Object[] path)
    {
        this.path = new ArrayList (Arrays.asList(path));
    }

    // }}}
    // {{{ getOperation

    /**
     * Get the operation of the TreeNotification.
     *
     * @return  The operation of the TreeNotification.
     */
    public final int getOperation()
    {
        return operation;
    }

    // }}}
    // {{{ setOperation

    /**
     * Set the operation of the TreeNotification.
     *
     * @param   operation       The operation to set.
     */
    public final void setOperation(int operation)
    {
        this.operation = operation;
    }

    // }}}

    // {{{ getIndex

    /**
     * Get the index of the TreeNotification.
     *
     * @return  The index of the TreeNotification.
     */
    public final int getIndex()
    {
        return index;
    }

    // }}}
    // {{{ setIndex

    /**
     * Set the index of the TreeNotification.
     *
     * @param   index   The index to set.
     */
    public final void setIndex(int index)
    {
        this.index = index;
    }

    // }}}

    // {{{ toString

    /**
     * Get a string representation of the TreeNotification.
     *
     * @return  The string representation.
     */
    public String toString()
    {
        StringBuffer sb = new StringBuffer ();

        for (int i = 0; i < path.size(); i++) {
            sb.append(path.get(i).toString());
            if (i < path.size() - 1) {
                sb.append(".");
            }
        }
        return (super.toString() +
                "[operation=" + operation +
                ",index=" + index +
                ",path=" + sb.toString() + "]");
    }

    // }}}
}
