// {{{ copyright

/********************************************************************
 *
 * $Id: WholeRowFocusRenderer.java,v 1.7 2000/06/06 16:41:26 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.gui;

// {{{ imports

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.table.DefaultTableCellRenderer;

// }}}

/**
 * This TableCellRenderer draws a focus border around a whole table row,
 * ignoring the focused column. <p>
 *
 * The border can either be drawn in a solid color, or by tiling an icon
 * image. The default is to use a stipple icon for the Windows Look and Feel
 * and a solid border for the others, where the border color is the
 * (selected) foreground color of the cells text. <p>
 *
 * Te get a smooth border, the column margin of the table's ColumnModel must
 * be set to 0. <p>
 *
 * Since the whole focused table row needs a repaint whenever the table gains
 * or loses the focus, the WholeRowFocusRenderer must also be added as a
 * FocusListener to the table to take care of the necessary repaint.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.7 $
 */
public class WholeRowFocusRenderer
    extends DefaultTableCellRenderer
    implements Border, FocusListener
{
    // {{{ variables

    /**
     * The Border insets.
     */
    private final static Insets insets = new Insets (1, 1, 1, 1);

    /**
     * Stipple icon used for windows L&F.
     */
    private static Icon stippleIcon;

    /**
     * Whether the current row is focused.
     */
    private boolean hasFocus;

    /**
     * Optional icon for the border.
     */
    private Icon icon;

    /**
     * The icon's width.
     */
    private int iconWidth;

    /**
     * The icon's height.
     */
    private int iconHeight;

    /**
     * An offset to get correct icon tile alignment for different columns.
     */
    private int xOffset;

    /**
     * The color to use for the border.
     */
    private Color borderColor;

    /**
     * The borderColor set by the user.
     */
    private Color userBorderColor;

    /**
     * Whether the current column is the leftmost.
     */
    boolean left;

    /**
     * Whether the current column is the rightmost.
     */
    boolean right;

    // }}}

    // {{{ constructor

    /**
     * Create a new WholeRowFocusRenderer
     */
    public WholeRowFocusRenderer ()
    {
        if (UIManager.getLookAndFeel().getID().indexOf("Windows") > 0) {
            // default for windows L&F is stippled
            setStippleBorder();
        }
    }

    // }}}

    // {{{ getTableCellRendererComponent

    /**
     * Get the Component used to paint the table cell.
     *
     * @param   table   The table to paint.
     * @param   value   The value of the cell.
     * @param   selected        Whether the cell is selected.
     * @param   focus           Whether the cell has the focus - ignored.
     * @param   row     The cell's row index.
     * @param   column  The cell's column index.
     *
     * @return  A JLabel that will paint the cell with the correct border.
     */
    public Component getTableCellRendererComponent
    (JTable table, Object value, boolean selected, boolean focus,
     int row, int column)
    {
        int anchorRow = table.getSelectionModel().getAnchorSelectionIndex();
        hasFocus = (row == anchorRow && table.hasFocus());
        JLabel label = (JLabel) super.getTableCellRendererComponent
            (table, value, selected, false, row, column);

        label.setBorder(this);
        left = column == 0;
        right = column == table.getColumnCount() - 1;
        if (hasFocus) {
            if (icon != null) {
                xOffset = table.getCellRect(row, column, true).x % iconWidth;
            } else if (userBorderColor == null) {
                borderColor = selected
                    ? table.getSelectionForeground()
                    : getForeground();
            }
        }
        return label;
    }

    // }}}
    // {{{ setBorderColor

    /**
     * Set the color for the focus border.
     *
     * @param   color   The color for the border.
     */
    public void setBorderColor(Color color)
    {
        userBorderColor = color;
        if (color != null) {
            borderColor = color;
            icon = null;
        }
    }

    // }}}
    // {{{ setStippleBorder

    /**
     * Set a stipple border similar to the one Windows uses.
     */
    public void setStippleBorder()
    {
        if (stippleIcon == null) {
            stippleIcon = new ImageIcon
                          (getClass().getResource
                           ("/de/qfs/lib/resources/icons/stipple.gif"));
        }
        setBorderIcon(stippleIcon);
    }

    // }}}
    // {{{ setBorderIcon

    /**
     * Set the icon to use for painting the border.
     *
     * @param   icon    The icon for the border.
     */
    public void setBorderIcon(Icon icon)
    {
        this.icon = icon;
        if (icon != null) {
            iconWidth = icon.getIconWidth();
            iconHeight = icon.getIconHeight();
            if (iconWidth <= 0 || iconHeight <= 0) {
                // useless icon
                this.icon = null;
            } else {
                userBorderColor = null;
            }
        }
    }

    // }}}

    //----------------------------------------------------------------------
    // The Border interface
    //----------------------------------------------------------------------
    // {{{ getBorderInsets

    /**
     * Get the insets for the border.
     *
     * @param   c       Ignored.
     *
     * @return  Insets(1, 1, 1, 1).
     */
    public Insets getBorderInsets(Component c)
    {
        return insets;
    }

    // }}}
    // {{{ isBorderOpaque

    /**
     * Query whether the border is opaque.
     *
     * @return  Always false.
     */
    public boolean isBorderOpaque()
    {
        return false;
    }

    // }}}
    // {{{ paintBorder

    /**
     * Draw the border.
     *
     * @param   c       Ignored.
     * @param   g       The Graphics to draw on.
     * @param   x       Left border coordinate.
     * @param   y       Top border coordinate.
     * @param   width   Border width.
     * @param   height  Border height.
     */
    public void paintBorder(Component c, Graphics g,
                            int x, int y, int width, int height)
    {
        if (! hasFocus) {
            // empty border
            return;
        }

        if (icon != null) {
            int ix = 0;
            int iy = 0;
            int relx, rely;
            Graphics cg = g.create();

            g.translate(x, y);

            // top edge
            cg.setClip(0, 0, width, 1);
            for (ix = -xOffset; ix < width; ix += iconWidth) {
                icon.paintIcon(c, cg, ix, 0);
            }

            // bottom edge
            cg.setClip(0, height - 1, width, 1);
            iy = (height - 1) - ((height - 1)%iconHeight);
            for (ix = -xOffset; ix < width; ix += iconWidth) {
                icon.paintIcon(c, cg, ix, iy);
            }

            if (left) {
                // left edge
                cg.setClip(0, 1, 1, height - 2);
                for (iy = 0; iy < height; iy += iconHeight) {
                    icon.paintIcon(c, cg, -xOffset, iy);
                }
            }

            if (right) {
                // right edge
                cg.setClip(width - 1, 1, 1, height - 2);
                ix = width - 1 - ((width - 1 - xOffset)%iconWidth);
                for (iy = 0; iy < height; iy += iconHeight) {
                    icon.paintIcon(c, cg, ix, iy);
                }
            }
            cg.dispose();
            g.translate(-x, -y);
        } else {
            g.setColor(borderColor);
            g.drawLine(x, y, x + width - 1, y);
            g.drawLine(x, y + height - 1, x + width - 1, y + height - 1);
            if (left) {
                g.drawLine(x, y, x, y + height - 1);
            }
            if (right) {
                g.drawLine(x + width - 1, y, x + width - 1, y + height - 1);
            }
        }
    }

    // }}}

    //----------------------------------------------------------------------
    // The FocusListener interface
    //----------------------------------------------------------------------
    // {{{ focusGained

    /**
     * Notify the renderer that the table gained the focus.
     *
     * @param   e       Details about the event.
     */
    public void focusGained(FocusEvent e)
    {
        if (e.getSource() instanceof JTable) {
            repaintFocusRow((JTable) e.getSource());
        }
    }

    // }}}
    // {{{ focusLost

    /**
     * Notify the renderer that the table lost the focus.
     *
     * @param   e       Details about the event.
     */
    public void focusLost(FocusEvent e)
    {
        if (e.getSource() instanceof JTable) {
            repaintFocusRow((JTable) e.getSource());
        }
    }

    // }}}
    // {{{ repaintRow

    /**
     * Repaint the focused row of a table.
     *
     * @param   table   The table to repaint.
     */
    private void repaintFocusRow(JTable table)
    {
        int row = table.getSelectionModel().getAnchorSelectionIndex();
        if (row < 0) {
            return;
        }
        Rectangle r1 = table.getCellRect(row, 0, true);
        Rectangle r2 = table.getCellRect(row, table.getColumnCount() - 1,
                                         true);
        table.repaint(r1.x, r1.y, r2.x - r1.x + r2.width, r2.height);
    }

    // }}}
}
