// {{{ copyright

/********************************************************************
 *
 * $Id: IntTests.jpp,v 1.11 2001/02/17 09:41:28 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.log;


// {{{ imports

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;
import java.io.PrintWriter;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

// }}}

/**
 * Integration tests for the log package.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.11 $
 */
public class IntTests
    extends TestCase
{
    // {{{ variables

    /**
     * Logger used in tests.
     */
    private final static Logger logger = new Logger (IntTests.class);

    /**
     * Line separator (system dependent).
     */
    private final static String LINESEP =
        System.getProperty("line.separator");

    /**
     * Id for entries logged.
     */
    private int id;

    // }}}

    // {{{ constructor

    /**
     * Create a new IntTests.
     *
     * @param   name    The name of the test.
     */
    public IntTests(String name)
    {
            super(name);
    }

    // }}}

    // {{{ setUp

    /**
     * Set up the test fixture.
     */
    protected void setUp()
    {
        id = 0;
        Log.setOutputLevel(Log.ERRDETAIL);
    }

    // }}}
    // {{{ suite

    /**
     * Get the suite of tests.
     *
     * @return  The tests.
     */
    public static Test suite()
    {
            return new TestSuite(IntTests.class);
    }

    // }}}

    //----------------------------------------------------------------------
    // Tests
    //----------------------------------------------------------------------
    // {{{ testLevels

    /**
     * Test setting the level for the logger.
     */
    public void testLevels()
    {
        Logger.setDefaultLogLevel(Log.ERR);
        assert(logger.level == Log.ERR);

        Logger.setLogLevel("de.", Log.MTD);
        assert(logger.level == Log.MTD);

        Logger.setLogLevel(IntTests.class.getName(), Log.DBG);
        assert(logger.level == Log.DBG);
    }

    // }}}
    // {{{ testFilter

    /**
     * Test filtering.
     */
    public void testFilter()
    {
        Filter filter = new Filter ();
        Log.addFilter(filter);

        Logger.setLogLevel(IntTests.class.getName(), Log.DBG);
        Log.setPreQueueLevel(Log.DBG);

        log(Log.DBG, 5);

        flush();
        assert(filter.count == 5);
        assertNotNull(filter.entries);
        assert(filter.entries[filter.entries.length - 1].getMessage()
               .equals("4"));

        Log.setQueueing(false);
        log(Log.DBG, 5);

        assert(filter.count == 10);
        assert(filter.entries.length == 1);
        assert(filter.entries[0].getMessage().equals("9"));

        Log.setQueueing(true);
        Log.removeFilter(filter);
    }

    // }}}
    // {{{ testPreQueueLevel

    /**
     * Test the pre queue level.
     */
    public void testPreQueueLevel()
    {
        Filter filter = new Filter ();
        Log.addFilter(filter);

        Logger.setLogLevel(IntTests.class.getName(), Log.DBGDETAIL);
        Log.setPreQueueLevel(Log.DBG);
        Log.setQueueing(true);

        log(Log.DBGDETAIL, 5);
        flush();
        assert(filter.count == 0);

        log(Log.DBG, 5);
        flush();
        assert(filter.count == 5);
        assertNotNull(filter.entries);
        assert(filter.entries[filter.entries.length - 1].getMessage()
               .equals("9"));

        Log.removeFilter(filter);
    }

    // }}}
    // {{{ testFlushBuffer

    /**
     * Test the flush buffer.
     */
    public void testFlushBuffer()
    {
        Filter filter = new Filter ();
        Log.addFilter(filter);

        Logger.setLogLevel(IntTests.class.getName(), Log.DBGDETAIL);
        Log.setPreQueueLevel(Log.WRNDETAIL);
        Log.setFlushBufferSize(5);
        Log.setPostFlushSize(3);
        Log.setFlushTriggerLevel(Log.WRN);
        Log.setQueueing(true);

        log(Log.DBGDETAIL, 10);
        flush();
        assert(filter.count == 0);

        log(Log.WRN, 1);
        flush();
        assert(filter.count == 6);
        assertNotNull(filter.entries);
        assert(filter.entries[filter.entries.length - 1].getMessage()
               .equals("10"));

        log(Log.DBG, 5);
        flush();
        assert(filter.count == 9);
        assertNotNull(filter.entries);
        assert(filter.entries[filter.entries.length - 1].getMessage()
               .equals("13"));

        Log.removeFilter(filter);
        Log.setFlushBufferSize(0);
        Log.setPostFlushSize(0);
    }

    // }}}
    // {{{ testLogLevel

    /**
     * Test the log level.
     */
    public void testLogLevel()
    {
        PrintStream err = System.err;
        ByteArrayOutputStream out = new ByteArrayOutputStream ();
        System.setErr(new PrintStream (out));

        Logger.setLogLevel(IntTests.class.getName(), Log.DBGDETAIL);
        Log.setPreQueueLevel(Log.DBGDETAIL);
        Log.setOutputLevel(Log.DBG);
        Log.setQueueing(true);

        log(Log.DBGDETAIL, 5);
        flush();
        String tmp = out.toString();
        assert(tmp.equals(""));

        log(Log.DBG, 5);
        flush();
        tmp = out.toString();
        assert(tmp.length() > 0);
        assert(tmp.endsWith("9" + LINESEP));

        System.setErr(err);
    }

    // }}}
    // {{{ testStreamLogWriter

    /**
     * Test a stream writer in a LevelFilter.
     * @since    0.98.0
     */
    public void testStreamLogWriter()
    {
        Filter filter = new Filter ();
        Log.addFilter(filter);

        ByteArrayOutputStream out = new ByteArrayOutputStream ();
        StreamLogWriter slw = new StreamLogWriter (new PrintWriter(out), true);
        LevelFilter lf = new LevelFilter (Log.DBG);
        lf.setFilterMode(lf.FILTER_USED);
        lf.setLogWriter(slw);
        Log.addFilter(lf);

        Logger.setLogLevel(IntTests.class.getName(), Log.DBGDETAIL);
        Log.setPreQueueLevel(Log.DBGDETAIL);
        Log.setQueueing(true);

        log(Log.DBGDETAIL, 5);
        flush();
        String tmp = out.toString();
        assert(tmp.equals(""));
        assert(filter.count == 5);

        log(Log.DBG, 5);
        flush();
        tmp = out.toString();
        assert(tmp.length() > 0);
        assert(tmp.endsWith("9" + LINESEP));
        assert(filter.count == 5);

        lf.setFilterMode(lf.FILTER_UNUSED);
        log(Log.DBGDETAIL, 5);
        flush();
        assert(filter.count == 5);
        log(Log.DBG, 5);
        flush();
        assert(filter.count == 10);

        lf.setFilterMode(lf.FILTER_ALL);
        log(Log.DBGDETAIL, 5);
        flush();
        assert(filter.count == 10);
        log(Log.DBG, 5);
        flush();
        assert(filter.count == 10);

        lf.setFilterMode(lf.FILTER_NONE);
        log(Log.DBGDETAIL, 5);
        flush();
        assert(filter.count == 15);
        log(Log.DBG, 5);
        flush();
        assert(filter.count == 20);

        Log.removeFilter(filter);
        Log.removeFilter(lf);
        slw.close();
    }

    // }}}
    // {{{ testStreamLogWriter2

    /**
     * Test a stream writer directly in Log.
     * @since    0.98.0
     */
    public void testStreamLogWriter2()
    {
        Filter filter = new Filter ();
        Log.addFilter(filter);

        ByteArrayOutputStream out = new ByteArrayOutputStream ();
        StreamLogWriter slw = new StreamLogWriter (new PrintWriter (out),
                                                   true);

        LogWriter old = Log.getLogWriter();
        Log.setLogWriter(slw);

        Logger.setLogLevel(IntTests.class.getName(), Log.DBGDETAIL);
        Log.setPreQueueLevel(Log.DBGDETAIL);
        Log.setOutputLevel(Log.DBG);
        Log.setQueueing(true);

        log(Log.DBGDETAIL, 5);
        flush();
        String tmp = out.toString();
        assert(tmp.equals(""));
        assert(filter.count == 5);

        log(Log.DBG, 5);
        flush();
        tmp = out.toString();
        assert(tmp.length() > 0);
        assert(tmp.endsWith("9" + LINESEP));
        assert(filter.count == 10);

        Log.setOutputLevel(Log.DEFAULT_OUTPUT_LEVEL);
        Log.setLogWriter(old);
        Log.removeFilter(filter);
        slw.close();
    }

    // }}}
    // {{{ testFileLogWriter

    /**
     * Test log file creation with a FileLogWriter.
     * @since    0.98.0
     */
    public void testFileLogWriter()
    {
        String logfile = "/tmp/xIntTestx.log";
        String logfile1 = "/tmp/xIntTestx1.log";
        String logfile2 = "/tmp/xIntTestx2.log";

        LogWriter old = Log.getLogWriter();

        StreamLogWriter slw = null;
        try {
            slw = new FileLogWriter("IntTest", logfile,
                                    FileLogWriter.MODE_CREATE, false);
        } catch (IOException ex) {
            assert(false);
        }
        Log.setLogWriter(slw);

        assert(countLines(logfile) == 1);

        Logger.setLogLevel(IntTests.class.getName(), Log.DBGDETAIL);
        Log.setPreQueueLevel(Log.DBGDETAIL);
        Log.setQueueing(true);
        Log.setOutputLevel(Log.DBG);

        log(Log.DBGDETAIL, 5);
        flush();
        assert(countLines(logfile) == 1);

        log(Log.DBG, 5);
        flush();
        assert(countLines(logfile) == 6);

        log(Log.DBG, 5);
        flush();
        assert(countLines(logfile) == 11);

        slw.close();

        try {
            slw = new FileLogWriter("IntTest", logfile,
                                    FileLogWriter.MODE_CREATE, true);
        } catch (IOException ex) {
            assert(false);
        }
        Log.setLogWriter(slw);
        assert(countLines(logfile) == 1);

        log(Log.DBG, 5);
        flush();
        assert(countLines(logfile) == 6);

        log(Log.DBG, 5);
        flush();
        assert(countLines(logfile) == 11);

        slw.close();

        try {
            slw = new FileLogWriter("IntTest", logfile,
                                    FileLogWriter.MODE_APPEND, false);
        } catch (IOException ex) {
            assert(false);
        }
        Log.setLogWriter(slw);
        assert(countLines(logfile) == 11);

        log(Log.DBG, 5);
        flush();
        assert(countLines(logfile) == 16);

        slw.close();

        log(Log.DBG, 5);
        flush();
        assert(countLines(logfile) == 16);

        new File(logfile1).delete();
        try {
            slw = new FileLogWriter("IntTest", logfile,
                                    FileLogWriter.MODE_UNIQUE, true);
        } catch (IOException ex) {
            assert(false);
        }
        Log.setLogWriter(slw);
        assert(countLines(logfile) == 16);
        assert(countLines(logfile1) == 1);

        log(Log.DBG, 5);
        flush();
        assert(countLines(logfile) == 16);
        assert(countLines(logfile1) == 6);

        log(Log.DBG, 5);
        flush();
        assert(countLines(logfile) == 16);
        assert(countLines(logfile1) == 11);

        slw.close();

        log(Log.DBG, 5);
        flush();
        assert(countLines(logfile) == 16);
        assert(countLines(logfile1) == 11);

        new File(logfile2).delete();
        try {
            slw = new FileLogWriter("IntTest", logfile,
                                    FileLogWriter.MODE_UNIQUE, false);
        } catch (IOException ex) {
            assert(false);
        }
        Log.setLogWriter(slw);
        assert(countLines(logfile) == 16);
        assert(countLines(logfile1) == 11);
        assert(countLines(logfile2) == 1);

        log(Log.DBG, 5);
        flush();
        assert(countLines(logfile) == 16);
        assert(countLines(logfile1) == 11);
        assert(countLines(logfile2) == 6);

        slw.close();
        log(Log.DBG, 5);
        flush();
        assert(countLines(logfile) == 16);
        assert(countLines(logfile1) == 11);
        assert(countLines(logfile2) == 6);

        Log.setLogWriter(old);
    }

    // }}}
    // {{{ testRingFileLogWriter

    /**
     * Test log file creation with a RingFileLogWriter.
     * @since    0.98.0
     */
    public void testRingFileLogWriter()
    {
        String logfile = "/tmp/xIntTestx.log";
        String logfile1 = "/tmp/xIntTestx1.log";
        String logfile2 = "/tmp/xIntTestx2.log";
        String logfile3 = "/tmp/xIntTestx3.log";

        new File(logfile).delete();
        new File(logfile1).delete();
        new File(logfile2).delete();
        new File(logfile3).delete();

        LogWriter old = Log.getLogWriter();

        Logger.setLogLevel(IntTests.class.getName(), Log.DBGDETAIL);
        Log.setPreQueueLevel(Log.DBGDETAIL);
        Log.setQueueing(true);
        Log.setOutputLevel(Log.DBG);

        StreamLogWriter slw = null;
        try {
            slw = new RingFileLogWriter("IntTest", logfile, 1, 2);
        } catch (IOException ex) {
            assert(false);
        }
        Log.setLogWriter(slw);

        assert(countLines(logfile) == 1);
        assert (! new File (logfile1).exists());
        assert (! new File (logfile2).exists());

        // 20 messages will just fill a log file

        logger.log(Log.ERR, "mark0", "");
        log(Log.DBG, 10);
        flush();


        log(Log.DBG, 10);
        flush();

        assert (new File (logfile1).exists());
        assert (contains(logfile1, "mark0"));
        assert (! new File (logfile2).exists());

        logger.log(Log.ERR, "mark1", "");
        log(Log.DBG, 20);
        flush();

        assert (new File (logfile1).exists());
        assert (contains(logfile1, "mark1"));
        assert (new File (logfile2).exists());
        assert (contains(logfile2, "mark0"));

        logger.log(Log.ERR, "mark2", "");
        log(Log.DBG, 20);
        flush();

        assert (new File (logfile1).exists());
        assert (contains(logfile1, "mark2"));
        assert (new File (logfile2).exists());
        assert (contains(logfile2, "mark1"));
        assert (! new File (logfile3).exists());

        Log.setLogWriter(old);
    }

    // }}}
    // {{{ testLogStream

    /**
     * Test a log stream.
     */
    public void testLogStream()
    {
        Filter filter = new Filter ();
        Log.addFilter(filter);

        PrintStream out = new PrintStream
            (new LogStream (Log.DBG, "clazz", "method"));

        Log.setPreQueueLevel(Log.DBGDETAIL);
        Log.setQueueing(true);

        out.print("bla");
        flush();
        assert(filter.count == 0);

        out.flush();
        flush();
        assert(filter.count == 1);

        out.println("bla");
        flush();
        assert(filter.count == 2);

        Log.setPreQueueLevel(Log.WRN);
        out.println("bla");
        flush();
        assert(filter.count == 2);

        Log.removeFilter(filter);
    }

    // }}}
    // {{{ testExceptions

    /**
     * Test exception logging.
     */
    public void testExceptions()
    {
        Filter filter = new Filter ();
        Log.addFilter(filter);

        Logger.setLogLevel(IntTests.class.getName(), Log.DBG);
        Log.setPreQueueLevel(Log.DBG);
        Log.setOutputLevel(0);

        try {
            throw new Exception ("test");
        } catch (Exception ex) {
            logger.log("testExceptions()", ex);
        }

        flush();
        assert(filter.count == 2);
        assertNotNull(filter.entries);
        assert(filter.entries[filter.entries.length - 1].getLevel() ==
               Log.ERRDETAIL);

        try {
            throw new Exception ("test");
        } catch (Exception ex) {
            logger.log(Log.DBG, "testExceptions()", ex);
        }

        flush();
        assert(filter.count == 3);
        assert(filter.entries[filter.entries.length - 1].getLevel() ==
               Log.DBG);

        logger.dumpStack(Log.WRN, "testExceptions()", "test");

        flush();
        assert(filter.count == 5);
        assert(filter.entries[filter.entries.length - 1].getLevel() ==
               Log.WRNDETAIL);

        Log.removeFilter(filter);
    }

    // }}}

    //----------------------------------------------------------------------
    // Helpers
    //----------------------------------------------------------------------
    // {{{ log

    /**
     * Log some messages.
     *
     * @param   level   Level to log at.
     * @param   count   Number of entries to log.
     */
    private void log(int level, int count)
    {
        for (int i = 0; i < count; i++) {
            logger.log(level, "log", "" + (id + i));
        }
        id += count;
    }

    // }}}
    // {{{ flush

    /**
     * Flush the log queue.
     */
    private void flush()
    {
        if (Log.isQueueing()) {
            Log.setQueueing(false);
            Log.setQueueing(true);
        }
    }

    // }}}
    // {{{ countLines

    /**
     * Count the number of lines in a file, used to test logging to a file.
     *
     * @param   filename        The name of the file to count the lines of.
     *
     * @return  The number of lines in the file. -1 if the file could not be
     *          opened.
     */
    private int countLines(String filename)
    {
        try {
            BufferedReader in = new BufferedReader (new FileReader (filename));
            int num;
            for (num = 0; in.readLine() != null; num++);
            return num;
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
            return -1;
        } catch (IOException ex) {
            ex.printStackTrace();
            return -1;
        }
    }

    // }}}
    // {{{ contains

    /**
     * Test whether a file contains a String.
     *
     * @param   file    The file to read.
     * @param   search  The String to search for. May not cross lines.
     *
     * @return  True if the file contains the string.
     */
    private boolean contains(String file, String search)
    {
        try {
            BufferedReader in = new BufferedReader (new FileReader (file));
            String line = in.readLine();
            while (line != null) {
                if (line.indexOf(search) >= 0) {
                    return true;
                }
                line = in.readLine();
            }
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        return false;
    }

    // }}}

    // {{{ class Filter

    /**
     * Helper class to test LogFilters.
     */
    private class Filter
        implements LogFilter
    {
        // {{{ variables

        /**
         * Number of entries filtered.
         */
        public int count;

        /**
         * The last set of entries filtered.
         */
        public LogEntry[] entries;

        // }}}

        // {{{ filter

        /**
         * Decide whether a message should be logged.
         *
         * @param logEntry  The entry for the log message.
         *
         * @return  True, if the message should be passed on, false if it
         *          should be blocked.
         */
        public boolean filter(LogEntry logEntry)
        {
            //  System.err.println("Got single");
            count++;
            entries = new LogEntry[] {logEntry};
            return true;
        }

        // }}}
        // {{{ filter

        /**
         * Decide for each message in an array whether it should be logged.
         *
         * @param       entries The list of LogEntries.
         *
         * @return  An array containing the entries that were not filtered out.
         */
        public LogEntry[] filter(LogEntry[] entries)
        {
            //  System.err.println("Got " + entries.length);
            count += entries.length;
            this.entries = entries;
            return entries;
        }

        // }}}
    }

    // }}}
}

