// {{{ copyright

/********************************************************************
 *
 * $Id: RemoteLogLevelCallbackImplBase.java,v 1.2 2000/10/26 15:04:09 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.logrmi;

// {{{ imports

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

// }}}

/**
 * If a class wants to implement the {@link RemoteLogLevelCallback
 * RemoteLogLevelCallback} interface it should do so by extending this
 * abstract class instead of {@link UnicastRemoteObject
 * UnicastRemoteObject}. That way it doesn't have to care about the RMI stubs,
 * since they are part of the de.qfs.lib.logrmi package. Otherwise it will
 * have to make the stubs available itself.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.2 $
 */
public abstract class RemoteLogLevelCallbackImplBase
    extends UnicastRemoteObject
    implements RemoteLogLevelCallback
{
    // {{{ constructor

    /**
     * Create a new RemoteLogLevelCallbackImplBase.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public RemoteLogLevelCallbackImplBase ()
        throws RemoteException
    {
    }

    // }}}

    // {{{ getLogLevels

    /**
     * Get the current log levels from the callback. This method is needed to
     * synchronize the LogLevelListener and the RemoteLogLevelCallback after
     * the connection has been established.
     *
     * @return  An object array that contains an alternating sequence of
     *          class/package names and log levels. The levels are Integers
     *          that may be null for classes for which a level has been
     *          requested but not explicitly set.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public abstract Object[] getLogLevels()
        throws RemoteException;

    // }}}
    // {{{ setLogLevel

    /**
     * Callback method for a LogLevelListener to change the log level for
     * a class or package.
     *
     * @param   name    The name of the affected class or package.
     * @param   level   The new log level.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public abstract void setLogLevel(String name, int level)
        throws RemoteException;

    // }}}
    // {{{ removeLogLevel

    /**
     * Callback method for a LogLevelListener to remove the log level for
     * a class or package.
     *
     * @param   name    The name of the affected class or package.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public abstract void removeLogLevel(String name)
        throws RemoteException;

    // }}}
    // {{{ getOutputLevel

    /**
     * Get the current output log level from the callback's application.
     *
     * @return  The current output log level.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public abstract int getOutputLevel()
        throws RemoteException;

    // }}}
    // {{{ setOutputLevel

    /**
     * Set the output log level for the callback's application.
     *
     * @param level     Output log level to set.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public abstract void setOutputLevel(int level)
        throws RemoteException;

    // }}}
    // {{{ getPreQueueLevel

    /**
     * Return the current pre-queue level from the callback's application.
     *
     * @return  The current pre-queue level.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public abstract int getPreQueueLevel()
        throws RemoteException;

    // }}}
    // {{{ setPreQueueLevel

    /**
     * Set the pre-queue level for the callback's application.
     *
     * @param level     The pre-queue level to set.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public abstract void setPreQueueLevel(int level)
        throws RemoteException;

    // }}}
    // {{{ isQueueing

    /**
     * Check whether the callback's application is using a log queue.
     *
     * @return  Whether the callback's application is queueing log entries.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     *
     * @see     de.qfs.lib.log.Log#isQueueing
     */
    public abstract boolean isQueueing()
        throws RemoteException;

    // }}}
    // {{{ setQueueing

    /**
     * Set whether the callback's application should use a log queue.
     *
     * @param   queue   Whether to queue log entries.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     *
     * @see     de.qfs.lib.log.Log#setQueueing
     */
    public abstract void setQueueing(boolean queue)
        throws RemoteException;

    // }}}
    // {{{ getQueueSize

    /**
     * Get the size of log queue of the callback's application.
     *
     * @return  The size of the log queue.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     *
     * @see     de.qfs.lib.log.Log#getQueueSize
     */
    public abstract int getQueueSize()
        throws RemoteException;

    // }}}
    // {{{ setQueueSize

    /**
     * Set the size of log queue of the callback's application.
     *
     * @param   size    The size of the log queue.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     *
     * @see     de.qfs.lib.log.Log#setQueueSize
     */
    public abstract void setQueueSize(int size)
        throws RemoteException;

    // }}}
    // {{{ isDropOnOverflow

    /**
     * Check whether the callback's application is dropping entries when
     * its log queue overflows.
     *
     * @return  Whether entries are dropped on overflow.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     *
     * @see     de.qfs.lib.log.Log#isDropOnOverflow
     */
    public abstract boolean isDropOnOverflow()
        throws RemoteException;

    // }}}
    // {{{ setDropOnOverflow

    /**
     * Set whether the callback's application should drop entries when
     * its log queue overflows.
     *
     * @param   drop    Whether to drop entries on overflow.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     *
     * @see     de.qfs.lib.log.Log#setDropOnOverflow
     */
    public abstract void setDropOnOverflow(boolean drop)
        throws RemoteException;

    // }}}
    // {{{ getFlushBufferSize

    /**
     * Get the size of the flush buffer of the callback's application.
     *
     * @return  The flush buffer size or 0 to indicate no flush buffer.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public abstract int getFlushBufferSize()
        throws RemoteException;

    // }}}
    // {{{ setFlushBufferSize

    /**
     * Set the size of the flush buffer of the callback's application. Setting
     * it to 0 will turn the flush buffer off.
     *
     * @param   size    The size of the flush buffer.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public abstract void setFlushBufferSize(int size)
        throws RemoteException;

    // }}}
    // {{{ getFlushTriggerLevel

    /**
     * Get the level that will trigger a flush of the messages saved in the
     * flush buffer of the callback's application.
     *
     * @return  The flush buffer's trigger level.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public abstract int getFlushTriggerLevel()
        throws RemoteException;

    // }}}
    // {{{ setFlushTriggerLevel

    /**
     * Set the level that will trigger a flush of the messages saved in the
     * flush buffer of the callback's application.
     *
     * @param   level   The trigger level to set.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public abstract void setFlushTriggerLevel(int level)
        throws RemoteException;

    // }}}
    // {{{ getPostFlushSize

    /**
     * Get the number of messages to pass unfiltered through the pre-queue
     * stage after a flush happened in the callback's application.
     *
     * @return  The number of messages to pass.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public abstract int getPostFlushSize()
        throws RemoteException;

    // }}}
    // {{{ setPostFlushSize

    /**
     * Set the number of messages to pass unfiltered through the pre-queue
     * stage after a flush happened in the callback's application.
     *
     * @param   size    The number of messages to pass.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public abstract void setPostFlushSize(int size)
        throws RemoteException;

    // }}}
    // {{{ disconnect

    /**
     * End the connection with the RemoteLogLevelCallback.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public abstract void disconnect()
         throws RemoteException;

    // }}}
}
