// {{{ copyright

/********************************************************************
 *
 * $Id: RemoteLogLevelListenerImplBase.java,v 1.1 2000/07/04 13:43:51 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.logrmi;

// {{{ imports

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import de.qfs.lib.log.LogLevelEvent;

// }}}

/**
 * If a class wants to implement the {@link RemoteLogLevelListener
 * RemoteLogLevelListener} interface it should do so by extending this
 * abstract class instead of {@link UnicastRemoteObject
 * UnicastRemoteObject}. That way it doesn't have to care about the RMI stubs,
 * since they are part of the de.qfs.lib.logrmi package. Otherwise it will
 * have to make the stubs available itself.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.1 $
 */
public abstract class RemoteLogLevelListenerImplBase
    extends UnicastRemoteObject
    implements RemoteLogLevelListener
{
    // {{{ constructor

    /**
     * Create a new RemoteLogLevelListenerImplBase.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public RemoteLogLevelListenerImplBase ()
         throws RemoteException
    {
    }

    // }}}

    // {{{ classAdded

    /**
     * Notify the listener that a Logger for a formerly unknown class has
     * requested its log level, adding the class to the log level tree.
     *
     * @param   event   The LogLevelEvent holding the details.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public abstract void classAdded(LogLevelEvent event)
        throws RemoteException;

    // }}}
    // {{{ levelChanged

    /**
     * Notify the listener that a log level has been changed.
     *
     * @param   event   The LogLevelEvent holding the details.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public abstract void levelChanged(LogLevelEvent event)
        throws RemoteException;

    // }}}
    // {{{ levelRemoved

    /**
     * Notify the listener that a log level has been unset.
     *
     * @param   event   The LogLevelEvent holding the details.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public abstract void levelRemoved(LogLevelEvent event)
        throws RemoteException;

    // }}}
    // {{{ setLogLevelCallback

    /**
     * Sets the RemoteLogLevelCallback for the RemoteLogLevelListener.
     *
     * @param   callback        The LogLevelCallback to use to change the
     *                          log levels.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public abstract void setLogLevelCallback(RemoteLogLevelCallback callback)
        throws RemoteException;

    // }}}
    // {{{ disconnect

    /**
     * End the connection with the RemoteLogLevelListener.
     *
     * @throws  RemoteException If something RMI specific goes wrong.
     */
    public abstract void disconnect()
         throws RemoteException;

    // }}}
}
