// {{{ copyright

/********************************************************************
 *
 * $Id: BooleanOptionEdit.java,v 1.11 2001/04/17 16:41:03 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.option;

// {{{ imports

import java.awt.BorderLayout;

import javax.swing.JCheckBox;

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

// {{{ doc

/**
 * An OptionEdit for a {@link BooleanOption BooleanOption}. It uses the
 * following resources: <p>
 *
 * <TABLE BORDER="1" CELLPADDING="3" CELLSPACING="0" WIDTH="100%">
 * <TR>
 * <TH WIDTH="40%">Resource name</TH>
 * <TH>Usage</TH>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>option.<em>name</em>.label</code></TD>
 * <TD>The label for the Option (optional, default empty).</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>option.<em>name</em>.mnemonic</code></TD>
 * <TD>Keyboard accelerator (optional, default none).</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>option.<em>name</em>.disabled</code></TD>
 * <TD>If true, the edit will be disabled and remain so even if {@link
 * #setEnabled setEnabled(true)} is called (optional, default false).</TD>
 * </TR>
 * </TABLE>
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.11 $
 */

// }}}
public class BooleanOptionEdit
    extends OptionEdit
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger = new Logger (BooleanOptionEdit.class);

    /**
     * The CheckBox Component of the BooleanOptionEdit.
     */
    protected JCheckBox check;

    // }}}

    //----------------------------------------------------------------------
    // Constructor
    //----------------------------------------------------------------------
    // {{{ BooleanOptionEdit

    /**
     * Create a new BooleanOptionEdit. Don't call this directly, use {@link
     * OptionEdit#makeEdit OptionEdit.makeEdit} instead.
     */
    public BooleanOptionEdit ()
    {
    }

    // }}}

    //----------------------------------------------------------------------
    // Public methods
    //----------------------------------------------------------------------
    // {{{ init

    /**
     * Initialize the OptionEdit. Called from {@link #makeEdit makeEdit}.
     */
    public void init()
    {
        setLayout(new BorderLayout ());
        check = new JCheckBox
            (Option.getResources().getString
             ("option." + option.getName() + ".label",
              "Missing: " + option.getName()));
        check.setName("CheckBox-" + option.getName());
        String mnemo = getString("mnemonic", null);
        if (mnemo != null && mnemo.length() > 0) {
            check.setMnemonic(mnemo.charAt(0));
        }
        add (check, BorderLayout.WEST);
        Boolean val = (Boolean) option.getValue();
        if (val != null) {
            check.setSelected(val.booleanValue());
        }
        disabled = getBoolean("disabled", false);
        if (disabled) {
            setEnabled(false);
        }
    }

    // }}}
    // {{{ checkValue

    /**
     * Check the currently entered value of the OptionEdit. If the value is
     * not valid, request the focus.
     *
     * @return  True if the value is valid, false otherwise.
     */
    public boolean checkValue()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "checkValue()", "");
        }
        Boolean val = new Boolean (check.isSelected());
        if (! option.checkValue(this, val)) {
            check.requestFocus();
            return false;
        }
        return true;
    }

    // }}}
    // {{{ readValue

    /**
     * Read the currently entered value and store it in the Option.
     */
    public void readValue()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "readValue()", "");
        }
        Boolean val = new Boolean (check.isSelected());
        option.setValue(val);
    }

    // }}}
    // {{{ update

    /**
     * Update the display for a possibly new Option.
     *
     * @param   option The new option.
     */
    public void update(Option option)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "update(Option)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "option: " + option);
        }
        this.option = option;
        Boolean val = (Boolean) option.getValue();
        if (val != null) {
            check.setSelected(val.booleanValue());
        }
    }

    // }}}

    //----------------------------------------------------------------------
    // Override some methods from JPanel
    //----------------------------------------------------------------------
    // {{{ setEnabled

    /**
     * Enable or diesable the BooleanOptionEdit.
     *
     * @param   enabled True to enable, false to disable.
     */
    public void setEnabled(boolean enabled)
    {
        if (disabled) {
            enabled = false;
        }
        check.setEnabled(enabled);
    }

    // }}}
    // {{{ isEnabled

    /**
     * Query whether the BooleanOptionEdit is enabled.
     *
     * @return  True if it is enabled.
     */
    public boolean isEnabled()
    {
        return check.isEnabled();
    }

    // }}}
    // {{{ requestFocus

    /**
     * Request the focus for the BooleanOptionEdit.
     */
    public void requestFocus()
    {
        check.requestFocus();
    }

    // }}}
}
