// {{{ copyright

/********************************************************************
 *
 * $Id: ChoiceOptionGroup.java,v 1.5 2000/11/29 14:38:13 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.option;

// {{{ imports

import java.awt.Component;

import java.util.Vector;

//  import javax.swing.JPanel;
import javax.swing.JComboBox;

import de.qfs.lib.gui.Grid;
import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;
import de.qfs.lib.util.MapResourceBundle;

// }}}

/**
 * A ChoiceOptionGroup is represented by a JComboBox representing one Option.
 * The Option should be an {@link IntegerOption IntegerOption}, its value
 * between 1 and [number of choices]. <p>
 *
 * The labels of the choices are defined by the resources set with {@link
 * Option#setResources Option.setResources}. See the package documentation
 * for details about the necessary resources. <p>
 *
 * To build a ChoiceOptionGroup with a dynamic set of choices created at
 * runtime, create a custom OptionGroup derived from ChoiceOptionGroup and
 * override {@link #getChoices getChoices}.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.5 $
 * @deprecated  Removed in version 0.98.0. This class was experimental and
 *              should never have made it into a release. Please use {@link
 *              ChoiceOption ChoiceOptions} and {@link ChoiceOptionEdit
 *              ChoiceOptionEdits} instead. ChoiceOptionGroup will be removed
 *              in the next version.
 */
public class ChoiceOptionGroup extends OptionGroup
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger = new Logger (ChoiceOptionGroup.class);

    /**
     * The possible choices.
     */
    protected Object[] choices;

    /**
     * The JComboBox for the ChoiceOptionGroup.
     */
    protected JComboBox combo;

    /**
     * The Option that holds the value for the selected choice.
     */
    protected Option option;

    // }}}

    // {{{ constructors

    /**
     * No arg constructor for specialized OptionGroups.
     */
    protected ChoiceOptionGroup ()
    {
    }

    /**
     * Create a new ChoiceOptionGroup.
     *
     * @param   name    The name of the group.
     * @param   options The Options to edit.
     */
    public ChoiceOptionGroup (String name, OptionSet options)
    {
        super (name, options);
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "ChoiceOptionGroup(String,OptionSet)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "name: " + name);
        }
        if (logger.level >= Log.WRN) {
            logger.log(Log.WRN, "FramedOptionGroup(String,OptionSet)",
                       "Warning: ChoiceOptionGroups have been\n " +
                       "deprecated as of version 0.98.0. " +
                       "Please replace the Option for the group named '" +
                       name + "' with a ChoiceOption and use its edit " +
                       "instead of the group.");
        }
        getChoices();
    }

    // }}}

    // {{{ checkValues

    /**
     * Check the values.
     *
     * @return  True if the selected choice is accepted by the Option.
     */
    public boolean checkValues()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "checkValues()", "");
        }
        int idx = combo.getSelectedIndex();
        if (! option.checkValue(combo, new Integer(idx + 1))) {
            combo.requestFocus();
            return false;
        }
        return true;
    }

    // }}}
    // {{{ readValues

    /**
     * Store the value for the selected Choice in the Option.
     */
    public void readValues()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "readValues()", "");
        }
        int idx = combo.getSelectedIndex();
        option.setValue(new Integer (idx + 1));
    }

    // }}}
    // {{{ notifyOptionObservers

    /**
     * Notify the option's Observers.
     */
    public void notifyOptionObservers()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "notifyOptionObservers()", "");
        }
        option.notifyObservers();
    }

    // }}}
    // {{{ update

    /**
     * Update display for possibly new Options. The name and type of
     * the Options in the OptionSet must be the same as before.
     *
     * @param   options The new OptionSet.
     */
    public void update(OptionSet options)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "update(OptionSet)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "options: " + options +
                       " this: " + this);
        }
        this.options = options;
        option = options.getOption(option.getName());
        if (option == null) {
            throw new NullPointerException();
        }
        int current = ((Integer) option.getValue()).intValue();
        if (current >= 1 && current <= choices.length ) {
            combo.setSelectedIndex(current - 1);
        }
    }

    // }}}
    // {{{ setEnabled

    /**
     * Set the enabled state for the buttons.
     *
     * @param   enabled The enabled state to set.
     */
    public void setEnabled (boolean enabled)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "setEnabled(boolean)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "enabled: " + enabled +
                       ", disabled: " + disabled);
        }

        if (disabled) {
            enabled = false;
        }
        combo.setEnabled(enabled);
    }

    // }}}
    // {{{ getComponent

    /**
     * Get the AWT Component for the OptionGroup.
     *
     * @return  The Component.
     */
    public Component getComponent()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getComponent()", "");
        }
        if (combo == null) {
            combo = new JComboBox (choices);
            combo.setName("OptionGroup-" + name + "-" +
                          options.getConfigurableName());
            int current = ((Integer) option.getValue()).intValue();
            if (current >= 1 && current <= choices.length ) {
                combo.setSelectedIndex(current - 1);
            }
        }
        if (disabled) {
            combo.setEnabled(false);
        }
        return combo;
    }

    // }}}

    // {{{ getChoices

    /**
     * Create the possible choices from the resources. Also set the option
     * according to the <code>option.group.<em>groupname</em>.option</code>
     * resource.
     */
    protected void getChoices()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getChoices()", "");
        }
        MapResourceBundle rb = Option.getResources();
        Vector choices = new Vector();

        String numchoices =
            rb.getString("option.group." + name + ".numchoices", null);
        if (numchoices == null) {
            if (logger.level >= Log.ERR) {
                logger.log(Log.ERR, "getChoices()",
                           "No numchoices for ChoiceOptionGroup " + name);
                return;
            }
        }
        int num = Integer.parseInt(numchoices);

        String optionname =
            rb.getString("option.group." + name + ".option", null);
        if (optionname == null) {
            if (logger.level >= Log.ERR) {
                logger.log(Log.ERR, "getChoices()",
                           "No option for ChoiceOptionGroup " + name);
                return;
            }
        }
        option = options.getOption(optionname);
        if (option == null) {
            if (logger.level >= Log.ERR) {
                logger.log(Log.ERR, "getChoices()",
                           "No option for ChoiceOptionGroup " + name);
                return;
            }
        }

        for (int i = 1; i <= num; i++) {
            String choice = rb.getString("option.group." + name + ".choice" +
                                        i + ".label", null);
            if (choice == null) {
                if (logger.level >= Log.ERR) {
                    logger.log(Log.ERR, "getTabs()",
                               "No label for choice " + i +
                               " in ChoiceOptionGroup " + name);
                }
            } else {
                choices.addElement(choice);
            }
        }

        this.choices = new Object [choices.size()];
        choices.copyInto(this.choices);
    }

    // }}}
}
