// {{{ copyright

/********************************************************************
 *
 * $Id: IntegerOption.java,v 1.7 2000/06/06 16:41:27 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.option;

// {{{ imports

import java.awt.Component;

import de.qfs.lib.gui.Message;
import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

/**
 * An IntegerOption has a value of class Integer and an additional flag
 * defining whether negative values are OK.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.7 $
 */
public class IntegerOption
    extends Option
    implements ConfigurableOption
{
    // {{{ variables

    /**
     * The UID used for serialization.
     */
    final static long serialVersionUID = 8271881954001543763L;

    /**
     * The Logger used for logging.
     */
    private final static Logger logger = new Logger (IntegerOption.class);

    /**
     * Whether negative values are allowed.
     *
     * @serial
     */
    private boolean negativeOK;

    // }}}

    // {{{ constructors

    /**
     * Create a new IntegerOption.
     */
    public IntegerOption ()
    {
    }

    /**
     * Create a new IntegerOption with a name and the value null.
     *
     * @param   name    The name to use.
     */
    public IntegerOption (String name)
    {
        super (name, null);
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "IntegerOption(String,int)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "name: " + name);
        }
    }

    /**
     * Create a new IntegerOption with a name and a value.
     *
     * @param   name    The name to use.
     * @param   value   The value to use.
     */
    public IntegerOption (String name, int value)
    {
        super (name, new Integer (value));
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "IntegerOption(String,int)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "name: " + name + ", " +
                       "value: " + value);
        }
    }

    /**
     * Create a new IntegerOption with a name and a value and the nullOK and
     * negativeOK flags.
     *
     * @param   name    The name to use.
     * @param   value   The value to use.
     * @param   nullOK  The nullOK flag to use.
     * @param   negativeOK  Whether negative values are allowed.
     */
    public IntegerOption (String name, int value, boolean nullOK,
                          boolean negativeOK)
    {
        this (name, new Integer (value), nullOK, negativeOK);
    }

    /**
     * Create a new IntegerOption with a name and a value and the nullOK and
     * negativeOK flags.
     *
     * @param   name    The name to use.
     * @param   value   The value to use.
     * @param   nullOK  The nullOK flag to use.
     * @param   negativeOK  Whether negative values are allowed.
     */
    public IntegerOption (String name, Integer value, boolean nullOK,
                          boolean negativeOK)
    {
        super (name, value, nullOK);
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "IntegerOption(String,int,boolean,boolean)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "name: " + name + ", " +
                       "value: " + value + ", " +
                       "nullOK: " + nullOK + ", " +
                       "negativeOK: " + negativeOK);
        }
        this.negativeOK = negativeOK;
    }

    // }}}

    //----------------------------------------------------------------------
    // The negativeOK flag
    //----------------------------------------------------------------------
    // {{{ isNegativeOK

    /**
     * Query wether negative values are allowed.
     *
     * @return  True if negative values are OK.
     */
    public final boolean isNegativeOK()
    {
        return negativeOK;
    }

    // }}}
    // {{{ setNegativeOK

    /**
     * Set wether negative values are allowed.
     *
     * @param   negativeOK  Whether negative values are allowed.
     */
    public final void setNegativeOK(boolean negativeOK)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "setNegativeOK(boolean)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "negativeOK: " + negativeOK);
        }
        this.negativeOK = negativeOK;
    }

    // }}}

    //----------------------------------------------------------------------
    // Checking
    //----------------------------------------------------------------------
    // {{{ checkValue

    /**
     * Check, whether the Option accepts a value. If it does not, the reason
     * should be explained in a message dialog unless component is null.
     *
     * @param   component       The Component in which the Option is being
     *                          edited.
     * @param   value           The value to check.
     *
     * @return  True, if the value is acceptable.
     */
    public boolean checkValue(Component component, Object value)
    {
        if (value != null
            && !negativeOK
            && ((Integer) value).intValue() < 0) {
            if (component != null) {
                Message.showMessage(component, "option.error.negative");
            }
            return false;
        }
        return super.checkValue(component, value);
    }

    // }}}

    //----------------------------------------------------------------------
    // Standard methods
    //----------------------------------------------------------------------
    // {{{ equals

    /**
     * Two IntegerOptions are considered equal if their names and values
     * are equal.
     *
     * @param   object  The object to compare to.
     *
     * @return  True if objects are equal
     */
    public boolean equals(Object object)
    {
        return (object instanceof IntegerOption) && super.equals(object);
    }

    // }}}

    //----------------------------------------------------------------------
    // the ConfigurableOption interface
    //----------------------------------------------------------------------
    // {{{ getConfigurableValue

    /**
     * Get the Option's value as a String.
     *
     * @return  The Option's value as a String.
     */
    public String getConfigurableValue()
    {
        Object value = getValue();
        return value == null ? "" : value.toString();
    }

    // }}}
    // {{{ setConfigurableValue

    /**
     * Set the Option's value from a String.
     *
     * @param   value   The value to set.
     */
    public void setConfigurableValue(String value)
    {
        if (value.equals("")) {
            setValue(null);
        } else {
            try {
                setValue(new Integer (value));
            } catch (NumberFormatException ex) {
                logger.log("setConfigurableValue(String)", ex);
            }
        }
    }

    // }}}
}
