// {{{ copyright

/********************************************************************
 *
 * $Id: KeyOption.java,v 1.3 2001/04/24 16:06:49 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 2000 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.option;

// {{{ imports

import java.io.Serializable;

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

/**
 * This option holds the specific data of a {@link java.awt.event.KeyEvent
 * KeyEvent}.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.3 $
 * @since       0.98.1
 */
public class KeyOption
    extends Option
    implements ConfigurableOption
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger = new Logger (KeyOption.class);

    // }}}

    //----------------------------------------------------------------------
    // Constructor
    //----------------------------------------------------------------------
    // {{{ KeyOption(String,Data)

    /**
     * Create a new KeyOption.
     *
     * @param   name    The name of the option.
     * @param   data    The initial value.
     */
    public KeyOption (String name, Data data)
    {
        super (name, data);
    }

    // }}}
    // {{{ KeyOption(String,int,char,int)

    /**
     * Create a new KeyOption.
     *
     * @param   name      The name of the option.
     * @param   keyCode   The initial keyCode value.
     * @param   keyChar   The initial keyChar value.
     * @param   modifiers The initial modifiers value.
     */
    public KeyOption (String name, int keyCode, char keyChar, int modifiers)
    {
        this (name, new Data (keyCode, keyChar, modifiers));
    }

    // }}}

    //----------------------------------------------------------------------
    // Standard methods
    //----------------------------------------------------------------------
    // {{{ equals

    /**
     * Two KeyOptions are considered equal if their names and values
     * are equal.
     *
     * @param   object  The object to compare to.
     *
     * @return  True if objects are equal
     */
    public boolean equals(Object object)
    {
        return (object instanceof KeyOption) && super.equals(object);
    }

    // }}}

    //----------------------------------------------------------------------
    // The ConfigurableOption interface
    //----------------------------------------------------------------------
    // {{{ getConfigurableValue

    /**
     * Get the Option's value as a String. A null value is represented
     * as the empty String, otherwise the result is of the form
     * "keyCode,keyChar,modifiers".
     *
     * @return  The Option's value as a String.
     */
    public String getConfigurableValue()
    {
        if (getValue() instanceof Data) {
            Data data = (Data) getValue();
            return "" + data.keyCode + "," + (int) data.keyChar +
                "," + data.modifiers;
        }
        return "";
    }

    // }}}
    // {{{ setConfigurableValue

    /**
     * Set the Option's value from a String.
     *
     * @param   value   The value to set. Guaranteed to be non-null.
     */
    public void setConfigurableValue(String value)
    {
        if (value.equals("")) {
            setValue(null);
        } else {
            int idx1 = value.indexOf(',');
            int idx2 = value.indexOf(',', idx1 + 1);
            if (idx1 < 0 || idx2 < 0) {
                if (logger.level >= Log.ERR) {
                    logger.log(Log.ERR, "setConfigurableValue(String)",
                               "Bad value: " + value);
                }
                return;
            }
            try {
                setValue(new Data
                         (Integer.parseInt(value.substring(0, idx1)),
                          (char) Integer.parseInt
                                 (value.substring(idx1 + 1, idx2)),
                          Integer.parseInt(value.substring(idx2 + 1))));
            } catch (NumberFormatException ex) {
                logger.log("setConfigurableValue(String)", ex);
            }
        }
    }

    // }}}

    //----------------------------------------------------------------------
    // Helper class
    //----------------------------------------------------------------------
    // {{{ class Data

    /**
     * Struct that contains the data for the KeyOption.
     */
    public static class Data
        implements Cloneable, Serializable
    {
        // {{{ variables

        /**
         * The keyCode value.
         */
        public int keyCode;

        /**
         * The keyChar value.
         */
        public char keyChar;

        /**
         * The modifiers value.
         */
        public int modifiers;

        // }}}

        //------------------------------------------------------------------
        // Constructor
        //------------------------------------------------------------------
        // {{{ Data

        /**
         * Create a new Data.
         *
         * @param   keyCode   The initial keyCode value.
         * @param   keyChar   The initial keyChar value.
         * @param   modifiers The initial modifiers value.
         */
        public Data (int keyCode, char keyChar, int modifiers)
        {
            this.keyCode = keyCode;
            this.keyChar = keyChar;
            this.modifiers = modifiers;
        }

        // }}}

        //------------------------------------------------------------------
        // Standard methods
        //------------------------------------------------------------------
        // {{{ clone

        /**
         * Create a copy of the Data.
         *
         * @return      The copy.
         */
        public Object clone()
        {
            try {
                return super.clone();
            } catch (CloneNotSupportedException ex) {
                // can't happen
                return null;
            }
        }

        // }}}
        // {{{ hashCode

        /**
         * Get the hash code for the Data.
         *
         * @return      The hash code for the data.
         */
        public int hashCode()
        {
            // not too unique, but probably OK for most cases
            return modifiers << 24 + keyChar << 8 + keyCode;
        }

        // }}}
        // {{{ equals

        /**
         * Test whether an Object is equal to the Data.
         *
         * @param       o       The Object to test.
         *
         * @return True if the Object is of class Data and has the same
         *         values.
         */
        public boolean equals(Object o)
        {
            if (! (o instanceof Data)) {
                return false;
            }
            Data d = (Data) o;
            return d.keyCode == keyCode
                && d.keyChar == keyChar
                && d.modifiers == modifiers;
        }

        // }}}
        // {{{ toString

        /**
         * Get a String representation for the data.
         *
         * @return      A String representation for the data.
         */
        public String toString()
        {
            return super.toString() + "[" +
                "keyCode=" + keyCode +
                ",keyChar=" + (int) keyChar +
                ",modifiers=" + modifiers + "]";
        }

        // }}}
    }

    // }}}
}
