// {{{ copyright

/********************************************************************
 *
 * $Id: KeyOptionEdit.java,v 1.4 2001/04/17 16:41:06 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 2000 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.option;

// {{{ imports

import java.awt.BorderLayout;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.qfs.lib.gui.Borders;
import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

// {{{ doc

/**
 * Edit for a {@link KeyOption KeyOption}, built from a JLabel and a custom
 * component. It uses the following resources: <p>
 *
 * <TABLE BORDER="1" CELLPADDING="3" CELLSPACING="0" WIDTH="100%">
 * <TR>
 * <TH WIDTH="40%">Resource name</TH>
 * <TH>Usage</TH>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>option.<em>name</em>.label</code></TD>
 * <TD>The label for the Option (optional, default empty).</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>option.<em>name</em>.internal</code></TD>
 * <TD>If true, place the label inside the component (optional, default
 * false).</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>option.<em>name</em>.mnemonic</code></TD>
 * <TD>Keyboard accelerator (optional, default none).</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>option.<em>name</em>.disabled</code></TD>
 * <TD>If true, the edit will be disabled and remain so even if {@link
 * #setEnabled setEnabled(true)} is called (optional, default false).</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>option.<em>name</em>.capturetab</code></TD>
 * <TD>If true the tab key is accepted as input and cannot be used to
 * focus the next component (optional, default false).</TD>
 * </TR>
 * </TABLE>
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.4 $
 * @since       0.98.1
 */

// }}}
public class KeyOptionEdit
    extends OptionEdit
    implements KeyListener
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger = new Logger (KeyOptionEdit.class);

    /**
     * The label of the component.
     */
    protected JLabel label;

    protected String ilabel;

    /**
     * The custom component.
     */
    protected KeyLabel key;

    /**
     * The current keyCode value.
     */
    protected int keyCode;

    /**
     * The current keyChar value.
     */
    protected char keyChar = 65535;

    /**
     * The current modifiers value.
     */
    protected int modifiers;

    /**
     * The registered ChangeListeners.
     */
    protected Vector listeners = new Vector ();

    // }}}

    // {{{ constructor

    /**
     * Create a new KeyOptionEdit. Don't call this directly, use
     * {@link OptionEdit#makeEdit OptionEdit.makeEdit}.
     */
    public KeyOptionEdit()
    {
    }

    // }}}

    // {{{ init

    /**
     * Initialize the OptionEdit. Called from {@link #makeEdit makeEdit}.
     */
    public void init()
    {
        setLayout(new BorderLayout ());
        ilabel = getString("label", "");
        label = new JLabel (ilabel);
        if (ilabel.length() > 0) {
            ilabel += " ";
        }
        label.setName("Label-" + option.getName());

        String mnemo = getString("mnemonic", null);
        if (mnemo != null && mnemo.length() > 0) {
            label.setDisplayedMnemonic(mnemo.charAt(0));
        }
        disabled = getBoolean("disabled", false);
        label.setForeground(UIManager.getColor
                            (disabled ? "textInactiveText" : "textText"));

        boolean internal = getBoolean("internal", false);
        if (! internal) {
            add(label, BorderLayout.NORTH);
            ilabel = "";
        }

        key = new KeyLabel (getBoolean("capturetab", false));
        key.setName("Key-" + option.getName());
        if (internal) {
            if (mnemo != null && mnemo.length() > 0) {
                key.setDisplayedMnemonic(mnemo.charAt(0));
            }
            key.setLabelFor(key);
        } else {
            label.setLabelFor(key);
        }
        key.setForeground(UIManager.getColor("textText"));
        key.addKeyListener(this);
        add(key, BorderLayout.CENTER);
        if (option.getValue() instanceof KeyOption.Data) {
            KeyOption.Data data = (KeyOption.Data) option.getValue();
            keyCode = data.keyCode;
            keyChar = data.keyChar;
            modifiers = data.modifiers;
        }
        key.setValues(ilabel, keyCode, keyChar, modifiers);
        if (disabled) {
            setEnabled(false);
        }
    }

    // }}}
    // {{{ checkValue

    /**
     * Check the currently entered value of the OptionEdit. If the value is
     * not valid, request the focus.
     *
     * @return  True if the value is valid, false otherwise.
     */
    public boolean checkValue()
    {
        if (! option.checkValue(this, new KeyOption.Data
                                (keyCode, keyChar, modifiers))) {
            key.requestFocus();
            return false;
        }
        return true;
    }

    // }}}
    // {{{ readValue

    /**
     * Read the currently entered value and store it in the Option.
     */
    public void readValue()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "readValue()", "");
        }
        option.setValue(new KeyOption.Data
                        (keyCode, keyChar, modifiers));
    }

    // }}}
    // {{{ update

    /**
     * Update the display for a possibly new Option.
     *
     * @param   option The new option.
     */
    public void update(Option option)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "update(Option)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "option: " + option);
        }
        this.option = option;
        if (option.getValue() instanceof KeyOption.Data) {
            KeyOption.Data data = (KeyOption.Data) option.getValue();
            keyCode = data.keyCode;
            keyChar = data.keyChar;
            modifiers = data.modifiers;
        } else {
            keyCode = 0;
            keyChar = 65535;
            modifiers = 0;
        }
        key.setValues(ilabel, keyCode, keyChar, modifiers);
    }

    // }}}

    //----------------------------------------------------------------------
    // Override some methods from JPanel
    //----------------------------------------------------------------------
    // {{{ setEnabled

    /**
     * Enable or disable the KeyOptionEdit.
     *
     * @param   enabled True to enable, false to disable.
     */
    public void setEnabled(boolean enabled)
    {
        if (disabled) {
            enabled = false;
        }

        key.setEnabled(enabled);
        if (enabled) {
            label.setForeground(UIManager.getColor("textText"));
        } else {
            label.setForeground(UIManager.getColor("textInactiveText"));
        }
    }

    // }}}
    // {{{ isEnabled

    /**
     * Query whether the KeyOptionEdit is enabled.
     *
     * @return  True if it is enabled.
     */
    public boolean isEnabled()
    {
        return key.isEnabled();
    }

    // }}}
    // {{{ requestFocus

    /**
     * Request the focus for the KeyOptionEdit.
     */
    public void requestFocus()
    {
        key.requestFocus();
    }

    // }}}

    //----------------------------------------------------------------------
    // ChangeListener support
    //----------------------------------------------------------------------
    // {{{ addChangeListener

    /**
     * Add a ChangeListener to the KeyOptionEdit. It will be notified when the
     * selected key combination changes.
     *
     * @param   listener        The listener to add.
     */
    public synchronized void addChangeListener(ChangeListener listener)
    {
        listeners.removeElement(listener);
        listeners.addElement(listener);
    }


    // }}}
    // {{{ remvoveChangeListener

    /**
     * Remove a ChangeListener from the KeyOptionEdit.
     *
     * @param   listener        The listener to remove.
     */
    public synchronized void removeChangeListener(ChangeListener listener)
    {
        listeners.removeElement(listener);
    }

    // }}}
    // {{{ fireChanged

    /**
     * Notify the listeners that the key combination has changed.
     */
    protected void fireChanged()
    {
        Vector tmp;
        synchronized (this) {
            if (listeners.size() == 0) {
                return;
            }
            tmp = (Vector) listeners.clone();
        }
        ChangeEvent event = new ChangeEvent (this);
        for (int i = 0; i < tmp.size(); i++) {
            ((ChangeListener) tmp.elementAt(i)).stateChanged(event);
        }
    }

    // }}}
    // {{{ getKeyCode

    /**
     * Get the current keyCode value.
     *
     * @return  The current keyCode value.
     */
    public int getKeyCode()
    {
        return keyCode;
    }

    // }}}
    // {{{ getKeyChar

    /**
     * Get the current keyChar value.
     *
     * @return  The current keyChar value.
     */
    public char getKeyChar()
    {
        return keyChar;
    }

    // }}}
    // {{{ getModifiers

    /**
     * Get the current modifiers value.
     *
     * @return  The current modifiers value.
     */
    public int getModifiers()
    {
        return modifiers;
    }

    // }}}

    //----------------------------------------------------------------------
    // The KeyListener interface
    //----------------------------------------------------------------------
    // {{{ keyPressed

    /**
     * Notify the listener that a key was pressed.
     *
     * @param   e       The event containing the details.
     */
    public void keyPressed(KeyEvent e)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "keyPressed(KeyEvent)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "e: " + e);
        }
        if (key.isEnabled()) {
            keyCode = e.getKeyCode();
            keyChar = e.getKeyChar();
            modifiers = e.getModifiers();
            e.consume();
            key.setValues(ilabel, keyCode, keyChar, modifiers);
            fireChanged();
        }
    }

    // }}}
    // {{{ keyTyped

    /**
     * Notify the listener that a key was typed. Noop.
     *
     * @param   e       The event containing the details.
     */
    public void keyTyped(KeyEvent e)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "keyTyped(KeyEvent)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "e: " + e);
        }
    }

    // }}}
    // {{{ keyReleased

    /**
     * Notify the listener that a key was released. Noop.
     *
     * @param   e       The event containing the details.
     */
    public void keyReleased(KeyEvent e)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "keyReleased(KeyEvent)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "e: " + e);
        }
    }

    // }}}

    //----------------------------------------------------------------------
    // Helper classes
    //----------------------------------------------------------------------
    // {{{ class KeyLabel

    /**
     * Extended Label that displays a key combination.
     */
    protected static class KeyLabel
        extends JLabel
        implements FocusListener
    {
        // {{{ variables

        /**
         * The border to use when focused.
         */
        private Border focus =
            BorderFactory.createCompoundBorder
            (BorderFactory.createEtchedBorder(),
             BorderFactory.createCompoundBorder
             (BorderFactory.createEmptyBorder(1, 1, 1, 1),
              BorderFactory.createLineBorder
              (UIManager.getColor("activeCaptionBorder"), 1)));

        /**
         * The border to use when not focused.
         */
        private Border noFocus =
            BorderFactory.createCompoundBorder
            (BorderFactory.createEtchedBorder(),
             BorderFactory.createEmptyBorder(2, 2, 2, 2));

        /**
         * Whether to capture the tab key.
         */
        private boolean captureTab;

        // }}}

        //------------------------------------------------------------------
        // Constructor
        //------------------------------------------------------------------
        // {{{ KeyLabel

        /**
         * Create a new KeyLabel.
         *
         * @param       captureTab      Whether to capture the tab key.
         */
        public KeyLabel (boolean captureTab)
        {
            if (logger.level >= Log.MTD) {
                logger.log(Log.MTD, "KeyLabel()", "");
            }

            this.captureTab = captureTab;

            setRequestFocusEnabled(true);
            setBorder(noFocus);
            setText("X");
            setPreferredSize(getPreferredSize());
            addFocusListener(this);
            addMouseListener(new MouseAdapter () {
                public void mousePressed(MouseEvent e)
                {
                    KeyLabel.this.requestFocus();
                }
            });
        }

        // }}}

        // {{{ isFocusTraversable

        /**
         * Query whether the KeyLabel is traversable.
         *
         * @return      True if enabled.
         */
        public boolean isFocusTraversable()
        {
            return this.isEnabled();
        }

        // }}}
        // {{{ isManagingFocus

        /**
         * Query whether the KeyLabel manages focus.
         *
         * @return      True if capture tab is enabled.
         */
        public boolean isManagingFocus()
        {
            return captureTab;
        }

        // }}}
        // {{{ setValues

        /**
         * Set new values to display.
         *
         * @param       prefix    Prefix label
         * @param       keyCode   The keyCode of the key.
         * @param       modifiers The modifiers of the key.
         */
        public void setValues(String prefix, int keyCode, char keyChar,
                              int modifiers)
        {
            if (logger.level >= Log.MTD) {
                logger.log(Log.MTD, "setValues(int,int)",
                           logger.level < Log.MTDDETAIL ? "" :
                           "keyCode: " + keyCode + ", " +
                           "modifiers: " + modifiers);
            }
            StringBuffer sb = new StringBuffer (prefix);
            if (modifiers != 0) {
                sb.append(KeyEvent.getKeyModifiersText(modifiers));
                sb.append('-');
            }
            if  (keyCode != 0
                 && keyCode != KeyEvent.VK_SHIFT
                 && keyCode != KeyEvent.VK_CONTROL
                 && keyCode != KeyEvent.VK_ALT
                 && keyCode != KeyEvent.VK_META) {
                sb.append(KeyEvent.getKeyText(keyCode));
            } else if (keyCode == 0 && keyChar != 65535) {
                sb.append(keyChar);
            }
            setText(sb.toString());
        }

        // }}}

        //------------------------------------------------------------------
        // The FocusListener interface
        //------------------------------------------------------------------
        // {{{ focusGained

        /**
         * Notify the KeyLabel that it has gained the focus.
         *
         * @param       e       The event containing the details.
         */
        public void focusGained(FocusEvent e)
        {
            setBorder(this.isEnabled() ? focus : noFocus);
        }

        // }}}
        // {{{ focusLost

        /**
         * Notify the KeyLabel that it has lost the focus.
         *
         * @param       e       The event containing the details.
         */
        public void focusLost(FocusEvent e)
        {
            setBorder(noFocus);
        }

        // }}}
    }

    // }}}
}
