// {{{ copyright

/********************************************************************
 *
 * $Id: NestedOptionGroup.java,v 1.10 2001/04/17 16:41:15 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.option;

// {{{ imports

import java.awt.BorderLayout;
import java.awt.Component;

import javax.swing.JPanel;

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;
import de.qfs.lib.util.MapResourceBundle;

// }}}

/**
 * A NestedOptionGroup represents a {@link NestedOption NestedOption} in an
 * {@link OptionSet OptionSet}. It is used to edit hierarchies of {@link
 * Option Options}. <p>
 *
 * See the package documentation for details about the necessary resources.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.10 $
 */
public class NestedOptionGroup
    extends OptionGroup
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger = new Logger (NestedOptionGroup.class);

    /**
     * The option holding the nested OptionSet.
     */
    protected Option option;

    /**
     * The nested OptionGroup.
     */
    protected OptionGroup nested;

    // }}}

    // {{{ constructors

    /**
     * No arg constructor for customized OptionGroups.
     */
    protected NestedOptionGroup ()
    {
    }

   /**
     * Create a new NestedOptionGroup.
     *
     * @param   name    The name of the group.
     * @param   options The options to edit.
     */
    public NestedOptionGroup (String name, OptionSet options)
    {
        super(name, options);
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "NestedOptionGroup(String,OptionSet)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "name: " + name);
        }
        String optionname = getString("option", null);
        if (optionname == null) {
            if (logger.level >= Log.ERR) {
                logger.log(Log.ERR, "NestedOptionGroup(String,OptionSet)",
                           "No option for NestedOptionGroup " + name);
                return;
            }
        }

        option = options.getOption(optionname);
        if (option == null && !(option.getValue() instanceof OptionSet)) {
            if (logger.level >= Log.ERR) {
                logger.log(Log.ERR, "NestedOptionGroup(String,OptionSet)",
                           "No option for NestedOptionGroup " + name);
                return;
            }
        }

        nested = OptionGroup.makeGroup
            (((OptionSet) option.getValue()).getConfigurableName(),
             (OptionSet) option.getValue());
    }

    // }}}

    // {{{ checkValues

    /**
     * Check whether all members of the nested group contain valid values.
     *
     * @return  True, if everything is OK.
     */
    public boolean checkValues()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "checkValues()", "");
        }
        return nested.checkValues();
    }

    // }}}
    // {{{ readValues

    /**
     * Read the values from the nested group components and store them into the
     * Options. Should only be called after {@link #checkValues checkValues}
     * succeeded.
     */
    public void readValues()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "readValues()", "");
        }
        nested.readValues();
    }

    // }}}
    // {{{ notifyOptionObservers

    /**
     * Notify the option's Observers.
     */
    public void notifyOptionObservers()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "notifyOptionObservers()", "");
        }
        nested.notifyOptionObservers();
    }

    // }}}
    // {{{ update

    /**
     * Update display for possibly new Options. The name and type of
     * the Options in the OptionSet must be the same as before.
     *
     * @param   options The new OptionSet.
     */
    public void update(OptionSet options)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "update(OptionSet)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "my name: " + name);
        }
        this.options = options;
        option = options.getOption(option.getName());
        if (option == null) {
            throw new NullPointerException();
        }
        nested.update((OptionSet) option.getValue());
    }

    // }}}
    // {{{ setEnabled

    /**
     * Set the enabled state for the nested group.
     *
     * @param   enabled The enabled state to set.
     */
    public void setEnabled (boolean enabled)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "setEnabled(boolean)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "enabled: " + enabled);
        }

        nested.setEnabled(enabled);
    }

    // }}}
    // {{{ getComponent

    /**
     * Get the AWT Component for the OptionGroup.
     *
     * @return  The Component.
     */
    public Component getComponent()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getComponent()", "");
        }
        return nested.getComponent();
    }

    // }}}
    // {{{ setTitle

    /**
     * Set the title of the OptionGroup. Overrides the value read from the
     * .title property. Only useful if the group has a frame.
     *
     * @param   title   The title to set.
     * @since    0.98.0
     */
    public void setTitle(String title)
    {
        nested.setTitle(title);
    }

    // }}}

    //----------------------------------------------------------------------
    // Group hierarchy
    //----------------------------------------------------------------------
    // {{{ getParentGroup

    /**
     * Get the parent of the OptionGroup.
     *
     * @return  The OptionGroup's parent.
     * @since    0.98.0
     */
    public OptionGroup getParentGroup()
    {
        return nested.getParentGroup();
    }

    // }}}
    // {{{ setParentGroup

    /**
     * Set the parent of the OptionGroup.
     *
     * @param   parent  The parent to set.
     * @since    0.98.0
     */
    public void setParentGroup(OptionGroup parent)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "setParentGroup(OptionGroup)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "parent: " + parent);
        }
        nested.setParentGroup(parent);
    }

    // }}}
    // {{{ getMemberCount

    /**
     * Get the number of members of the OptionGroup.
     *
     * @return  The number of the OptionGroup's members.
     * @since    0.98.0
     */
    public int getMemberCount()
    {
        return nested.getMemberCount();
    }

    // }}}
    // {{{ getMember

    /**
     * Get a member of the group.
     *
     * @param   index   The index of the member.
     *
     * @return  The member, either an OptionGroup or an OptionEdit.
     * @since    0.98.0
     */
    public Object getMember(int index)
    {
        return nested.getMember(index);
    }

    // }}}
    // {{{ getIndexOfMember

    /**
     * Find a member of the OptionGroup and get its index.
     *
     * @param   member  The member to look for.
     *
     * @return  The member's index or -1.
     * @since    0.98.0
     */
    public int getIndexOfMember(Object member)
    {
        return nested.getIndexOfMember(member);
    }

    // }}}

}
