// {{{ copyright

/********************************************************************
 *
 * $Id: OptionGroupTreeAdapter.java,v 1.1 2001/04/13 16:08:34 gs Exp $
 *
 * Copyright (C) 2000 Gregor Schmid, Quality First Software
 * All rights reserved
 *
 *******************************************************************/

// }}}

package de.qfs.lib.option;

// {{{ imports

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;
import de.qfs.lib.tree.IndexBasedTreeAdapter;

// }}}

/**
 * This is a TreeAdapter for the OptionGroup hierarchy. It assumes
 * that all nodes passed are OptionGroups or OptionEdits.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.1 $
 * @since       0.98.1
 */
public class OptionGroupTreeAdapter
    extends IndexBasedTreeAdapter
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger =
        new Logger (OptionGroupTreeAdapter.class);

    // }}}

    // {{{ getParentNode

    /**
     * Get the parent of a node.
     *
     * @param   child   The node to query for the parent.
     *
     * @return  The parent of the node.
     */
    public Object getParentNode(Object child)
    {
        if (child instanceof OptionGroup) {
            return ((OptionGroup) child).getParentGroup();
        } else {
            return ((OptionEdit) child).getOptionGroup();
        }
    }

    // }}}
    // {{{ isLeaf

    /**
     * Query whether a node is a leaf.
     *
     * @param   node    The node to query.
     *
     * @return  True if the node is a leaf.
     */
    public boolean isLeaf(Object node)
    {
        return (node instanceof OptionEdit);
    }

    // }}}

    //----------------------------------------------------------------------
    // Index based
    //----------------------------------------------------------------------
    // {{{ getChildCount

    /**
     * Get the number of children of a node.
     *
     * @param   parent  The node to query.
     *
     * @return  The number of children of the node.
     */
    public int getChildCount(Object parent)
    {
        if (parent instanceof OptionGroup) {
            return ((OptionGroup) parent).getMemberCount();
        }
        return 0;
    }

    // }}}
    // {{{ getChildNode

    /**
     * Get a child of a node.
     *
     * @param   parent  The node to query.
     * @param   index   The index of the child.
     *
     * @return  The child at the index.
     *
     * @throws   IndexOutOfBoundsException If the index is invalid.
     */
    public Object getChildNode(Object parent, int index)
         throws IndexOutOfBoundsException
    {
        if (! (parent instanceof OptionGroup)) {
            throw new IndexOutOfBoundsException
                ("parent " + parent + " is not an OptionGroup");
        }
        return ((OptionGroup) parent).getMember(index);
    }

    // }}}
    // {{{ getIndexOfChildNode

    /**
     * Get the index of a child in a node's list of children.
     *
     * @param   parent  The node to query.
     * @param   child   The child to look for.
     *
     * @return  The index of the child or -1 if it is not a child of this node.
     */
    public int getIndexOfChildNode(Object parent, Object child)
    {
        if (! (parent instanceof OptionGroup)) {
            return -1;
        }
        return ((OptionGroup) parent).getIndexOfMember(child);
    }

    // }}}

}
