// {{{ copyright

/********************************************************************
 *
 * $Id: RadioOptionGroup.jpp,v 1.17 2001/04/17 16:41:13 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.option;

// {{{ imports

import java.awt.Color;
import java.awt.Component;


import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;


import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;
import javax.swing.ButtonModel;
import javax.swing.UIManager;
import javax.swing.border.TitledBorder;

import de.qfs.lib.gui.Grid;
import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;
import de.qfs.lib.util.MapResourceBundle;

// }}}

// {{{ doc

/**
 * A RadioOptionGroup consists of a group of mutually exclusive RadioButtons
 * for one Option value. The option should be an {@link IntegerOption
 * IntegerOption}, its value between 1 and the number of buttons. <p>
 *
 * The RadioOptionGroup uses the following resources: <p>
 *
 * <TABLE BORDER="1" CELLPADDING="3" CELLSPACING="0" WIDTH="100%">
 * <TR>
 * <TH WIDTH="40%">Resource name</TH>
 * <TH>Usage</TH>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>option.group.<em>groupname</em>.type</code></TD>
 * <TD>A RadioOptionGroup is designated by the type <code>radio</code>.</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>option.<em>name</em>.disabled</code></TD>
 * <TD>If true, the group will be disabled and remain so even if
 * {@link OptionGroup#setEnabled setEnabled(true)} is called (optional,
 * default false).</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>option.group.<em>groupname</em>.option</code></TD>
 * <TD>The name of the Option for the group.</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>option.group.<em>groupname</em>.numchoices</code></TD>
 * <TD>The number of Buttons in the group.</TD>
 * </TR>
 * <TR>
 * <TD * WIDTH="40%"><code>option.group.<em>groupname</em>.choice<em>n</em>.label</code></TD>
 * <TD>The label for the n<sup>th</sup> button of the group, where n is
 * between 1 and numchoices.</TD>
 * </TR>
 * <TR>
 * <TD * WIDTH="40%"><code>option.group.<em>groupname</em>.choice<em>n</em>.mnemonic</code></TD>
 * <TD>Keyboard accelerator for the n<sup>th</sup> button of the group
 * (optional, default none).</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>option.group.<em>groupname</em>.columns</code></TD>
 * <TD>The number of columns in which to place the buttons of the group
 * (optional, default 1).</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>option.group.<em>groupname</em>.frame</code></TD>
 * <TD>Whether the group should have a framed border (optional, default
 * false).</TD>
 * </TR>
 * <TR>
 * <TD WIDTH="40%"><code>option.group.<em>groupname</em>.title</code></TD>
 * <TD>The title for a framed border of the group. Ignored if no frame is
 * specified(optional, default empty).</TD>
 * </TR>
 * </TABLE>
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.17 $
 */

// }}}
public class RadioOptionGroup extends OptionGroup
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger = new Logger (RadioOptionGroup.class);

    /**
     * The possible choices.
     */
    protected List choices;

    /**
     * The Grid Component that holds the members of the RadioOptionGroup.
     */
    protected Grid grid;

    /**
     * The Option that holds the value for the selected Button.
     */
    protected Option option;

    /**
     * The ButtonGroup that manages the JButtons.
     */
    protected ButtonGroup buttonGroup;

    /**
     * The JButton members of the ButtonGroup.
     */
    protected List buttons;

    // }}}

    //----------------------------------------------------------------------
    // Constructors
    //----------------------------------------------------------------------
    // {{{ RadioOptionGroup()

    /**
     * No arg constructor for customized OptionGroups.
     */
    protected RadioOptionGroup ()
    {
    }

    // }}}
    // {{{ RadioOptionGroup(String,OptionSet)

    /**
     * Create a new RadioOptionGroup.
     *
     * @param   name    The name of the group.
     * @param   options The Options to edit.
     */
    public RadioOptionGroup (String name, OptionSet options)
    {
        super (name, options);
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "RadioOptionGroup(String,OptionSet)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "name: " + name);
        }
        getChoices();
    }

    // }}}

    //----------------------------------------------------------------------
    // Public methods
    //----------------------------------------------------------------------
    // {{{ getOption

    /**
     * Get the option that holds the value for the RadioOptionGroup.
     *
     * @return  Get the RadioOptionGroup's option.
     * @since   0.98.0
     */
    public Option getOption()
    {
        return option;
    }


    // }}}
    // {{{ checkValues

    /**
     * Check the values.
     *
     * @return  True if the selected button is accepted by the Option.
     */
    public boolean checkValues()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "checkValues()", "");
        }
        if (buttonGroup == null) {
            // not initialized
            return true;
        }
        ButtonModel bm = buttonGroup.getSelection ();
        for (int i = 0; i < buttons.size(); i++) {
            if (((JRadioButton) buttons.get(i)).getModel() == bm) {
                if (! option.checkValue(grid, new Integer (i + 1))) {
                    ((JRadioButton) buttons.get(i)).requestFocus();
                    return false;
                }
                break;
            }
        }
        return true;
    }

    // }}}
    // {{{ readValues

    /**
     * Store the value for the selected Button in the Option.
     */
    public void readValues()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "readValues()", "");
        }
        if (buttonGroup == null) {
            // not initialized
            return;
        }
        ButtonModel bm = buttonGroup.getSelection ();
        for (int i = 0; i < buttons.size(); i++) {
            if (((JRadioButton) buttons.get(i)).getModel() == bm) {
                option.setValue(new Integer (i + 1));
                return;
            }
        }
        option.setValue(new Integer (0));
    }

    // }}}
    // {{{ notifyOptionObservers

    /**
     * Notify the option's Observers.
     */
    public void notifyOptionObservers()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "notifyOptionObservers()", "");
        }
        option.notifyObservers();
    }

    // }}}
    // {{{ update

    /**
     * Update display for possibly new Options. The name and type of
     * the Options in the OptionSet must be the same as before.
     *
     * @param   options The new OptionSet.
     */
    public void update(OptionSet options)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "update(OptionSet)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "options: " + options +
                       " this: " + this);
        }
        this.options = options;
        option = options.getOption(option.getName());
        if (option == null) {
            throw new NullPointerException();
        }
        int current = ((Integer) option.getValue()).intValue();
        if (current >= 1 && current <= buttons.size() ) {
            ((JRadioButton) buttons.get(current - 1)).setSelected(true);
        }
    }

    // }}}
    // {{{ setEnabled

    /**
     * Set the enabled state for the buttons.
     *
     * @param   enabled The enabled state to set.
     */
    public void setEnabled (boolean enabled)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "setEnabled(boolean)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "enabled: " + enabled +
                       ", disabled: " + disabled);
        }

        super.setEnabled(enabled);

        for (Iterator i = buttons.iterator(); i.hasNext(); ) {
            ((JRadioButton) i.next()).setEnabled(this.enabled);
        }
    }

    // }}}
    // {{{ getComponent

    /**
     * Get the AWT Component for the OptionGroup.
     *
     * @return  The Component.
     */
    public Component getComponent()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getComponent()", "");
        }
        if (grid == null) {
            makeGrid();
        }
        return grid;
    }

    // }}}

    //----------------------------------------------------------------------
    // Helper methods
    //----------------------------------------------------------------------
    // {{{ getChoices

    /**
     * Create the possible choices.
     */
    protected void getChoices()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getChoices()", "");
        }
        MapResourceBundle rb = Option.getResources();
        choices = new ArrayList();

        int num = getInt("numchoices", -1);
        if (num < 0) {
            if (logger.level >= Log.ERR) {
                logger.log(Log.ERR, "getChoices()",
                           "No numchoices for RadioOptionGroup " + name);
                return;
            }
        }

        String optionname = getString("option", null);
        if (optionname == null) {
            if (logger.level >= Log.ERR) {
                logger.log(Log.ERR, "getChoices()",
                           "No option for RadioOptionGroup " + name);
                return;
            }
        }
        option = options.getOption(optionname);
        if (option == null) {
            if (logger.level >= Log.ERR) {
                logger.log(Log.ERR, "getChoices()",
                           "No option for RadioOptionGroup " + name);
                return;
            }
        }


        for (int i = 1; i <= num; i++) {
            String choice = getString("choice" + i + ".label", null);
            if (choice == null) {
                if (logger.level >= Log.ERR) {
                    logger.log(Log.ERR, "getChoices()",
                               "No label for choice " + i +
                               " in RadioOptionGroup " + name);
                }
            } else {
                String mnemo = getString("choice" + i + ".mnemonic", null);
                choices.add(new String[] {choice, mnemo});
            }
        }
    }

    // }}}
    // {{{ makeGrid

    /**
     * Create the Grid for the group members.
     */
    protected void makeGrid()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "makeGrid()", "");
        }
        MapResourceBundle rb = Option.getResources();

        int columns = getInt("columns", 1);

        grid = new Grid();
        grid.setName("OptionGroup-" + name + "-" +
                     options.getConfigurableName());
        buttonGroup = new ButtonGroup ();
        buttons = new ArrayList ();

        for (int i = 0; i < choices.size(); i+= columns) {
            int num = Math.min(columns, choices.size() - i);
            Component[] coms = new Component [num];
            for (int j = 0; j < num; j++) {
                coms[j] = new JRadioButton
                    (((String[]) choices.get(i + j))[0]);
                String mnemo = ((String[]) choices.get(i + j))[1];
                if (mnemo != null && mnemo.length() > 0) {
                    ((JRadioButton) coms[j]).setMnemonic(mnemo.charAt(0));
                }
                coms[j].setName("Choice" + (i + j + 1) + "-" + name);
                buttonGroup.add((JRadioButton) coms[j]);
                buttons.add(coms[j]);
            }
            grid.addLine(coms);
        }
        // grid.addGlue();

        int current = ((Integer) option.getValue()).intValue();
        if (current >= 1 && current <= buttons.size() ) {
            ((JRadioButton) buttons.get(current - 1)).setSelected(true);
        }

        if (disabled) {
            setEnabled(false);
        }

        makeBorder(grid);
    }

    // }}}
}
