// {{{ copyright

/********************************************************************
 *
 * $Id: TextOption.java,v 1.11 2001/01/25 17:06:56 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.option;

// {{{ imports

import java.util.StringTokenizer;
import java.util.Vector;

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

/**
 * A TextOption is similar to a StringOption except that its edit Component
 * is a mulitline text area.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.11 $
 */
public class TextOption extends StringOption
{
    // {{{ variables

    /**
     * The UID used for serialization.
     */
    static final long serialVersionUID = 6585971683158105777L;

    /**
     * The Logger used for logging.
     */
    private static Logger logger = new Logger (TextOption.class);

    /**
     * Line separator (system specific).
     */
    public final static String LINESEP = System.getProperty("line.separator");

    // }}}

    // {{{ constructors

    /**
     * Create a new TextOption.
     */
    public TextOption()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "TextOption()", "");
        }
    }

    /**
     * Create a new TextOption with a name and a value.
     *
     * @param   name    The name to use.
     * @param   value   The value to use.
     */
    public TextOption(String name, String value)
    {
        super (name, value);
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "TextOption(String,String)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "name: " + name + ", " +
                       "value: " + value);
        }
    }

    /**
     * Create a new TextOption with a name and a value and the nullOK flag.
     *
     * @param   name    The name to use.
     * @param   value   The value to use.
     * @param   nullOK  The nullOK flag to use.
     */
    public TextOption(String name, String value, boolean nullOK)
    {
        super (name, value, nullOK);
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "TextOption(String,String,boolean)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "name: " + name + ", " +
                       "value: " + value + ", " +
                       "nullOK: " + nullOK);
        }
    }

    // }}}

    //----------------------------------------------------------------------
    // Static methods
    //----------------------------------------------------------------------
    // {{{ split

    /**
     * Split a String into an array of Strings.
     *
     * @param   value           The String to split.
     * @param   sep             The separator.
     * @param   stripEmpty      If true, don't include empty lines.
     */
    public static String[] split(String value, String sep, boolean stripEmpty)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "split(String,String)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "value: " + value + ", " +
                       "sep: " + sep);
        }
        if (value == null) {
            return null;
        }
        Vector vals = new Vector ();
        StringTokenizer st = new StringTokenizer (value, sep);
        while (st.hasMoreTokens()) {
            String val = st.nextToken();
            if (val.length() > 0 || !stripEmpty) {
                vals.addElement(val);
            }
        }
        String[] ret = new String[vals.size()];
        vals.copyInto(ret);
        return ret;
    }

    // }}}
    // {{{ join

    /**
     * Join an arry of Strings into one String.
     *
     * @param   values  The Strings to join.
     * @param   sep     The separator.
     */
    public static String join(String[] values, String sep)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "join(String[],String)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "values: " + values + ", " +
                       "sep: " + sep);
        }
        if (values == null) {
            return null;
        }
        StringBuffer sb = new StringBuffer ();
        for (int i = 0; i < values.length; i++) {
            sb.append(values[i]);
            if (i < values.length - 1) {
                sb.append(sep);
            }
        }
        return sb.toString();
    }

    // }}}

    //----------------------------------------------------------------------
    // instanceMethods
    //----------------------------------------------------------------------
    // {{{ toArray

    /**
     * Return the value of the TextOption as an array of Strings, each element
     * of the array representing one line of text.
     *
     * @param   stripEmpty      If true, don't include empty lines.
     *
     * @return  The array of Strings.
     * @since   0.98.0
     */
    public String[] toArray(boolean stripEmpty)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "toArray(boolean)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "stripEmpty: " + stripEmpty);
        }
        if (getValue() == null) {
            return null;
        }
        return split((String) getValue(), LINESEP, stripEmpty);
    }

    // }}}
    // {{{ fromArray

    /**
     * Create the value of the TextOption by joining Strings from an array with
     * the newline character.
     *
     * @param   lines   The Strings to join.
     * @since   0.98.0
     */
    public void fromArray(String[] lines)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "fromArray(String[])",
                       logger.level < Log.MTDDETAIL ? "" :
                       "lines: " + lines);
        }
        if (lines == null) {
            setValue(null);
        } else {
            setValue(join(lines, LINESEP));
        }
    }


    // }}}

}
