// {{{ copyright

/********************************************************************
 *
 * $Id: CommitableStateChange.java,v 1.5 2000/06/14 16:52:00 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.transaction;

// {{{ imports

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

/**
 * As part of a {@link Transaction Transaction} the CommitableStateChange
 * tells a {@link StateHolder StateHolder} when to save and restore
 * its state.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.5 $
 * @since       0.97.0
 */
public class CommitableStateChange
implements Commitable
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger =
        new Logger (CommitableStateChange.class);

    /**
     * The {@link StateHolder StateHolder} whose state is about to change.
     */
    protected StateHolder stateHolder;

    /**
     * Some information handed back by {@link StateHolder#saveState
     * saveState}.
     */
    protected Object saved;

    // }}}

    // {{{ constructor

    /**
     * Create a new CommitableStateChange.
     *
     * @param   stateHolder     The StateHolder whose state is about to change.
     */
    public CommitableStateChange (StateHolder stateHolder)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "CommitableStateChange(StateHolder)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "stateHolder: " + stateHolder);
        }
        this.stateHolder = stateHolder;
        saved = stateHolder.saveState();
    }

    // }}}

    // {{{ commit

    /**
     * Tell the StateHolder that its saved state will no longer be needed.
     *
     * @throws FatalTransactionException If the StateHolder fails to commit
     *          its state in a way that corrupts the whole Transaction.
     */
    public void commit()
         throws FatalTransactionException
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "commit()", "");
        }
        stateHolder.commitState(saved);
    }

    // }}}
    // {{{ rollback

    /**
     * Undo the StateHolder's change of state.
     *
     * @throws FatalTransactionException If the StateHolder fails to restore
     *          its state in a way that corrupts the whole Transaction.
     */
    public void rollback()
         throws FatalTransactionException
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "rollback()", "");
        }
        stateHolder.restoreState(saved);
    }

    // }}}
}
