// {{{ copyright

/********************************************************************
 *
 * $Id: FatalTransactionException.java,v 1.5 2000/06/14 16:52:01 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.transaction;

// {{{ imports

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

/**
 * A FatalTransactionException can be thrown by a {@link Commitable
 * Commitable}. This happens when a problem occurs, that makes continuation of
 * a {@link Transaction Transaction's} {@link Transaction#commit commit} or
 * {@link Transaction#rollback rollback} useless or even impossible, e.g. a
 * failed JDBC commit. <p>
 *
 * FatalTransactionException is derived from RuntimeException, since they are
 * so fatal, that there is typically only one handler for it, so it should
 * travel as far as possible to the top of the call stack. It would be too
 * extreme to make it an {@link java.lang.Error Error} though.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.5 $
 * @since       0.97.0
 */
public class FatalTransactionException extends RuntimeException
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger =
        new Logger (FatalTransactionException.class);

    /**
     * The Exception that caused the FatalTransactionException.
     *
     * @serial
     */
    private Exception nested;

    // }}}

    // {{{ constructors

    /**
     * Create a new FatalTransactionException.
     *
     * @param   detail  A message stating the reason for the Exception.
     */
    public FatalTransactionException (String detail)
    {
        this (detail, null);
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "FatalTransactionException(String)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "detail: " + detail);
        }
    }

    /**
     * Create a new FatalTransactionException.
     *
     * @param   detail  A message stating the reason for the Exception.
     * @param   nested  The Exception that caused the
     *          FatalTransactionException.
     */
    public FatalTransactionException (String detail, Exception nested)
    {
        super (detail);
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "FatalTransactionException(String,Exception)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "detail: " + detail + ", " +
                       "nested: " + nested);
        }

        this.nested = nested;
    }

    // }}}

    // {{{ getNestedException

    /**
     * Get the nested Exception.
     *
     * @return  The Exception that caused the FatalTransactionException.
     */
    public final Exception getNestedException()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getNestedException()", "");
        }
        return nested;
    }


    // }}}
}
