// {{{ copyright

/********************************************************************
 *
 * $Id: Transaction.jpp,v 1.8 2000/07/04 13:40:03 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.transaction;

// {{{ imports


import java.util.Map;
import java.util.HashMap;
import java.util.Collections;
import java.util.List;
import java.util.ArrayList;
import java.util.ListIterator;


import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;
import de.qfs.lib.util.ResourceWaiter;

// }}}

// {{{ doc

/**
 * Transaction instances allow actions to be collected and executed together
 * by {@link #commit commit} or to be undone together by {@link #rollback
 * rollback}. The Transaction class also has convenience methods to create and
 * handle Transactions, using the calling Thread as an identifier. There is no
 * support for transaction isolation of any kind.<p>
 *
 * With the {@link #add add} method {@link Commitable Commitables} can be
 * added to the Transaction, which are collected in a list. These are actions
 * that have to be commited to really have an effect or that can be rolled
 * back, to undo their effect. Commit and rollback always close the
 * Transaction first, so that no further Commitables can be added to it and
 * further calls to commit or rollback will do nothing.<p>
 *
 * When {@link #commit commit} is called on a Transaction, all of its
 * Commitable members are themselves commited in the order in which they
 * were added to the Transaction. In the case of {@link #rollback rollback}
 * each Commitable's {@link Commitable#rollback rollback} method is called
 * in the reverse order. <p>
 *
 * During either commit or rollback, RuntimeExceptions thrown by the
 * Commitable's methods will be logged but ignored until the whole
 * commit or rollback is completed. Only then the first RuntimeException
 * that occured will be rethrown. <p>
 *
 * A special case is the {@link FatalTransactionException
 * FatalTransactionException}. It may be thrown during commit or rollback to
 * signal that a problem has occured that makes continuation of commit or
 * rollback useless or even impossible, e.g. a failed JDBC commit. <p>
 *
 * There are two different ways of creating and managing Transactions. One way
 * is to simply create a Transaction through its constructor. This Transaction
 * has to be passed as a parameter to all methods that want to use it.  The
 * more convenient way is to use the static {@link #begin begin} method to
 * create a Transaction and retrieve it later with {@link #instance
 * instance}. The current Thread of the caller of these methods identifies the
 * Transaction to use. <p>
 *
 * It is OK to call {@link #instance instance} from a Thread for which no
 * Transaction has been created. In this case, a dummy Transaction will be
 * returned, which simply commits all Commitables at the moment they are added
 * to it. Of course, these can never be undone.  This mechanism is useful for
 * methods that don't care, whether they are part of a Transaction or not. <p>
 *
 * A Typical example might look like this:
 * <pre>
 * Transaction.begin();
 * // Watch out: the following block may raise a FatalTransactionException
 * try {
 *     Transaction.instance().add(new CommitableStateChange (object1));
 *     // modify object1...
 *     Transaction.instance().add(new CommitableNotify (object1));
 *     // similar for other objects...
 *     // call some methods that add more stuff via Transaction.instance...
 *     Transaction.instance().commit();
 * } finally {
 *     // if commit has been called, rollback won't do anything
 *     Transaction.instance().rollback();
 * }
 * <pre>
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.8 $
 * @since       0.97.0
 */

// }}}
public class Transaction
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private static Logger logger = new Logger (Transaction.class);

    /**
     * The maximum number of concurrent Transactions. <= 0 means indefinite.
     */
    private static int maxTransactions;

    /**
     * Map from Thread to Transaction
     */
    private static Map transactions =
        Collections.synchronizedMap(new HashMap ());

    /**
     * List of ResourceWaiters waiting for a Transaction.
     */
    private static List waiters =
        Collections.synchronizedList(new ArrayList ());

    /**
     * Map from Transactions to Threads
     */
    private static Map threads =
        Collections.synchronizedMap(new HashMap ());

    /**
     * The dummy transaction.
     */
    private static Transaction dummyTransaction = new Dummy ();

    /**
     * The Commitables of the Transaction.
     */
    private List commitables = new ArrayList ();

    /**
     * Whether the Transaction has been closed.
     */
    private boolean closed;

    // }}}

    // {{{ constructor

    /**
     * Constructor is not hidden. Normally {@link #instance instance} should
     * be used, but Transactions may be useful in their own rights.
     */
    public Transaction ()
    {
    }

    // }}}

    // {{{ instance

    /**
     * Return the Transaction instance for the current Thread.
     *
     * @return  The current Transaction.
     */
    public static Transaction instance()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "instance()", "");
        }
        Transaction ta =
            (Transaction) transactions.get(Thread.currentThread());
        if (ta == null) {
            return dummyTransaction;
        }
        return ta;
    }

    // }}}
    // {{{ begin

    /**
     * Begin a new Transaction for the current Thread.  If the maximum number
     * of concurrent Transactions has already been reached, this call will
     * block until a Transaction is available.
     *
     * @return  The Transaction that is now available via {@link #instance
     *          instance}.
     *
     * @throws  OpenTransactionException If there is a Transaction for the
     *          current Thread that has not been commited or rolled back.
     */
    public static synchronized Transaction begin()
         throws OpenTransactionException
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "begin()", "");
        }
        Transaction ta =
            (Transaction) transactions.get(Thread.currentThread());
        if (ta != null) {
            throw new OpenTransactionException
                ("Transaction for Thread " + Thread.currentThread()
                 + " still open.");
        }
        if (maxTransactions > 0 && transactions.size() >= maxTransactions) {
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG, "begin()",
                           "No more Transactions available: size = " +
                           transactions.size() + ", max = " +
                           maxTransactions);
            }
            ResourceWaiter waiter = new ResourceWaiter (Transaction.class);
            waiters.add(waiter);
            waiter.waitFor(-1);
        }
        ta = new Transaction ();
        transactions.put(Thread.currentThread(), ta);
        threads.put(ta, Thread.currentThread());
        return ta;
    }

    // }}}
    // {{{ close

    /**
     * Remove a Transaction from the maps.
     *
     * @param   transaction     The Transaction to remove.
     */
    private static void close(Transaction transaction)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "close(Transaction)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "transaction: " + transaction);
        }
        transaction.close();
        Object thread = threads.get(transaction);
        if (thread != null) {
            synchronized(Transaction.class) {
                // otherwise the Transaction was created directly
                threads.remove(transaction);
                transactions.remove(thread);
                if ((maxTransactions <= 0
                     || transactions.size() < maxTransactions)
                    && waiters.size() > 0) {
                    if (logger.level >= Log.DBG) {
                        logger.log(Log.DBG, "close(Transaction)",
                                   "Transaction released, calling waiter.");
                    }
                    ((ResourceWaiter) waiters.remove(0)).handResource(null);
                }
            }
        }
    }

    // }}}
    // {{{ insideTransaction

    /**
     * Check, whether a Transaction has been created for the current Thread.
     *
     * @return  True, if a Transaction is in progress.
     */
    public static boolean insideTransaction()
    {
        return transactions.containsKey(Thread.currentThread());
    }

    // }}}
    // {{{ getMaxTransactions

    /**
     * Get the maximum number of concurrent Transactions.
     */
    public final static int getMaxTransactions()
    {
        return maxTransactions;
    }

    // }}}
    // {{{ setMaxTransactions

    /**
     * Set the maximum number of concurrent Transactions. <= 0 means
     * indefinite.
     *
     * @param   maxTransactions The maximum to set.
     */
    public final static void setMaxTransactions(int _maxTransactions)
    {
        maxTransactions = _maxTransactions;
    }

    // }}}

    // {{{ add

    /**
     * Add a Commitable to the Transaction's rollback list.
     *
     * @param   commitable      The Commitable to add.
     */
    public void add(Commitable commitable)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "add(Commitable)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "commitable: " + commitable);
        }
        commitables.add(commitable);
    }

    // }}}
    // {{{ commit

    /**
     * Close the Transaction and commit all commitables on its rollback
     * list. <p>
     *
     * If the Transaction has already been closed, this does nothing.
     *
     * @throws  FatalTransactionException       If something fails in a way
     *          that makes completion of commit impossible.
     */
    public void commit()
         throws FatalTransactionException
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "commit()", "");
        }

        if (closed) {
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG, "commit()",
                           "already closed");
            }
            return;
        }

        // first close the Transaction
        close(this);

        RuntimeException exception = null;

        for (ListIterator i = commitables.listIterator(); i.hasNext();) {
            try {
                ((Commitable) i.next()).commit();
            } catch (FatalTransactionException ex) {
                if (logger.level >= Log.ERR) {
                    logger.log("rollback()", ex);
                }
                // pass through
                throw ex;
            } catch (RuntimeException ex) {
                if (logger.level >= Log.ERR) {
                    logger.log("commit()", ex);
                }
                if (exception == null) {
                    exception = ex;
                }
            }
        }
        if (exception != null) {
            throw exception;
        }
    }

    // }}}
    // {{{ rollback

    /**
     * Close the Transaction and roll back all commitables on its rollback
     * list. <p>
     *
     * If the Transaction has already been closed, this does nothing.
     *
     * @throws  FatalTransactionException       If something fails in a way
     *          that makes completion of rollback impossible.
     */
    public void rollback()
         throws FatalTransactionException
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "rollback()", "");
        }

        if (closed) {
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG, "commit()",
                           "already closed");
            }
            return;
        }

        // first close the Transaction
        close(this);

        RuntimeException exception = null;

        for (ListIterator i = commitables.listIterator(commitables.size());
             i.hasPrevious();) {
            try {
                ((Commitable) i.previous()).rollback();
            } catch (FatalTransactionException ex) {
                if (logger.level >= Log.ERR) {
                    logger.log("rollback()", ex);
                }
                // pass through
                throw ex;
            } catch (RuntimeException ex) {
                if (logger.level >= Log.ERR) {
                    logger.log("rollback()", ex);
                }
                if (exception == null) {
                    exception = ex;
                }
            }
        }
        if (exception != null) {
            throw exception;
        }
    }

    // }}}
    // {{{ close

    /**
     * Close the transaction. Further calls to commit or rollback will be
     * ignored. It is probably a bad idea to call this method directly instead
     * of calling commit or rollback, but if you want to get rid of a
     * transaction without commit or rollback, this is the way to do it.
     */
    public void close()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "close()", "");
        }
        closed = true;
    }

    // }}}
    // {{{ Dummy Transaction class

    /**
     * The class for the dummy Transaction used outside of a
     * Transaction context.
     */
    private static class Dummy extends Transaction
    {
        // {{{ constructor

        /**
         * Create the dummy Transaction.
         */
        public Dummy ()
        {
            if (logger.level >= Log.MTD) {
                logger.log(Log.MTD, "Dummy()", "");
            }
        }


        // }}}

        // {{{ add

        /**
         * Add a Commitable to the dummy Transaction. Instead of storing
         * the Commitable, its commit method is called immediately.
         *
         * @param       commitable      The Commitable.
         *
         * @throws FatalTransactionException If the Commitable's
         * commit method throws a FatalTransactionException.
         */
        public void add(Commitable commitable)
             throws FatalTransactionException
        {
            if (logger.level >= Log.MTD) {
                logger.log(Log.MTD, "add(Commitable)",
                           logger.level < Log.MTDDETAIL ? "" :
                           "commitable: " + commitable);
            }
            commitable.commit();
        }

        // }}}
        // {{{ commit

        /**
         * This is a noop.
         */
        public void commit()
        {
            if (logger.level >= Log.MTD) {
                logger.log(Log.MTD, "commit()", "");
            }
        }

        // }}}
        // {{{ rollback

        /**
         * This is a noop.
         */
        public void rollback()
        {
            if (logger.level >= Log.MTD) {
                logger.log(Log.MTD, "rollback()", "");
            }
        }


        // }}}

        private static Logger logger = new Logger (Dummy.class);
    }

    // }}}
}

