// {{{ copyright

/********************************************************************
 *
 * $Id: AbstractTraversal.java,v 1.2 2001/04/06 15:03:52 gs Exp $
 *
 * Copyright (C) 2000 Gregor Schmid, Quality First Software
 * All rights reserved
 *
 *******************************************************************/

// }}}

package de.qfs.lib.tree;

// {{{ imports

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

import java.util.Stack;

// }}}

/**
 * Abstract base class for implementations of the Traversal interface.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.2 $
 * @since       0.98.1
 */
public abstract class AbstractTraversal
    implements Traversal
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger = new Logger (AbstractTraversal.class);

    /**
     * The adapter for the tree.
     */
    TreeAdapter adapter;

    /**
     * The userObject for the traversal.
     */
    Object userObject;

    /**
     * The levels representing the ancestors of the current node.
     */
    Stack levels;

    /**
     * The data for the current node.
     */
    Level currentLevel;

    // }}}

    //----------------------------------------------------------------------
    // Constructor
    //----------------------------------------------------------------------
    // {{{ AbstractTraversal

    /**
     * Create a new AbstractTraversal.
     */
    protected AbstractTraversal (TreeAdapter adapter, Object root)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "AbstractTraversal(TreeAdapter,Object)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "adapter: " + adapter + ", " +
                       "root: " + root);
        }

        this.adapter = adapter;
        levels = new Stack ();
        currentLevel = new Level (root);
    }

    // }}}

    //------------------------------------------------------------------
    // The Traversal interface
    //------------------------------------------------------------------
    // {{{ getAdapter

    /**
     * Get the adapter for the tree being traversed.
     *
     * @return  The adapter for the tree being traversed.
     */
    public TreeAdapter getAdapter()
    {
        return adapter;
    }

    // }}}
    // {{{ getNode

    /**
     * Get the current node of the traversal.
     *
     * @return  The current node.
     */
    public Object getNode()
    {
        return currentLevel.node;
    }

    // }}}
    // {{{ getIndex

    /**
     * Get the index of the current node in its parent. For trees that are not
     * index-based , this is more efficient than {@link
     * TreeAdapter#getIndexOfChildNode TreeAdapter.getIndexOfChildNode}.
     *
     * @return  The index of the current node in its parent node.
     */
    public int getIndex()
    {
        return currentLevel.index;
    }

    // }}}
    // {{{ getParent

    /**
     * Get the parent node of the current node. The parent of the root of the
     * traversal will always be null, even if it is not the true root of the
     * tree.
     *
     * @return  The parent node of the current node.
     */
    public Object getParent()
    {
        return levels.isEmpty() ?  null
            : ((Level) levels.peek()).node;
    }

    // }}}
    // {{{ getPath

    /**
     * Get the path of the current node, up to the root of the traversal. If
     * the traversal doesn't start at the root of the tree, this path is
     * different from the path obtained with {@link TreeUtil#getPath
     * TreeUtil.getPath}.
     *
     * @return The path of the current node, up to the root of the
     * traversal. The first object in the array is the root, the last is the
     * current node.
     */
    public Object[] getPath()
    {
        Object[] ret = new Object[levels.size() + 1];
        for (int i = 0; i < levels.size(); i++) {
            ret[i] = ((Level) levels.elementAt(i)).node;
        }
        ret[levels.size()] = getNode();
        return ret;
    }

    // }}}
    // {{{ getPreviousSibling

    /**
     * Get the previous sibling of the current node. If the tree is not link
     * based, this method is faster than {@link TreeAdapter#getPreviousSibling
     * TreeAdapter.getPreviousSibling}.
     *
     * @return The previous sibling of the current node or null if it is the
     * first child of its parent or the root of the traversal.
     */
    public Object getPreviousSibling()
    {
        return currentLevel.previous;
    }

    // }}}
    // {{{ getUserObject

    /**
     * Get a user object assoiated with the traversal, set with {@link
     * #setUserObject setUserObject}.
     *
     * @return  The user object assoiated with the traversal.
     */
    public Object getUserObject()
    {
        return userObject;
    }

    // }}}
    // {{{ setUserObject

    /**
     * Set a user object for the traversal. Simplifies keeping some state
     * during the traversal with multiple threads.
     *
     * @param   object  The object to associate with the traversal.
     */
    public void setUserObject(Object object)
    {
        userObject = object;
    }

    // }}}

    //----------------------------------------------------------------------
    // Helper classes
    //----------------------------------------------------------------------
    // {{{ class Level

    /**
     * Structure that holds information about one level of the tree.
     */
    protected static class Level
    {
        // {{{ variables

        /**
         * The current node.
         */
        public Object node;

        /**
         * The previous node.
         */
        public Object previous;

        /**
         * The index of the node
         */
        public int index;

        // }}}

        //------------------------------------------------------------------
        // Constructor
        //------------------------------------------------------------------
        // {{{ Level(Object)

        /**
         * Create a new Level.
         */
        public Level (Object node)
        {
            if (logger.level >= Log.MTD) {
                logger.log(Log.MTD, "Level(Object)",
                           logger.level < Log.MTDDETAIL ? "" :
                           "node: " + node);
            }
            this.node = node;
        }

        // }}}
        // {{{ Level(Object,Object,int)

        /**
         * Create a new Level.
         */
        public Level (Object node, Object previous, int index)
        {
            if (logger.level >= Log.MTD) {
                logger.log(Log.MTD, "Level(Object)",
                           logger.level < Log.MTDDETAIL ? "" :
                           "node: " + node);
            }
            this.node = node;
            this.previous = previous;
            this.index = index;
        }

        // }}}
    }

    // }}}
}
