// {{{ copyright

/********************************************************************
 *
 * $Id: IndexBasedTreeAdapter.java,v 1.2 2001/04/06 15:03:52 gs Exp $
 *
 * Copyright (C) 2000 Gregor Schmid, Quality First Software
 * All rights reserved
 *
 *******************************************************************/

// }}}

package de.qfs.lib.tree;

// {{{ imports

import java.util.Enumeration;

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

/**
 * This abstract base class for an index based TreeAdapter implements the link
 * and enumeration based methods in terms of the index based ones, so it
 * leaves only the bare minimum to be implemented.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.2 $
 * @since       0.98.1
 */
public abstract class IndexBasedTreeAdapter
    implements TreeAdapter
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger =
        new Logger (IndexBasedTreeAdapter.class);

    // }}}

    //----------------------------------------------------------------------
    // Index based
    //----------------------------------------------------------------------
    // {{{ isIndexBased

    /**
     * Query whether the TreeAdapter implementation supports index based child
     * access efficiently. <p>
     *
     * If the list of children of a node is implemented as a linked list for
     * example, index based access is very inefficient, while an array based
     * implementation should retrun true. The index based methods {@link
     * #getChildCount getChildCount}, {@link #getChildNode getChildNode} and
     * {@link #getIndexOfChildNode getIndexOfChildNode} have to be implemented
     * in any case.
     *
     * @return  True if index based access is efficient, false if it is
     *          inefficient.
     */
    public boolean isIndexBased()
    {
        return true;
    }

    // }}}

    //----------------------------------------------------------------------
    // Link based
    //----------------------------------------------------------------------
    // {{{ isLinkBased

    /**
     * Query whether the TreeAdapter implementation supports link based child
     * access efficiently. <p>
     *
     * If the list of children of a node is implemented as a linked list for
     * example, link based access is very efficient, while an array based
     * implementation should retrun false. The link based methods {@link
     * #getFirstChildNode getFirstChildNode}, {@link #getLastChildNode
     * getLastChildNode}, {@link #getNextSibling getNextSibling} and {@link
     * #getPreviousSibling getPreviousSibling} have to be implemented in any
     * case.
     *
     * @return  True if link based access is efficient, false if it is
     *          inefficient.
     */
    public boolean isLinkBased()
    {
        return false;
    }

    // }}}
    // {{{ getFirstChildNode

    /**
     * Get the first child of a node.
     *
     * @param   parent  The node whose child to get.
     *
     * @return  The first child of the parent or null if it doesn't have any
     *          children.
     */
    public Object getFirstChildNode(Object parent)
    {
        return getChildNode(parent, 0);
    }

    // }}}
    // {{{ getLastChildNode

    /**
     * Get the last child of a node.
     *
     * @param   parent  The node whose child to get.
     *
     * @return  The last child of the parent or null if it doesn't have any
     *          children.
     */
    public Object getLastChildNode(Object parent)
    {
        return getChildNode(parent, getChildCount(parent) - 1);
    }

    // }}}
    // {{{ getPreviousSibling

    /**
     * Get the previous sibling of a node.
     *
     * @param   node    The node whose sibling to get.
     *
     * @return  The previous sibling or null if the node doesn't have a parent
     *          or is the parent's first child.
     */
    public Object getPreviousSibling(Object node)
    {
        Object parent = getParentNode(node);
        if (parent == null) {
            return null;
        }
        int idx = getIndexOfChildNode(parent, node);
        if (idx <= 0) {
            return null;
        }
        return getChildNode(parent, idx - 1);
    }

    // }}}
    // {{{ getNextSibling

    /**
     * Get the next sibling of a node.
     *
     * @param   node    The node whose sibling to get.
     *
     * @return  The next sibling or null if the node doesn't have a parent
     *          or is the parent's last child.
     */
    public Object getNextSibling(Object node)
    {
        Object parent = getParentNode(node);
        if (parent == null) {
            return null;
        }
        int idx = getIndexOfChildNode(parent, node);
        if (idx < 0 || idx >= getChildCount(parent) - 1) {
            return null;
        }
        return getChildNode(parent, idx + 1);
    }

    // }}}

    //----------------------------------------------------------------------
    // Enumeration based
    //----------------------------------------------------------------------
    // {{{ isEnumerationBased

    /**
     * Query whether the TreeAdapter implementation supports enumeration based
     * child access efficiently. <p>
     *
     * This method should return true only if Enumeration based child access
     * if more efficient than index or linked based access.
     *
     * @return  True if Enumeration based access is efficient, false if it is
     *          inefficient.
     */
    public boolean isEnumerationBased()
    {
        return false;
    }

    // }}}
    // {{{ getChildren

    /**
     * Get an Enumeration for the children of a node. An implementation is
     * free to always return null unless {@link #isEnumerationBased
     * isEnumerationBased} returns true.
     *
     * @param   parent  The node whose children to get.
     *
     * @return  An Enumeration for the children or null, if the node doesn't
     *          have any or Enumeration based access is not supported.
     */
    public Enumeration getChildren(Object parent)
    {
        return new ChildEnumeration (parent);
    }

    // }}}
    // {{{ class ChildEnumeration

    /**
     * Helper class that enumerates the children of a node.
     */
    protected class ChildEnumeration
        implements Enumeration
    {
        // {{{ variables

        /**
         * The parent node.
         */
        private Object parent;

        /**
         * The current enumeration index.
         */
        private int index;

        // }}}

        //------------------------------------------------------------------
        // Constructor
        //------------------------------------------------------------------
        // {{{ ChildEnumeration

        /**
         * Create a new ChildEnumeration.
         *
         * @param       parent  The node whose children to enumerate.
         */
        public ChildEnumeration (Object parent)
        {
            this.parent = parent;
        }

        // }}}

        //------------------------------------------------------------------
        // The Enumeration interface
        //------------------------------------------------------------------
        // {{{ hasMoreElements

        /**
         * Query whether there are more children.
         *
         * @return      True if more children are available.
         */
        public synchronized boolean hasMoreElements()
        {
            return index < getChildCount(parent);
        }

        // }}}
        // {{{ nextElement

        /**
         * Get the next child.
         *
         * @return      The next child.
         */
        public synchronized Object nextElement()
        {
            return getChildNode(parent, index++);
        }

        // }}}
    }

    // }}}
}
