// {{{ copyright

/********************************************************************
 *
 * $Id: TreeAdapter.java,v 1.3 2001/04/20 16:07:24 gs Exp $
 *
 * Copyright (C) 2000 Gregor Schmid, Quality First Software
 * All rights reserved
 *
 *******************************************************************/

// }}}

package de.qfs.lib.tree;

// {{{ imports

import java.util.Enumeration;

// }}}

/**
 * With the help of this interface the methods of the {@link TreeUtil
 * TreeUtil} class can be applied to every kind of tree structure without
 * the need to change anything in the implementation of the structure itself
 * (provided that the basic tree functionality is there). <p>
 *
 * A TreeAdapter can also be used to get different structural views of the
 * same tree hierachy. An Example would be a DOM TreeAdapter that hides all
 * non-Element nodes.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.3 $
 * @since       0.98.1
 */
public interface TreeAdapter
{
    // {{{ getParentNode

    /**
     * Get the parent of a node.
     *
     * @param   child   The node to query for the parent.
     *
     * @return  The parent of the node.
     */
    public Object getParentNode(Object child);

    // }}}
    // {{{ isLeaf

    /**
     * Query whether a node is a leaf.
     *
     * @param   node    The node to query.
     *
     * @return  True if the node is a leaf.
     */
    public boolean isLeaf(Object node);

    // }}}

    //----------------------------------------------------------------------
    // Index based
    //----------------------------------------------------------------------
    // {{{ isIndexBased

    /**
     * Query whether the TreeAdapter implementation supports index based child
     * access efficiently. <p>
     *
     * If the list of children of a node is implemented as a linked list for
     * example, index based access is very inefficient, while an array based
     * implementation should retrun true. The index based methods {@link
     * #getChildCount getChildCount}, {@link #getChildNode getChildNode} and
     * {@link #getIndexOfChildNode getIndexOfChildNode} have to be implemented
     * in any case.
     *
     * @return  True if index based access is efficient, false if it is
     *          inefficient.
     */
    public boolean isIndexBased();

    // }}}
    // {{{ getChildCount

    /**
     * Get the number of children of a node.
     *
     * @param   parent  The node to query.
     *
     * @return  The number of children of the node.
     */
    public int getChildCount(Object parent);

    // }}}
    // {{{ getChildNode

    /**
     * Get a child of a node.
     *
     * @param   parent  The node to query.
     * @param   index   The index of the child.
     *
     * @return  The child at the index.
     *
     * @throws   IndexOutOfBoundsException If the index is invalid.
     */
    public Object getChildNode(Object parent, int index)
         throws IndexOutOfBoundsException;

    // }}}
    // {{{ getIndexOfChildNode

    /**
     * Get the index of a child in a node's list of children.
     *
     * @param   parent  The node to query.
     * @param   child   The child to look for.
     *
     * @return  The index of the child or -1 if it is not a child of this node.
     */
    public int getIndexOfChildNode(Object parent, Object child);

    // }}}

    //----------------------------------------------------------------------
    // Link based
    //----------------------------------------------------------------------
    // {{{ isLinkBased

    /**
     * Query whether the TreeAdapter implementation supports link based child
     * access efficiently. <p>
     *
     * If the list of children of a node is implemented as a linked list for
     * example, link based access is very efficient, while an array based
     * implementation should retrun false. The link based methods {@link
     * #getFirstChildNode getFirstChildNode}, {@link #getLastChildNode
     * getLastChildNode}, {@link #getNextSibling getNextSibling} and {@link
     * #getPreviousSibling getPreviousSibling} have to be implemented in any
     * case.
     *
     * @return  True if link based access is efficient, false if it is
     *          inefficient.
     */
    public boolean isLinkBased();

    // }}}
    // {{{ getFirstChildNode

    /**
     * Get the first child of a node.
     *
     * @param   parent  The node whose child to get.
     *
     * @return  The first child of the parent or null if it doesn't have any
     *          children.
     */
    public Object getFirstChildNode(Object parent);

    // }}}
    // {{{ getLastChildNode

    /**
     * Get the last child of a node.
     *
     * @param   parent  The node whose child to get.
     *
     * @return  The last child of the parent or null if it doesn't have any
     *          children.
     */
    public Object getLastChildNode(Object parent);

    // }}}
    // {{{ getPreviousSibling

    /**
     * Get the previous sibling of a node.
     *
     * @param   node    The node whose sibling to get.
     *
     * @return  The previous sibling or null if the node doesn't have a parent
     *          or is the parent's first child.
     */
    public Object getPreviousSibling(Object node);

    // }}}
    // {{{ getNextSibling

    /**
     * Get the next sibling of a node.
     *
     * @param   node    The node whose sibling to get.
     *
     * @return  The next sibling or null if the node doesn't have a parent
     *          or is the parent's last child.
     */
    public Object getNextSibling(Object node);

    // }}}

    //----------------------------------------------------------------------
    // Enumeration based
    //----------------------------------------------------------------------
    // {{{ isEnumerationBased

    /**
     * Query whether the TreeAdapter implementation supports enumeration based
     * child access efficiently. <p>
     *
     * This method should return true only if Enumeration based child access
     * if more efficient than index or linked based access.
     *
     * @return  True if Enumeration based access is efficient, false if it is
     *          inefficient.
     */
    public boolean isEnumerationBased();

    // }}}
    // {{{ getChildren

    /**
     * Get an Enumeration for the children of a node. An implementation is
     * free to always return null unless {@link #isEnumerationBased
     * isEnumerationBased} returns true.
     *
     * @param   parent  The node whose children to get.
     *
     * @return  An Enumeration for the children or null, if the node doesn't
     *          have any or Enumeration based access is not supported.
     */
    public Enumeration getChildren(Object parent);

    // }}}
}
