// {{{ copyright

/********************************************************************
 *
 * $Id: TreeUtil.java,v 1.2 2001/04/06 15:03:52 gs Exp $
 *
 * Copyright (C) 2000 Gregor Schmid, Quality First Software
 * All rights reserved
 *
 *******************************************************************/

// }}}

package de.qfs.lib.tree;

// {{{ imports

import java.lang.reflect.InvocationTargetException;

import java.util.Enumeration;

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;
import de.qfs.lib.util.Visitor;

// }}}

/**
 * This is a collection of static convenience methods that are useful for tree
 * management. <p>
 *
 * All methods operate on trees in terms of a {@link TreeAdapter TreeAdapter}.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.2 $
 * @since       0.98.1
 */
public class TreeUtil
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger = new Logger (TreeUtil.class);

    // }}}

    //----------------------------------------------------------------------
    // Family business
    //----------------------------------------------------------------------
    // {{{ getRoot(TreeAdapter,Object)

    /**
     * Get the root of a tree hierarchy, i.e. the first ancestor of node that
     * has a null parent.
     *
     * @param   adapter The tree hierarchy adapter.
     * @param   node    The node to start the search from.
     *
     * @return  The root of the hierarchy.
     */
    public static Object getRoot(TreeAdapter adapter, Object node)
    {
        Object parent = adapter.getParentNode(node);
        return parent == null ? node : getRoot(adapter, parent);
    }

    // }}}
    // {{{ getPath(TreeAdapter,Object)

    /**
     * Get the path from a node's root to the node.
     *
     * @param   adapter The tree hierarchy adapter.
     * @param   node    The node whose path to get.
     *
     * @return  An array of the node's ancestors with the root as its first
     *          and the node as its last element.
     */
    public static Object[] getPath(TreeAdapter adapter, Object node)
    {
        // First count the nodes in the path. This is probably faster than
        // using a Vector or List and then copying the result to an array.
        Object tmp = node;
        int i = 1;
        for (; adapter.getParentNode(tmp) != null; i++) {
            tmp = adapter.getParentNode(tmp);
        }
        // Now create the array and fill it.
        Object[] path = new Object[i];
        tmp = node;
        for (i--; i >= 0; i--) {
            path[i] = tmp;
            tmp = adapter.getParentNode(tmp);
        }
        return path;
    }

    // }}}
    // {{{ getPath(TreeAdapter,Object,Object)

    /**
     * Get the relative path from some root to a node.
     *
     * @param   adapter The tree hierarchy adapter.
     * @param   node    The node whose path to get.
     * @param   root    The node to stop the search at
     *
     * @return  An array of the node's ancestors with root as its first
     *          and the node as its last element. If root is not an ancestor
     *          of node, null is returned.
     */
    public static Object[] getPath(TreeAdapter adapter, Object node,
                                   Object root)
    {
        // First count the nodes in the path. This is probably faster than
        // using a Vector or List and then copying the result to an array.
        Object tmp = node;
        int i = 1;
        for (; tmp != null && tmp != root; i++) {
            tmp = adapter.getParentNode(tmp);
        }
        if (tmp == null) {
            // root is not an ancestor of node
            return null;
        }
        // Now create the array and fill it.
        Object[] path = new Object[i];
        tmp = node;
        for (i--; i >= 0; i--) {
            path[i] = tmp;
            tmp = adapter.getParentNode(tmp);
        }
        return path;
    }

    // }}}
    // {{{ getFollowingNode(TreeAdapter,Object)

    /**
     * Get the following node in document order.
     *
     * @param   adapter The tree hierarchy adapter.
     * @param   node    The node to start from.
     *
     * @return  The following node or null if it is the last child.
     */
    public static Object getFollowingNode(TreeAdapter adapter, Object node)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getFollowingNode(TreeAdapter,Object)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "adapter: " + adapter + ", " +
                       "node: " + node);
        }
        if (adapter.getChildCount(node) > 0) {
            return adapter.getFirstChildNode(node);
        }
        while (node != null) {
            Object ret = adapter.getNextSibling(node);
            if (ret != null) {
                return ret;
            }
            node = adapter.getParentNode(node);
        }
        return null;
    }

    // }}}
    // {{{ getPrecedingNode(TreeAdapter,Object)

    /**
     * Get the preceding node in document order.
     *
     * @param   adapter The tree hierarchy adapter.
     * @param   node    The node to start from.
     *
     * @return  The preceding node or null if it is the root node.
     */
    public static Object getPrecedingNode(TreeAdapter adapter, Object node)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getPrecedingNode(TreeAdapter,Object)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "adapter: " + adapter + ", " +
                       "node: " + node);
        }
        Object prev = adapter.getPreviousSibling(node);
        return prev == null ? adapter.getParentNode(node)
            : getLastDescendant(adapter, prev);
    }

    // }}}
    // {{{ getFollowingNode(TreeAdapter,Object)

    /**
     * Get the following node in document order.
     *
     * @param   adapter The tree hierarchy adapter.
     * @param   node    The node to start from.
     * @param   limit   Limit the search to this node and its descendants.
     *
     * @return  The following node or null if it is the last child.
     */
    public static Object getFollowingNode(TreeAdapter adapter, Object node,
                                          Object limit)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getFollowingNode(TreeAdapter,Object)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "adapter: " + adapter + ", " +
                       "node: " + node);
        }
        if (adapter.getChildCount(node) > 0) {
            return adapter.getFirstChildNode(node);
        }
        while (node != null && node != limit) {
            Object ret = adapter.getNextSibling(node);
            if (ret != null) {
                return ret;
            }
            node = adapter.getParentNode(node);
        }
        return null;
    }

    // }}}
    // {{{ getPrecedingNode(TreeAdapter,Object)

    /**
     * Get the preceding node in document order.
     *
     * @param   adapter The tree hierarchy adapter.
     * @param   node    The node to start from.
     * @param   limit   Limit the search to this node and its descendants.
     *
     * @return  The preceding node or null if it is the root node.
     */
    public static Object getPrecedingNode(TreeAdapter adapter, Object node,
                                          Object limit)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getPrecedingNode(TreeAdapter,Object)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "adapter: " + adapter + ", " +
                       "node: " + node);
        }
        if (node == limit) {
            return null;
        }
        Object prev = adapter.getPreviousSibling(node);
        return prev == null ? adapter.getParentNode(node)
            : getLastDescendant(adapter, prev);
       }

    // }}}
    // {{{ getLastDescendant(TreeAdapter,Object)

    /**
     * Get the last descendant of a node.
     *
     * @param   adapter The tree hierarchy adapter.
     * @param   node    The node to start from.
     *
     * @return  The last descendant or the node itself it it doesn't have
     *          children.
     */
    public static Object getLastDescendant(TreeAdapter adapter, Object node)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "getLastDescendant(TreeAdapter,Object)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "adapter: " + adapter + ", " +
                       "node: " + node);
        }
        Object child = adapter.getLastChildNode(node);
        return child == null ? node : getLastDescendant(adapter, child);
    }

    // }}}

    //----------------------------------------------------------------------
    // Copying
    //----------------------------------------------------------------------
    // {{{ copyTree(MutableTreeAdapter,Object)

    /**
     * Copy a whole (sub)tree.
     *
     * @param   adapter The tree hierarchy adapter.
     * @param   node    The root node of the (sub)tree to copy.
     *
     * @return  The copy of the tree.
     */
    public static Object copyTree(MutableTreeAdapter adapter, Object node)
    {
        Object result = adapter.cloneNode(node);
        if (adapter.isIndexBased()) {
            int cnt = adapter.getChildCount(node);
            for (int i = 0; i < cnt; i++) {
                Object copy = copyTree(adapter,
                                       adapter.getChildNode(node, i));
                adapter.addNode(result, copy);
                adapter.setParentNode(copy, result);
            }
        } else if (adapter.isLinkBased()) {
            Object child = adapter.getFirstChildNode(result);
            while (child != null) {
                Object copy = copyTree(adapter, child);
                adapter.addNode(result, copy);
                adapter.setParentNode(copy, result);
                child = adapter.getNextSibling(child);
            }
        } else {
            for (Enumeration enum = adapter.getChildren(result);
                 enum.hasMoreElements(); ) {
                Object copy = copyTree(adapter, enum.nextElement());
                adapter.addNode(result, copy);
                adapter.setParentNode(copy, result);
            }
        }
        return result;
    }

    // }}}

    //----------------------------------------------------------------------
    // Traversal
    //----------------------------------------------------------------------
    // {{{ traverse(TreeAdapter,Object,TraversalCallback)

    /**
     * Traverse a tree and notify a Callback about every node entered and
     * exited on the way. <p>
     *
     * External synchronization is required if multithreaded access to
     * the tree is possible.
     *
     * @param   adapter  The tree hierarchy adapter.
     * @param   node     The node to start the traversal at.
     * @param   callback The Callback to notify.
     *
     * @return This method may also be used to search for something in a
     * tree. The callback method may abort the traversal early by throwing a
     * ReturnFromTraversalException. In that case the return value will be
     * taken from the exception. Otherwise null is returned.
     */
    public static Object traverse(TreeAdapter adapter, Object node,
                                  TraversalCallback callback)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "traverse(TreeAdapter,Object,Callback)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "adapter: " + adapter + ", " +
                       "node: " + node + ", " +
                       "callback: " + callback);
        }
        try {
            new IndexTraversal (adapter, node).traverse(callback);
        } catch (ReturnFromTraversalException ex) {
            return ex.getResult();
        }
        return null;
        // if (adapter.isIndexBased()) {
            // traverseByIndex(adapter, node, callback);
        // } else if (adapter.isLinkBased()) {
            // traverseByLink(adapter, node, callback);
        // } else {
            // traverseByEnumeration(adapter, node, callback);
        // }
    }

    // }}}
    // {{{ traverseFrom(TreeAdapter,Object,Object,TraversalCallback)

    /**
     * Traverse a tree and notify a Callback about every node entered and
     * exited on the way. <p>
     *
     * External synchronization is required if multithreaded access to
     * the tree is possible.
     *
     * @param   adapter  The tree hierarchy adapter.
     * @param   node     The root node for the traversal.
     * @param   start    The node to start the traversal at. All nodes befor
     *                   this one will be skipped. Must be a descendant of
     *                   node, or nothing is traversed.
     * @param   callback The Callback to notify.
     *
     * @return This method may also be used to search for something in a
     * tree. The callback method may abort the traversal early by throwing a
     * ReturnFromTraversalException. In that case the return value will be
     * taken from the exception. Otherwise null is returned.
     */
    public static Object traverseFrom(TreeAdapter adapter, Object node,
                                      Object start, TraversalCallback callback)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD,
                       "traverseFrom(TreeAdapter,Object,Object,Callback)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "adapter: " + adapter + ", " +
                       "node: " + node + ", " +
                       "start: " + start + ", " +
                       "callback: " + callback);
        }

        Object[] path = getPath(adapter, start, node);
        if (path == null) {
            return null;
        }

        try {
            new IndexTraversal (adapter, node).traverseFrom
                (callback,path);
        } catch (ReturnFromTraversalException ex) {
            return ex.getResult();
        }
        return null;
    }

    // }}}
    // {{{ visitTree(TreeAdapter,Object,Object)

    /**
     * Visit every node in a tree.
     *
     * @param   adapter The tree hierarchy adapter.
     * @param   visitor The visitor that visits the nodes.
     * @param   node    The node to start at.
     *
     * @return This method may also be used to search for something in a
     * tree. The visitor may abort the traversal early by throwing a
     * ReturnFromTraversalException. In that case the return value will be
     * taken from the exception. Otherwise null is returned.
     */
    public static Object visitTree(TreeAdapter adapter, Object visitor,
                                   Object node)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "visitTree(TreeAdapter,Object,Object)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "adapter: " + adapter + ", " +
                       "visitor: " + visitor + ", " +
                       "node: " + node);
        }
        try {
            if (adapter.isIndexBased()) {
                visitByIndex(adapter, visitor, node);
            } else if (adapter.isLinkBased()) {
                visitByLink(adapter, visitor, node);
            } else {
                visitByEnumeration(adapter, visitor, node);
            }
        } catch (ReturnFromTraversalException ex) {
            return ex.getResult();
        }
        return null;
    }

    // }}}

    //----------------------------------------------------------------------
    // Helper methods
    //----------------------------------------------------------------------
    // {{{ visitByIndex

    /**
     * Visit a tree and notify a Callback about every node entered and
     * exited on the way. <p>
     *
     * External synchronization is required if multithreaded access to
     * the tree is possible.
     *
     * @param   adapter  The tree hierarchy adapter.
     * @param   node     The node to start the traversal at.
     * @param   callback The Callback to notify.
     */
    private static void visitByIndex(TreeAdapter adapter, Object visitor,
                                     Object node)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "visitByIndex(TreeAdapter,Object,Object)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "adapter: " + adapter + ", " +
                       "visitor: " + visitor + ", " +
                       "node: " + node);
        }
        try {
            Visitor.visit(visitor, node);
        } catch (NoSuchMethodException ex) {
            if (logger.level >= Log.ERR) {
                logger.log("safeVisit(Object,Object)", ex);
            }
        } catch (InvocationTargetException ex) {
            if (ex.getTargetException() instanceof
                ReturnFromTraversalException) {
                throw (ReturnFromTraversalException) ex.getTargetException();
            }
            if (logger.level >= Log.ERR) {
                logger.log("safeVisit(Object,Object)",
                           ex.getTargetException());
            }
        }
        for (int i = 0; i < adapter.getChildCount(node); i++) {
            visitByIndex(adapter, visitor, adapter.getChildNode(node, i));
        }
    }

    // }}}
    // {{{ visitByLink

    /**
     * Visit a tree and notify a Callback about every node entered and
     * exited on the way. <p>
     *
     * External synchronization is required if multithreaded access to
     * the tree is possible.
     *
     * @param   adapter  The tree hierarchy adapter.
     * @param   node     The node to start the traversal at.
     * @param   callback The Callback to notify.
     */
    private static void visitByLink(TreeAdapter adapter, Object visitor,
                                    Object node)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "visitByLink(TreeAdapter,Object,Object)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "adapter: " + adapter + ", " +
                       "visitor: " + visitor + ", " +
                       "node: " + node);
        }
        Visitor.safeVisit(visitor, node);
        Object child = adapter.getFirstChildNode(node);
        while (child != null) {
            visitByLink(adapter, visitor, child);
            child = adapter.getNextSibling(child);
        }
    }

    // }}}
    // {{{ visitByEnumeration

    /**
     * Visit a tree and notify a Callback about every node entered and
     * exited on the way. <p>
     *
     * External synchronization is required if multithreaded access to
     * the tree is possible.
     *
     * @param   adapter  The tree hierarchy adapter.
     * @param   node     The node to start the traversal at.
     * @param   callback The Callback to notify.
     */
    private static void visitByEnumeration(TreeAdapter adapter, Object visitor,
                                           Object node)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD,
                       "visitByEnumeration(TreeAdapter,Object,Object)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "adapter: " + adapter + ", " +
                       "visitor: " + visitor + ", " +
                       "node: " + node);
        }
        Visitor.safeVisit(visitor, node);
        for (Enumeration enum = adapter.getChildren(node);
             enum.hasMoreElements(); ) {
            visitByEnumeration(adapter, visitor, enum.nextElement());
        }
    }

    // }}}
}
