// {{{ copyright

/********************************************************************
 *
 * $Id: ClassWrapper.java,v 1.9 2000/10/26 15:04:14 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.util;

// {{{ imports

import java.util.Hashtable;
import java.util.Vector;

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

// {{{ doc

/**
 * A ClassWrapper is one possible answer to the problem of parallel class
 * hierarchies. <p>
 *
 * These parallel class hierarchies do occur often in OO programming but cause
 * problems with purely statically typed languages, since there is no language
 * construct that maps to the abstract model. <p>
 *
 * This class uses Java's reflection interface to dynamically create wrapper
 * objects that are semantically connected to objects from a parallel class
 * hierarchy. A cache is used internally for speedup of repetitive calls.<p>
 *
 * Wrapper classes are expected to have the same name as the wrapped object's
 * class (without package), plus a suffix. It is not necessary for the
 * wrapping classes to have a hierarchy of the same layout as the wrapped
 * classes.<p>
 *
 * If a wrapper class for an object does not exist, the object's baseclass
 * is tried recursively until class Object is reached.<p>
 *
 * An example:
 *
 * <pre>
 * // Create a ClassWrapper for package a.b.c with suffix Wrapper
 * ClassWrapper wrapper = new ClassWrapper ("a.b.c", "Wrapper");
 * wrapper.makeWrapper (new JTextField);
 * </pre>
 *
 * will first try to create an <code>a.b.c.JTextFieldWrapper<code/>, then an
 * <code>a.b.c.JTextComponentWrapper</code>, and so on until
 * <code>a.b.c.ObjectWrapper</code>. <p>
 *
 * Classes of the parallel hierarchy need not be aware that they are being
 * wrapped, so third party software can be integrated without
 * problems. However, a class that is aware of its wrappers can implement the
 * {@link ClassWrapperAware ClassWrapperAware} interface to speed up the
 * ClassWrapper's lookup mechanism. If wrapper objects are spread over more
 * than one package, additional packages to search for wrappers can be added
 * via {@link #addPackage addPackage}. <p>
 *
 * Caveat: Inner classes cannot be wrapped yet, due to the $ sign in
 * their name.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.9 $
 */

// }}}
public class ClassWrapper
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private static Logger logger = new Logger (ClassWrapper.class);

    /**
     * The packages to search for the classes.
     */
    private Vector packagenames = new Vector ();

    /**
     * The suffix to append to the client's class name.
     */
    private String suffix;

    /**
     * Whether to use the DynamicClassLoader
     */
    private boolean dynamic;

    /**
     * A cache for already located wrappers.
     */
    private Hashtable cache;

    // }}}

    // {{{ constructors

    /**
     * Create a ClassWrapper for objects from a class hierarchy.
     *
     * @param   packagename     The package to search for wrapper classes.
     * @param   suffix          The suffix to append to the wrapped object's
     *                          class name.
     */
    public ClassWrapper (String packagename, String suffix)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "ClassWrapper(String,String)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "packagename: " + packagename + ", " +
                       "suffix: " + suffix);
        }
        packagenames.addElement(packagename);
        this.suffix = suffix;
        cache = new Hashtable ();
    }

    /**
     * Create a ClassWrapper for objects from a class hierarchy.
     *
     * @param   packagename     The package to search for wrapper classes.
     * @param   suffix          The suffix to append to the wrapped object's
     *                          class name.
     * @param   dynamic         If true, use DynamicClassLoader to create
     *                          the wrappers. This will disable the lookup
     *                          cache.
     */
    public ClassWrapper (String packagename, String suffix, boolean dynamic)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "ClassWrapper(String,String,boolean)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "packagename: " + packagename + ", " +
                       "suffix: " + suffix + ", " +
                       "dynamic: " + dynamic);
        }
        packagenames.addElement(packagename);
        this.suffix = suffix;
        this.dynamic = dynamic;
        if (!dynamic) {
            cache = new Hashtable ();
        }
    }

    // }}}

    // {{{ isDynamic

    /**
     * Get the dynamic flag of the ClassWrapper.
     *
     * @return  The dynamic flag of the ClassWrapper.
     *
     * @since   0.98.0
     */
    public final boolean isDynamic()
    {
        return dynamic;
    }

    // }}}
    // {{{ setDynamic

    /**
     * Set the dynamic flag of the ClassWrapper.
     *
     * @param   dynamic The dynamic flag to set.
     *
     * @since   0.98.0
     */
    public final synchronized void setDynamic(boolean dynamic)
    {
        this.dynamic = dynamic;
        if (dynamic && cache == null) {
            cache = new Hashtable ();
        }
    }

    // }}}
    // {{{ makeWrapper

    /**
     * A factory method that dynamically creates a wrapper object for the
     * most derived class or baseclass of the client, for which a wrapper
     * is available.
     *
     * @param   client  The object to wrap.
     *
     * @return  The new wrapper or null, if the client cannot be
     *          wrapped.
     */
    public synchronized Object makeWrapper (Object client)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "makeWrapper(Object)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "client: " + client);
        }

        Class clientClass = client.getClass();
        try {
            Class targetClass = null;
            // first search the cache
            if (! dynamic) {
                targetClass = (Class) cache.get(clientClass);
            }
            // next try the ClassWrapperAware approach
            if (targetClass == null
                && client instanceof ClassWrapperAware) {
                Class cl = ((ClassWrapperAware) client)
                    .getWrapperClass(suffix);
                if (dynamic && cl != null) {
                    String classname = cl.getName();
                    if (classname != null) {
                        try {
                            targetClass =
                                DynamicClassLoader.loadDynamicClass(classname);
                        } catch (ClassNotFoundException ex) {
                            if (logger.level >= Log.ERR) {
                                logger.log(Log.ERR, "makeWrapper(Object)",
                                           "ClassWrapperAware returned " +
                                           "class not found: " + classname);
                                logger.log("makeWrapper(Object)", ex);
                            }
                            return null;
                        }
                    }
                } else if (cl != null) {
                    targetClass = cl;
                    cache.put(clientClass, targetClass);
                }
            }
            while (targetClass == null && clientClass != Object.class) {
                String classname = clientClass.getName();
                int pos = classname.lastIndexOf(".");
                if (pos >= 0) {
                    classname = classname.substring(pos + 1);
                }
                for (int i = 0; i < packagenames.size(); i++) {
                    String target = (String) packagenames.elementAt(i) +
                        "." + classname + suffix;
                    try {
                        if (dynamic) {
                            targetClass = DynamicClassLoader.loadDynamicClass
                                (target);
                        } else {
                            ClassLoader cl = getClass().getClassLoader();
                            if (cl == null) {
                                targetClass = Class.forName(target);
                            } else {
                                targetClass = cl.loadClass(target);
                            }
                            cache.put(clientClass, targetClass);
                        }
                        break;
                    } catch (ClassNotFoundException ex) {
                    } catch (Error ex) {
                        if (logger.level >= Log.ERR) {
                            logger.log("makeWrapper(Object)", ex);
                        }
                    }
                }
                clientClass = clientClass.getSuperclass();
            }
            if (targetClass != null) {
                return targetClass.newInstance();
            }
        } catch (InstantiationException ex) {
            if (logger.level >= Log.ERR) {
                logger.log("makeWrapper(Object)", ex);
            }
        } catch (IllegalAccessException ex) {
            if (logger.level >= Log.ERR) {
                logger.log("makeWrapper(Object)", ex);
            }
        }
        return null;
    }

    // }}}
    // {{{ addPackage

    /**
     * Add a package to be searched for classes. Packages added later are
     * considered more specific and are searched first.
     *
     * @param   packagename     The name of the package to add.
     */
    public synchronized void addPackage(String packagename)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "addPackage(String)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "packagename: " + packagename);
        }
        packagenames.insertElementAt(packagename, 0);
    }

    // }}}
}
