// {{{ copyright

/********************************************************************
 *
 * $Id: DelayedAction.java,v 1.4 2000/06/06 16:41:29 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.util;

// {{{ imports

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Timer;

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

// {{{ doc

/**
 * A DelayedAction is something that is generally useful in event handling,
 * if subsequent events of the same kind cancel each other and the number of
 * pending events is unknown at the time one event is handled. <p>
 *
 * In such a case, handling only the last event can be a big improvement in
 * performance. <p>
 *
 * The DelayedAction works by starting a timer that will execute a Runnable
 * after a certain delay. If another event is received, before the timer has
 * fired, the pending action is dropped and the timer is restarted for the new
 * action. <p>
 *
 * An example application for this pattern is a SelectionListener for a
 * Component, that updates the display of another Component whenever the
 * selection of the first Component changes. If the display of the second
 * Component is complex, e.g. a large table, there will be an annoying delay
 * if e.g. the user uses the arrow keys to navigate through a list or tree
 * component and the display is updated for every element on the way. A slight
 * delay of a few hundred milliseconds (300 is a good value) after the target
 * element has been reached will go almost unnoticed though. <p>
 *
 * Another example use is in a daemon thread that listens for some kind of
 * output from other threads and acts on it. It is often a lot more
 * efficient to collect a few elements from whatever source in a buffer,
 * instead of handling each as soon as it arrives. <p>
 *
 * Warning: As a side effect of using a {@link javax.swing.Timer Timer},
 * execution of the Runnable is happening inside the AWT event loop thread.
 * This is an advantage if any Swing methods are called, but it means that the
 * actions should either not take too long, so as not to affect GUI reactivity,
 * or delegate the task to yet another thread.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.4 $
 */

// }}}
public class DelayedAction
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger = new Logger (DelayedAction.class);

    /**
     * The Timer for the delay.
     */
    private Timer timer;

    /**
     * The action to invoke when the timer fires.
     */
    private Runnable action;

    // }}}

    // {{{ constructor

    /**
     * Create a new DelayedAction.
     *
     * @param   delay   The delay in milliseconds until an action is performed.
     */
    public DelayedAction (int delay)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "DelayedAction(int)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "delay: " + delay);
        }
        timer = new Timer (delay, new ActionListener () {
            public void actionPerformed(ActionEvent e)
            {
                Runnable runnable;
                synchronized (DelayedAction.this) {
                    runnable = action;
                    action = null;
                }
                if (runnable != null) {
                    runnable.run();
                }
            }
        });
        timer.setRepeats(false);
    }

    // }}}

    // {{{ perform

    /**
     * Schedule an action to be performed after the delay. This cancels any
     * pending actions for the DelayedAction.
     *
     * @param   runnable        The Runnable to execute after the delay.
     */
    public synchronized void perform (Runnable runnable)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "perform(Runnable)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "runnable: " + runnable);
        }
        action = runnable;
        timer.restart();
    }

    // }}}
    // {{{ cancel

    /**
     * Cancel a pending action.
     */
    public synchronized void cancel()
    {
        timer.stop();
        action = null;
    }

    // }}}
    // {{{ isActionPending

    /**
     * Query the DelayedAction whether an action is currently pending.
     *
     * @return  True if an action is pending.
     */
    public synchronized boolean isActionPending()
    {
        return action != null;
    }

    // }}}
}
