// {{{ copyright

/********************************************************************
 *
 * $Id: LogSetup.java,v 1.5 2001/01/19 19:12:16 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 2000 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.util;

// {{{ imports

import java.io.IOException;

import de.qfs.lib.log.FileLogWriter;
import de.qfs.lib.log.LevelFilter;
import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;
import de.qfs.lib.log.RingFileLogWriter;
import de.qfs.lib.logrmi.RemoteLogWriter;

import java.util.Hashtable;

// }}}

// {{{ doc

/**
 * This utility class simplifies the setup of the logging system via command
 * line options. It supports a wide range of options that give access to
 * almost every aspect of the logging system. <p>
 *
 * Usage:
 * <pre>
 * // create your {@link ArgsParser ArgsParser} as usual.
 * ArgsParser ap = new ArgsParser (...);
 *
 * // add the logging options
 * LogSetup.instance().addOptions(ap);
 *
 * // parse as usual
 * int restargs = ap.parse(args);
 *
 * // then let the LogSetup do its job
 * LogSetup.instance().setupLogging("myname", ap.getOptions());
 *
 * ...
 *
 * // to clean up and finish:
 * LogSetup.instance().stopLogging();
 * </pre>
 *
 * The following options are handled by LogSetup: <p>
 * <table border="1" cellpadding="2" cellspacing="0">
 * <tr>
 * <th colspan="2" align="left">Logger levels</th>
 * </tr>
 * <tr>
 * <td><code>-log-<em>name</em> <em>value</em></code></td>
 * <td>
 * Set the level of the {@link de.qfs.lib.log.Logger Logger} named
 * <em>name</em> to <em>value</em> (see {@link
 * de.qfs.lib.log.Logger#setLogLevel Logger.setLogLevel}).
 * </td>
 * </tr>
 * <tr>
 * <th colspan="2" align="left">Log class settings</th>
 * </tr>
 * <tr>
 * <td><code>-logoutputlevel <em>value</em></code></td>
 * <td>
 * Set the ouput log level to <em>value</em> (see {@link
 * de.qfs.lib.log.Log#setOutputLevel Log.setOutputLevel}).
 * </td>
 * </tr>
 * <tr>
 * <td><code>-logprequeuelevel <em>value</em></code></td>
 * <td>
 * Set the pre queue level to <em>value</em> (see {@link
 * de.qfs.lib.log.Log#setPreQueueLevel Log.setPreQueueLevel}).
 * </td>
 * </tr>
 * <tr>
 * <td><code>-logqueueing [<em>value</em>]</code></td>
 * <td>
 * Turn on queueing (default is true, use <em>value</em> false to turn off,
 * (see {@link de.qfs.lib.log.Log#setQueueing Log.setQueueing}).
 * </td>
 * </tr>
 * <tr>
 * <td><code>-logqueuesize <em>value</em></code></td>
 * <td>
 * Set the size of the log queue to <em>value</em> (see {@link
 * de.qfs.lib.log.Log#setQueueSize Log.setQueueSize}).
 * </td>
 * </tr>
 * <tr>
 * <td><code>-logqueuedrop [<em>value</em>]</code></td>
 * <td>
 * Set whether messages should be dropped when the queue overflows (default is
 * false, use with great care, see {@link
 * de.qfs.lib.log.Log#setDropOnOverflow Log.setDropOnOverflow}).
 * </td>
 * </tr>
 * <tr>
 * <td><code>-logflushsize <em>value</em></code></td>
 * <td>
 * Set the size of the flush buffer to <em>value</em> (see {@link
 * de.qfs.lib.log.Log#setFlushBufferSize Log.setFlushBufferSize}).
 * </td>
 * </tr>
 * <tr>
 * <td><code>-logflushlevel <em>value</em></code></td>
 * <td>
 * Set the flush trigger level to <em>value</em> (see {@link
 * de.qfs.lib.log.Log#setFlushTriggerLevel Log.setFlushTriggerLevel}).
 * </td>
 * </tr>
 * <tr>
 * <td><code>-logpostflush <em>value</em></code></td>
 * <td>
 * Set the post flush size to <em>value</em> (see {@link
 * de.qfs.lib.log.Log#setPostFlushSize Log.setPostFlushSize}).
 * </td>
 * </tr>
 * <tr>
 * <th colspan="2" align="left">Writing to a file</th>
 * </tr>
 * </tr>
 * <tr>
 * <td><code>-logfile <em>name</em></code></td>
 * <td>
 * Log messages to the file named <em>name</em>.
 * </td>
 * </tr>
 * <tr>
 * <td><code>-logfilelevel <em>value</em></code></td>
 * <td>
 * Write only messages with a level up to <em>value</em> to the log file
 * (default is -1, meaning write all messages).
 * </td>
 * </tr>
 * <tr>
 * <td><code>-logfilesize <em>value</em></code></td>
 * <td>
 * Limit the size of a log file to <em>value</em> kByte. If set, a
 * {@link de.qfs.lib.log.RingFileLogWriter RingFileLogWriter} is used to
 * create the file.
 * </td>
 * </tr>
 * <tr>
 * <td><code>-logfilenum <em>value</em></code></td>
 * <td>
 * Set the number of log files ot keep in the ring (see {@link
 * de.qfs.lib.log.RingFileLogWriter RingFileLogWriter}). Ignored unless
 * <code>-logfilesize</code> is also set.
 * </td>
 * </tr>
 * <tr>
 * <td><code>-logfilemode <em>value</em></code></td>
 * <td>
 * Set the creation mode for the log file. May be one of <code>"create"</code>,
 * <code>"append"</code> or <code>"unique"</code> (see {@link
 * de.qfs.lib.log.FileLogWriter FileLogWriter}, default is
 * <code>"create"</code>). Ignored if <code>-logfilesize</code> is also set.
 * </td>
 * </tr>
 * <tr>
 * <td><code>-logfileclose [<em>value</em>]</code></td>
 * <td>
 * Set whether the log file should be closed after each write (see {@link
 * de.qfs.lib.log.FileLogWriter FileLogWriter}, default is false). Ignored if
 * <code>-logfilesize</code> is also set.
 * </td>
 * </tr>
 * <tr>
 * <th colspan="2" align="left">Communication with the log server
 * <a href="http://www.qfs.de/en/projects/qflog/index.html"><em>qflog</em></a>
 * </th>
 * </tr>
 * <tr>
 * <td><code>-logserver [<em>name</em>]</code></td>
 * <td>
 * Contact the logserver named <em>name</em> (default is <code>"qflog"</code>).
 * </td>
 * </tr>
 * <tr>
 * <td><code>-logregistry [<em>value</em>]</code></td>
 * <td>
 * If true, register with the RMI registry so a log server can connect later
 * on (default is false).
 * </td>
 * </tr>
 * <tr>
 * <td><code>-logport <em>value</em></code></td>
 * <td>
 * Set the port of the RMI registry to use to <em>value</em>. Ignored unless
 * <code>-logregistry</code> is also set.
 * </td>
 * </tr>
 * <tr>
 * <td><code>-logcreateregistry [<em>value</em>]</code></td>
 * <td>
 * Whether to create an RMI registry if none is available (default is false).
 * Ignored unless <code>-logregistry</code> is also set.
 * </td>
 * </tr>
 * <tr>
 * <td><code>-loglocalonly [<em>value</em>]</code></td>
 * <td>
 * Whether to restrict log server connections to localhost (default is true).
 * Ignored unless <code>-logregistry</code> is also set.
 * </td>
 * </tr>
 * </table>
 * <p>
 * <strong>Note:</strong> The log server
 * <a href="http://www.qfs.de/en/projects/qftest/index.html"><em>qflog</em></a>
 * contains the class <code>de.qfs.apps.qflog.logview.LogSetup</code> that
 * extends this one to recognize the option <code>-logview</code> which
 * creates an internal LogView component.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.5 $
 * @since       0.98.0
 */

// }}}
public class LogSetup
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger = new Logger (LogSetup.class);

    /**
     * The singleton instance. Can be set from a derived class.
     */
    protected static LogSetup instance = new LogSetup ();

    /**
     * The options the LogSetup passes to the {@link ArgsParser ArgsParser}.
     */
    public final static String[] OPTIONS = {
        "+log-*",
        "=logcreateregistry",
        "+logfile",
        "=logfileclose",
        "+logfilelevel",
        "+logfilemode",
        "+logfilenum",
        "+logfilesize",
        "+logflushlevel",
        "+logflushsize",
        "=loglocalonly",
        "+logoutputlevel",
        "+logport",
        "+logpostflush",
        "+logprequeuelevel",
        "=logqueuedrop",
        "=logqueueing",
        "+logqueuesize",
        "=logregistry",
        "?logserver",
    };

    /**
     * Default number of files for RingFileLogWriter.
     */
    public final static int DEFAULT_LOG_FILE_NUM = 5;

    /**
     * The LevelFilter for the file writer.
     */
    protected LevelFilter fileFilter;

    /**
     * The LevelFilter for the remote writer.
     */
    protected LevelFilter remoteFilter;

    // }}}

    //----------------------------------------------------------------------
    // Constructor
    //----------------------------------------------------------------------
    // {{{ LogSetup

    /**
     * Protected constructor. Use the {@link #instance instance} method
     * instead.
     */
    protected LogSetup ()
    {
    }

    // }}}

    //----------------------------------------------------------------------
    // Singleton
    //----------------------------------------------------------------------
    // {{{ instance

    /**
     * Get the singleton LogSetup instance.
     *
     * @return  The singleton instance.
     */
    public static LogSetup instance()
    {
        return instance;
    }

    // }}}

    //----------------------------------------------------------------------
    // Public methods
    //----------------------------------------------------------------------
    // {{{ addOptions

    /**
     * Add the LogSetup's options to an ArgsParser, so it will recognize them.
     *
     * @param   parser  The parser to add the options to.
     */
    public void addOptions(ArgsParser parser)
    {
        parser.addOptions(OPTIONS);
    }

    // }}}
    // {{{ setupLogging

    /**
     * Set up the logging system after the {@link ArgsParser ArgsParser}
     * did its job.
     *
     * @param   client  The client name to use for log files and qflog.
     * @param   ap The ArgsParser that parsed the command line.
     */
    public synchronized void setupLogging(String client, ArgsParser ap)
    {
        // set the levels for the Loggers
        Logger.setLogLevels(ap.getOptions());

        // Log options
        Log.setOutputLevel(ap.getInt("logoutputlevel",
                                     Log.DEFAULT_OUTPUT_LEVEL));
        Log.setPreQueueLevel(ap.getInt("logprequeuelevel",
                                       Log.DEFAULT_PRE_QUEUE_LEVEL));
        Log.setQueueing(ap.getBoolean("logqueueing",
                                      Log.DEFAULT_QUEUEING));
        Log.setQueueSize(ap.getInt("logqueuesize",
                                   Log.DEFAULT_QUEUE_SIZE));
        Log.setDropOnOverflow(ap.getBoolean("logqueuedrop",
                                            Log.DEFAULT_DROP_ON_OVERFLOW));
        Log.setFlushBufferSize(ap.getInt("logflushsize",
                                         Log.DEFAULT_FLUSH_BUFFER_SIZE));
        Log.setFlushTriggerLevel(ap.getInt("logflushlevel",
                                           Log.DEFAULT_FLUSH_TRIGGER_LEVEL));
        Log.setPostFlushSize(ap.getInt("logpostflush",
                                       Log.DEFAULT_POST_FLUSH_SIZE));

        // Check for LogFile
        String logfile = ap.getString("logfile", null);
        if (logfile != null) {
            int size = ap.getInt("logfilesize", -1);
            if (size < 0) {
                String smode = ap.getString("logfilemode", "create");
                int mode = FileLogWriter.MODE_CREATE;
                if (smode.equalsIgnoreCase("append")) {
                    mode = FileLogWriter.MODE_APPEND;
                } else if (smode.equalsIgnoreCase("unique")) {
                    mode = FileLogWriter.MODE_UNIQUE;
                }
                try {
                    fileFilter = FileLogWriter.logToFile
                        (client, logfile, mode,
                         ap.getBoolean("logfileclose", true));
                } catch (IOException ex) {
                    if (logger.level >= Log.WRN) {
                        logger.log(Log.WRN,
                                   "setupLogging(String,Hashtable)", ex);
                    }
                }
            } else {
                int num = ap.getInt("logfilenum", DEFAULT_LOG_FILE_NUM);
                try {
                    fileFilter = RingFileLogWriter.logToFile
                        (client, logfile, size, num);
                } catch (IOException ex) {
                    if (logger.level >= Log.WRN) {
                        logger.log(Log.WRN,
                                   "setupLogging(String,Hashtable)", ex);
                    }
                }
            }
        }
        if (fileFilter != null) {
            int filelevel = ap.getInt("logfilelevel", -1);
            if (filelevel >= 0) {
                fileFilter.setLevel(filelevel);
                fileFilter.setFilterMode(fileFilter.FILTER_NONE);
            }
        }

        // Check for RemoteLogWriter
        String logserver = ap.getString("logserver", null);
        if ("".equals(logserver)) {
            logserver = "qflog";
        }
        boolean registry = ap.getBoolean("logregistry", false);
        if (registry) {
            int port = ap.getInt("logport", -1);
            boolean create = ap.getBoolean("logcreateregistry", false);
            boolean local = ap.getBoolean("loglocalonly", true);
            if (logserver != null) {
                try {
                    // logserver and registry
                    remoteFilter = RemoteLogWriter.logRemote
                        (logserver, client, port, create, local);
                } catch (Exception ex) {
                    if (logger.level >= Log.MSG) {
                        logger.log(Log.MSG,
                                   "setupLogging(String,Hashtable)", ex);
                    }
                }
            } else {
                try {
                    // register only
                    remoteFilter = RemoteLogWriter.logRemote
                        (client, port, create, local);
                } catch (Exception ex) {
                    if (logger.level >= Log.WRN) {
                        logger.log(Log.WRN,
                                   "setupLogging(String,Hashtable)", ex);
                    }
                }
            }
        } else if (logserver != null) {
            // logserver only
            try {
                remoteFilter = RemoteLogWriter.logRemote(logserver, client);
            } catch (Exception ex) {
                if (logger.level >= Log.MSG) {
                    logger.log(Log.MSG, "setupLogging(String,Hashtable)", ex);
                }
            }
        }
    }

    // }}}
    // {{{ stopLogging

    /**
     * Stop the logging system: call {@link de.qfs.lib.log.Log#setQueueing
     * Log.setQueueing(false)} to flush pending messages and close open log
     * files and disconnect from the log server as necessary.
     */
    public synchronized void stopLogging()
    {
        Log.setQueueing(false);
        if (fileFilter != null) {
            fileFilter.getLogWriter().close();
            Log.removeFilter(fileFilter);
            fileFilter = null;
        }
        if (remoteFilter != null) {
            remoteFilter.getLogWriter().close();
            Log.removeFilter(remoteFilter);
            remoteFilter = null;
        }
    }

    // }}}
    // {{{ getFileFilter

    /**
     * Get the LevelFilter returned from {@link
     * de.qfs.lib.log.FileLogWriter#logToFile FileLogWriter.logToFile} or
     * {@link de.qfs.lib.log.RingFileLogWriter#logToFile
     * RingFileLogWriter.logToFile}.
     *
     * @return  The LevelFilter used for logging to a file.
     */
    public LevelFilter getFileFilter()
    {
        return fileFilter;
    }

    // }}}
    // {{{ getRemoteFilter

    /**
     * Get the LevelFilter returned from {@link
     * de.qfs.lib.logrmi.RemoteLogWriter#logRemote RemoteLogWriter.logRemote}.
     *
     * @return  The LevelFilter used for logging to the log server.
     */
    public LevelFilter getRemoteFilter()
    {
        return remoteFilter;
    }

    // }}}

    //----------------------------------------------------------------------
    // Helper methods
    //----------------------------------------------------------------------
}
