// {{{ copyright

/********************************************************************
 *
 * $Id: Reflector.jpp,v 1.3 2001/04/21 19:55:05 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 2000 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.util;

// {{{ imports

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

/**
 * This class holds a collection of static methods for convenient access to
 * the reflection API.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.3 $
 * @since       0.98.0
 */
public abstract class Reflector
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger = new Logger (Reflector.class);

    // }}}

    //----------------------------------------------------------------------
    // Call a method
    //----------------------------------------------------------------------
    // {{{ call(Object,String,Class[],Object[],boolean)

    /**
     * Call a method of an object.
     *
     * @param   instance        The object whose method to call.
     * @param   name            The name of the method.
     * @param   parameterTypes  The classes of the method parameters.
     * @param   parameterValues The parameter values.
     * @param   forceAccess     Whether to force access to a normally
     *                          inaccessible method via the {@link
     *                          java.lang.reflect.AccessibleObject#setAccessible
     *                          AccessibleObject.setAccessible}
     *                          method (only works with JDK 1.2 and above).
     *
     * @return  The result of the method call.
     *
     * @throws  IllegalAccessException  If the method cannot be accessed.
     * @throws  NoSuchMethodException   If no such method exists.
     * @throws  InvocationTargetException       If execution of the method
     *                                          throws an exception.
     */
    public static Object call(Object instance, String name,
                              Class[] parameterTypes,
                              Object[] parameterValues,
                              boolean forceAccess)
        throws IllegalAccessException, NoSuchMethodException,
               InvocationTargetException
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "call(Object,String,Class[],Object[],boolean)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "instance: " + instance + ", " +
                       "name: " + name + ", " +
                       "forceAccess: " + forceAccess);
        }
        Class cl = instance.getClass();

        if (forceAccess) {
            // This is more complicated. We cannot use getMethod, since this
            // won't return a private member of super class
            while (true) {
                try {
                    Method method = cl.getDeclaredMethod(name, parameterTypes);
                    boolean accessible = method.isAccessible();
                    try {
                        if (! accessible) {
                            method.setAccessible(true);
                        }
                        return method.invoke(instance, parameterValues);
                    } finally {
                        if (! accessible) {
                            method.setAccessible(false);
                        }
                    }
                } catch (NoSuchMethodException ex) {
                    if (cl == Object.class) {
                        throw ex;
                    }
                    cl = cl.getSuperclass();
                }
            }
        }

        return cl.getMethod(name, parameterTypes)
            .invoke(instance, parameterValues);
    }

    // }}}
    // {{{ call(Object,String,Class[],Object[])

    /**
     * Call a method of an object.
     *
     * @param   instance        The object whose method to call.
     * @param   name            The name of the method.
     * @param   parameterTypes  The classes of the method parameters.
     * @param   parameterValues The parameter values.
     *
     * @return  The result of the method call.
     *
     * @throws  IllegalAccessException  If the method cannot be accessed.
     * @throws  NoSuchMethodException   If no such method exists.
     * @throws  InvocationTargetException       If execution of the method
     *                                          throws an exception.
     */
    public static Object call(Object instance, String name,
                              Class[] parameterTypes,
                              Object[] parameterValues)
        throws IllegalAccessException, NoSuchMethodException,
               InvocationTargetException
    {
        return call(instance, name, parameterTypes, parameterValues, false);
    }

    // }}}
    // {{{ call(Object,String,boolean)

    /**
     * Call a no args method of an object.
     *
     * @param   instance        The object whose method to call.
     * @param   name            The name of the method.
     * @param   forceAccess     Whether to force access to a normally
     *                          inaccessible method via the {@link
     *                          java.lang.reflect.AccessibleObject#setAccessible
     *                          AccessibleObject.setAccessible}
     *                          method (only works with JDK 1.2 and above).
     *
     * @return  The result of the method call.
     *
     * @throws  IllegalAccessException  If the method cannot be accessed.
     * @throws  NoSuchMethodException   If no such method exists.
     * @throws  InvocationTargetException       If execution of the method
     *                                          throws an exception.
     */
    public static Object call(Object instance, String name,
                              boolean forceAccess)
        throws IllegalAccessException, NoSuchMethodException,
               InvocationTargetException
    {
        return call(instance, name, (Class[]) null, null, forceAccess);
    }

    // }}}
    // {{{ call(Object,String)

    /**
     * Call a no args method of an object.
     *
     * @param   instance        The object whose method to call.
     * @param   name            The name of the method.
     *
     * @return  The result of the method call.
     *
     * @throws  IllegalAccessException  If the method cannot be accessed.
     * @throws  NoSuchMethodException   If no such method exists.
     * @throws  InvocationTargetException       If execution of the method
     *                                          throws an exception.
     */
    public static Object call(Object instance, String name)
        throws IllegalAccessException, NoSuchMethodException,
               InvocationTargetException
    {
        return call(instance, name, (Class[]) null, null, false);
    }

    // }}}
    // {{{ call(Object,String,Class,Object,boolean)

    /**
     * Call a one arg method of an object.
     *
     * @param   instance        The object whose method to call.
     * @param   name            The name of the method.
     * @param   parameterType   The class of the method parameter.
     * @param   parameterValue  The parameter value.
     * @param   forceAccess     Whether to force access to a normally
     *                          inaccessible method via the {@link
     *                          java.lang.reflect.AccessibleObject#setAccessible
     *                          AccessibleObject.setAccessible}
     *                          method (only works with JDK 1.2 and above).
     *
     * @return  The result of the method call.
     *
     * @throws  IllegalAccessException  If the method cannot be accessed.
     * @throws  NoSuchMethodException   If no such method exists.
     * @throws  InvocationTargetException       If execution of the method
     *                                          throws an exception.
     */
    public static Object call(Object instance, String name,
                              Class parameterType,
                              Object parameterValue,
                              boolean forceAccess)
        throws IllegalAccessException, NoSuchMethodException,
               InvocationTargetException
    {
        return call(instance, name, new Class[] {parameterType},
                    new Object[] {parameterValue}, forceAccess);
    }

    // }}}
    // {{{ call(Object,String,Class,Object)

    /**
     * Call a one arg method of an object.
     *
     * @param   instance        The object whose method to call.
     * @param   name            The name of the method.
     * @param   parameterType   The class of the method parameter.
     * @param   parameterValue  The parameter value.
     *
     * @return  The result of the method call.
     *
     * @throws  IllegalAccessException  If the method cannot be accessed.
     * @throws  NoSuchMethodException   If no such method exists.
     * @throws  InvocationTargetException       If execution of the method
     *                                          throws an exception.
     */
    public static Object call(Object instance, String name,
                              Class parameterType,
                              Object parameterValue)
        throws IllegalAccessException, NoSuchMethodException,
               InvocationTargetException
    {
        return call(instance, name, new Class[] {parameterType},
                    new Object[] {parameterValue}, false);
    }

    // }}}
    // {{{ safeCall(Object,String,Class[],Object[],boolean)

    /**
     * Call a method of an object. Exceptions are caught and logged at level
     * {@link de.qfs.lib.log.Log#DBG Log.DBG}.
     *
     * @param   instance        The object whose method to call.
     * @param   name            The name of the method.
     * @param   parameterTypes  The classes of the method parameters.
     * @param   parameterValues The parameter values.
     * @param   forceAccess     Whether to force access to a normally
     *                          inaccessible method via the {@link
     *                          java.lang.reflect.AccessibleObject#setAccessible
     *                          AccessibleObject.setAccessible}
     *                          method (only works with JDK 1.2 and above).
     *
     * @return The result of the method call or null if an Exception is
     * caught.
     */
    public static Object safeCall(Object instance, String name,
                                  Class[] parameterTypes,
                                  Object[] parameterValues,
                                  boolean forceAccess)
    {
        try {
            return call(instance, name, parameterTypes, parameterValues,
                        forceAccess);
        } catch (Exception ex) {
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG,
                           "safeCall(Object,String,Class[],Object[],boolean)",
                           ex);
            }
            return null;
        }
    }

    // }}}
    // {{{ safeCall(Object,String,Class[],Object[])

    /**
     * Call a method of an object. Exceptions are caught and logged at level
     * {@link de.qfs.lib.log.Log#DBG Log.DBG}.
     *
     * @param   instance        The object whose method to call.
     * @param   name            The name of the method.
     * @param   parameterTypes  The classes of the method parameters.
     * @param   parameterValues The parameter values.
     *
     * @return The result of the method call or null if an Exception is
     * caught.
     */
    public static Object safeCall(Object instance, String name,
                                  Class[] parameterTypes,
                                  Object[] parameterValues)
    {
        try {
            return call(instance, name, parameterTypes, parameterValues,
                        false);
        } catch (Exception ex) {
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG,
                           "safeCall(Object,String,Class[],Object[])",
                           ex);
            }
            return null;
        }
    }

    // }}}
    // {{{ safeCall(Object,String,boolean)

    /**
     * Call a no args method of an object. Exceptions are caught and logged at
     * level {@link de.qfs.lib.log.Log#DBG Log.DBG}.
     *
     * @param   instance        The object whose method to call.
     * @param   name            The name of the method.
     * @param   forceAccess     Whether to force access to a normally
     *                          inaccessible method via the {@link
     *                          java.lang.reflect.AccessibleObject#setAccessible
     *                          AccessibleObject.setAccessible}
     *                          method (only works with JDK 1.2 and above).
     *
     * @return The result of the method call or null if an Exception is
     * caught.
     */
    public static Object safeCall(Object instance, String name,
                                  boolean forceAccess)
    {
        try {
            return call(instance, name, (Class[]) null, null, forceAccess);
        } catch (Exception ex) {
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG,
                           "safeCall(Object,String,boolean)",
                           ex);
            }
            return null;
        }
    }

    // }}}
    // {{{ safeCall(Object,String)

    /**
     * Call a no args method of an object. Exceptions are caught and logged at
     * level {@link de.qfs.lib.log.Log#DBG Log.DBG}.
     *
     * @param   instance        The object whose method to call.
     * @param   name            The name of the method.
     *
     * @return The result of the method call or null if an Exception is
     * caught.
     */
    public static Object safeCall(Object instance, String name)
    {
        try {
            return call(instance, name, (Class[]) null, null, false);
        } catch (Exception ex) {
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG,
                           "safeCall(Object,String)",
                           ex);
            }
            return null;
        }
    }

    // }}}
    // {{{ safeCall(Object,String,Class,Object,boolean)

    /**
     * Call a one arg method of an object. Exceptions are caught and logged at
     * level {@link de.qfs.lib.log.Log#DBG Log.DBG}.
     *
     * @param   instance        The object whose method to call.
     * @param   name            The name of the method.
     * @param   parameterType   The class of the method parameter.
     * @param   parameterValue  The parameter value.
     * @param   forceAccess     Whether to force access to a normally
     *                          inaccessible method via the {@link
     *                          java.lang.reflect.AccessibleObject#setAccessible
     *                          AccessibleObject.setAccessible}
     *                          method (only works with JDK 1.2 and above).
     *
     * @return The result of the method call or null if an Exception is
     * caught.
     */
    public static Object safeCall(Object instance, String name,
                                  Class parameterType,
                                  Object parameterValue,
                                  boolean forceAccess)
    {
        try {
            return call(instance, name, new Class[] {parameterType},
                        new Object[] {parameterValue}, forceAccess);
        } catch (Exception ex) {
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG,
                           "safeCall(Object,String,Class,Object,boolean)",
                           ex);
            }
            return null;
        }
    }

    // }}}
    // {{{ safeCall(Object,String,Class,Object,boolean)

    /**
     * Call a one arg method of an object. Exceptions are caught and logged at
     * level {@link de.qfs.lib.log.Log#DBG Log.DBG}.
     *
     * @param   instance        The object whose method to call.
     * @param   name            The name of the method.
     * @param   parameterType   The class of the method parameter.
     * @param   parameterValue  The parameter value.
     *
     * @return The result of the method call or null if an Exception is
     * caught.
     */
    public static Object safeCall(Object instance, String name,
                                  Class parameterType,
                                  Object parameterValue)
    {
        try {
            return call(instance, name, new Class[] {parameterType},
                        new Object[] {parameterValue}, false);
        } catch (Exception ex) {
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG,
                           "safeCall(Object,String,Class,Object)",
                           ex);
            }
            return null;
        }
    }

    // }}}

    //----------------------------------------------------------------------
    // Get an attribute value
    //----------------------------------------------------------------------
    // {{{ get(Object,String,boolean)

    /**
     * Get a field value from an object.
     *
     * @param   instance        The object whose field to get.
     * @param   name            The name of the field.
     * @param   forceAccess     Whether to force access to a normally
     *                          inaccessible field via the {@link
     *                          java.lang.reflect.AccessibleObject#setAccessible
     *                          AccessibleObject.setAccessible}
     *                          method (only works with JDK 1.2 and above).
     *
     * @return  The value of the field.
     *
     * @throws  IllegalAccessException  If the method cannot be accessed.
     * @throws  NoSuchFieldException    If no such field exists.
     * @throws  InvocationTargetException       If execution of the method
     *                                          throws an exception.
     */
    public static Object get(Object instance, String name,
                             boolean forceAccess)
        throws IllegalAccessException, NoSuchFieldException,
               InvocationTargetException
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "get(Object,String,boolean)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "instance: " + instance + ", " +
                       "name: " + name + ", " +
                       "forceAccess: " + forceAccess);
        }
        Class cl = instance.getClass();

        if (forceAccess) {
            // This is more complicated. We cannot use getField, since this
            // won't return a private member of super class
            while (true) {
                try {
                    Field field = cl.getDeclaredField(name);
                    boolean accessible = field.isAccessible();
                    try {
                        if (! accessible) {
                            field.setAccessible(true);
                        }
                        return field.get(instance);
                    } finally {
                        if (! accessible) {
                            field.setAccessible(false);
                        }
                    }
                } catch (NoSuchFieldException ex) {
                    if (cl == Object.class) {
                        throw ex;
                    }
                    cl = cl.getSuperclass();
                }
            }
        }

        return cl.getField(name).get(instance);
    }

    // }}}
    // {{{ get(Object,String)

    /**
     * Get a field value from an object.
     *
     * @param   instance        The object whose field to get.
     * @param   name            The name of the field.
     *
     * @return  The value of the field.
     *
     * @throws  IllegalAccessException  If the method cannot be accessed.
     * @throws  NoSuchFieldException    If no such field exists.
     * @throws  InvocationTargetException       If execution of the method
     *                                          throws an exception.
     */
    public static Object get(Object instance, String name)
        throws IllegalAccessException, NoSuchFieldException,
               InvocationTargetException
    {
        return get(instance, name, false);
    }

    // }}}
    // {{{ safeGet(Object,String,boolean)

    /**
     * Get a field value from an object. Exceptions are caught and logged at
     * level {@link de.qfs.lib.log.Log#DBG Log.DBG}.
     *
     * @param   instance        The object whose field to get.
     * @param   name            The name of the field.
     * @param   forceAccess     Whether to force access to a normally
     *                          inaccessible field via the {@link
     *                          java.lang.reflect.AccessibleObject#setAccessible
     *                          AccessibleObject.setAccessible}
     *                          method (only works with JDK 1.2 and above).
     *
     * @return  The value of the field or null if an Exception is caught.
     */
    public static Object safeGet(Object instance, String name,
                                 boolean forceAccess)
    {
        try {
            return get(instance, name, forceAccess);
        } catch (Exception ex) {
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG, "get(Object,String,boolean)", ex);
            }
            return null;
        }
    }

    // }}}
    // {{{ safeGet(Object,String)

    /**
     * Get a field value from an object. Exceptions are caught and logged at
     * level {@link de.qfs.lib.log.Log#DBG Log.DBG}.
     *
     * @param   instance        The object whose field to get.
     * @param   name            The name of the field.
     *
     * @return  The value of the field or null if an Exception is caught.
     */
    public static Object safeGet(Object instance, String name)
    {
        try {
            return get(instance, name, false);
        } catch (Exception ex) {
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG, "get(Object,String)", ex);
            }
            return null;
        }
    }

    // }}}

    //----------------------------------------------------------------------
    // Set an attribute value
    //----------------------------------------------------------------------
    // {{{ set(Object,String,Object,boolean)

    /**
     * Set a field value in an object.
     *
     * @param   instance        The object whose field to get.
     * @param   name            The name of the field.
     * @param   value           The value to set.
     * @param   forceAccess     Whether to force access to a normally
     *                          inaccessible field via the {@link
     *                          java.lang.reflect.AccessibleObject#setAccessible
     *                          AccessibleObject.setAccessible}
     *                          method (only works with JDK 1.2 and above).
     *
     * @throws  IllegalAccessException  If the method cannot be accessed.
     * @throws  NoSuchFieldException    If no such field exists.
     * @throws  InvocationTargetException       If execution of the method
     *                                          throws an exception.
     */
    public static void set(Object instance, String name,
                           Object value, boolean forceAccess)
        throws IllegalAccessException, NoSuchFieldException,
               InvocationTargetException
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "set(Object,String,Object,boolean)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "instance: " + instance + ", " +
                       "name: " + name + ", " +
                       "value: " + value + ", " +
                       "forceAccess: " + forceAccess);
        }
        Class cl = instance.getClass();

        if (forceAccess) {
            // This is more complicated. We cannot use getField, since this
            // won't return a private member of super class
            while (true) {
                try {
                    Field field = cl.getDeclaredField(name);
                    boolean accessible = field.isAccessible();
                    try {
                        if (! accessible) {
                            field.setAccessible(true);
                        }
                        field.set(instance, value);
                        return;
                    } finally {
                        if (! accessible) {
                            field.setAccessible(false);
                        }
                    }
                } catch (NoSuchFieldException ex) {
                    if (cl == Object.class) {
                        throw ex;
                    }
                    cl = cl.getSuperclass();
                }
            }
        }

        cl.getField(name).set(instance, value);
    }

    // }}}
    // {{{ set(Object,String,Object)

    /**
     * Set a field value in an object.
     *
     * @param   instance        The object whose field to get.
     * @param   name            The name of the field.
     * @param   value           The value to set.
     *
     * @throws  IllegalAccessException  If the method cannot be accessed.
     * @throws  NoSuchFieldException    If no such field exists.
     * @throws  InvocationTargetException       If execution of the method
     *                                          throws an exception.
     */
    public static void set(Object instance, String name,
                           Object value)
        throws IllegalAccessException, NoSuchFieldException,
               InvocationTargetException
    {
        set(instance, name, value, false);
    }

    // }}}
    // {{{ safeSet(Object,String,Object,boolean)

    /**
     * Set a field value in an object. Exceptions are caught and logged at
     * level {@link de.qfs.lib.log.Log#DBG Log.DBG}.
     *
     * @param   instance        The object whose field to get.
     * @param   name            The name of the field.
     * @param   value           The value to set.
     * @param   forceAccess     Whether to force access to a normally
     *                          inaccessible field via the {@link
     *                          java.lang.reflect.AccessibleObject#setAccessible
     *                          AccessibleObject.setAccessible}
     *                          method (only works with JDK 1.2 and above).
     */
    public static void safeSet(Object instance, String name,
                               Object value, boolean forceAccess)
    {
        try {
            set(instance, name, value, forceAccess);
        } catch (Exception ex) {
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG,
                           "safeSet(Object,String,Object,boolean)", ex);
            }
        }
    }

    // }}}
    // {{{ safeSet(Object,String,Object)

    /**
     * Set a field value in an object. Exceptions are caught and logged at
     * level {@link de.qfs.lib.log.Log#DBG Log.DBG}.
     *
     * @param   instance        The object whose field to get.
     * @param   name            The name of the field.
     * @param   value           The value to set.
     */
    public static void safeSet(Object instance, String name,
                               Object value)
    {
        try {
            set(instance, name, value, false);
        } catch (Exception ex) {
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG,
                           "safeSet(Object,String,Object)", ex);
            }
        }
    }

    // }}}
}
