// {{{ copyright

/********************************************************************
 *
 * $Id: StablePipedOutputStream.java,v 1.4 2000/06/06 17:00:04 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.util;

// {{{ imports

import java.io.PipedInputStream;
import java.io.IOException;
import java.io.PipedOutputStream;

// }}}

/**
 * A StablePipedOutputStream extends PipedOutputStream to pass all data
 * written to it through the same thread to avoid broken pipes when writing
 * within a temporary thread.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.4 $
 */
public class StablePipedOutputStream extends PipedOutputStream
{
    // {{{ variables

    /**
     * Buffer for new data.
     */
    private byte[] buf;

    /**
     * Determines lifetime of the handler thread.
     */
    private boolean cont = true;

    /**
     * Signal data to write.
     */
    private boolean finished = true;

    /**
     * Used to pass IOExceptions out of the handler thread.
     */
    private IOException exception;

    // }}}

    // {{{ constructors

    /**
     * Create a new StablePipedOutputStream.
     */
    public StablePipedOutputStream ()
    {
        new Thread () {
            public void run () {
                StablePipedOutputStream.this.run();
            }
        }.start();
    }

    /**
     * Create a new StablePipedOutputStream for a PipedInputStream.
     *
     * @param   sink    The target for the data.
     *
     * @throws  IOException     If an I/O error occurs.
     */
    public StablePipedOutputStream (PipedInputStream sink)
         throws IOException
    {
        super (sink);
        new Thread () {
            public void run () {
                StablePipedOutputStream.this.run();
            }
        }.start();
    }

    // }}}

    //----------------------------------------------------------------------
    // Overwrite some PipedOutputStream methods
    //----------------------------------------------------------------------
    // {{{ write(byte[], int, int)

    /**
     * Write len bytes from the specified byte array starting at
     * offset off to this output stream.
     *
     * @param   b       The array.
     * @param   off     The offset.
     * @param   len     The number of bytes.
     *
     * @throws IOException  If an I/O error occurs.
     */
    public synchronized void write(byte[] b, int off, int len)
         throws IOException
    {
        while (buf != null) {
            try {
                wait();
            } catch (InterruptedException ex) {
            }
        }
        buf = new byte[len];
        System.arraycopy(b, off, buf, 0, len);
        finished = false;
        notifyAll();
        while (!finished) {
            try {
                wait();
            } catch (InterruptedException ex) {
            }
        }
        buf = null;
        IOException tmp = null;;
        if (exception != null) {
            tmp = (IOException) exception.fillInStackTrace();
            exception = null;
        }
        notifyAll();
        if (tmp != null) {
            throw tmp;
        }
    }

    // }}}
    // {{{ write(int)

    /**
     * Write the specified byte to this output stream.
     *
     * @param   b       The byte to write.
     *
     * @throws IOException  If an I/O error occurs.
     */
    public synchronized void write(int b)
         throws IOException
    {
        while (buf != null) {
            try {
                wait();
            } catch (InterruptedException ex) {
            }
        }
        buf = new byte[] {(byte) b};
        finished = false;
        notifyAll();
        while (!finished) {
            try {
                wait();
            } catch (InterruptedException ex) {
            }
        }
        buf = null;
        IOException tmp = null;;
        if (exception != null) {
            tmp = (IOException) exception.fillInStackTrace();
            exception = null;
        }
        notifyAll();
        if (tmp != null) {
            throw tmp;
        }
    }

    // }}}
    // {{{ close

    /**
     * Close this output stream and release any system resources
     * associated with this stream.
     *
     * @throws IOException  If an I/O error occurs.
     */
    public synchronized void close()
         throws IOException
    {
        cont = false;
        Thread.currentThread().interrupt();
        super.close();
    }

    // }}}

    //----------------------------------------------------------------------
    // Helper methods
    //----------------------------------------------------------------------
    // {{{ run

    private synchronized void run()
    {
        while (cont) {
            while (finished) {
                try {
                    wait();
                } catch (InterruptedException ex) {
                }
            }
            try {
                super.write(buf, 0, buf.length);
            } catch (IOException ex) {
                exception = ex;
            }
            finished = true;
            notifyAll();
        }
    }

    // }}}
}
