// {{{ copyright

/********************************************************************
 *
 * $Id: TaskQueue.jpp,v 1.5 2001/02/17 09:41:32 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.util;

// {{{ imports


import java.util.LinkedList;



import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;


import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

/**
 * A TaskQueue is a Thread that executes runnables from a queue. <p>
 *
 * It is used to handle related tasks, one after the other, from a single
 * background thread. New tasks can be added with {@link #addTask addTask}
 * and the thread can be terminated with the {@link #finish finish} method.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.5 $
 */
public class TaskQueue
    extends Thread
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger = new Logger (TaskQueue.class);

    /**
     * Counter for thread names.
     */
    private static int id = 0;

    /**
     * The queue of runnables.
     */
    private LinkedList queue = new LinkedList ();

    /**
     * Flag to terminate the TaskQueue.
     */
    private int terminated = 0;

    // }}}

    // {{{ constructors

    /**
     * Create a new TaskQueue. This does not start the TaskQueue.
     */
    public TaskQueue ()
    {
        super("TaskQueue" + id++);
    }

    /**
     * Create a new TaskQueue. This does not start the TaskQueue.
     *
     * @param   name    The thread name for the TaskQueue.
     */
    public TaskQueue (String name)
    {
        super(name);
    }

    // }}}

    // {{{ run

    /**
     * Remove tasks from the queue one by one and execute them.
     */
    public void run()
    {
        while(terminated < 2) {
            Runnable task = null;
            synchronized (this) {
                while (terminated < 2 && queue.isEmpty()) {
                    try {
                        wait();
                    } catch (InterruptedException ex) {
                    }
                }
                if (terminated < 2 && ! queue.isEmpty()) {
                    task = (Runnable) queue.removeFirst();
                    if (queue.isEmpty() && terminated == 1) {
                        terminated = 2;
                    }
                }
            }
            if (task != null) {
                try {
                    task.run();
                } catch (Throwable ex) {
                    if (logger.level >= Log.ERR) {
                        logger.log("run()", ex);
                    }
                }
                task = null;
            }
        }
    }

    // }}}
    // {{{ finish

    /**
     * Terminate the operation of the TaskQueue.
     *
     * @param immediately       If true, terminate the TaskQueue as soon as
     *                          possible. Otherwise no more new tasks will be
     *                          accepted and the thread will be terminated
     *                          when all pending tasks have been executed.
     */
    public synchronized void finish(boolean immediately)
    {
        terminated = (immediately || queue.isEmpty()) ? 2 : 1;
        notifyAll();
    }

    // }}}
    // {{{ addTask

    /**
     * Add a task to the TaskQueue. It will be executed as soon as possible.
     *
     * @param   task    The Runnable to add.
     */
    public synchronized void addTask(Runnable task)
    {
        if (terminated == 0) {
            queue.addLast(task);
        }
        notifyAll();
    }

    // }}}

    //----------------------------------------------------------------------
    // Tests
    //----------------------------------------------------------------------

    // {{{ class UnitTest

    /**
     * Test cases for the TaskQueue class.
     *
     * XXX Need to design better tests as these don't work reliably with
     * native threads.
     */
    public static class UnitTest extends TestCase
    {
        // {{{ variables

        /**
         * The log level hierarchy.
         */
        private TaskQueue taskQueue;;

        // }}}

        // {{{ constructor

        /**
         * Create a new UnitTest.
         *
         * @param       name    The name of the test.
         */
        public UnitTest(String name)
        {
                super(name);
        }

        // }}}

        // {{{ setUp

        /**
         * Set up the test fixture.
         */
        protected void setUp()
        {
            taskQueue = new TaskQueue ("TestQueue");
        }

        // }}}
        // {{{ suite

        /**
         * Get the suite of tests.
         *
         * @return      The tests.
         */
        public static Test suite()
        {
                return new TestSuite(UnitTest.class);
        }

        // }}}

        // {{{ testHighPriority

        /**
         * Test TaskQueue with high priority.
         */
        public void XXXtestHighPriority()
        {
            taskQueue.setPriority(Thread.MAX_PRIORITY);

            TestTask task = new TestTask ();

            taskQueue.addTask(task);
            assert(task.count == 0);

            taskQueue.start();
            Thread.currentThread().yield();
            assert(task.count == 1);

            taskQueue.addTask(task);
            assert(task.count == 2);

            taskQueue.finish(true);
            assert(! taskQueue.isAlive());
        }

        // }}}
        // {{{ testLowPriority

        /**
         * Test TaskQueue with low priority.
         */
        public void XXXtestLowPriority()
        {
            taskQueue.setPriority(Thread.MIN_PRIORITY);

            TestTask task = new TestTask ();

            taskQueue.addTask(task);
            assert(task.count == 0);

            taskQueue.start();
            assert(task.count == 0);

            taskQueue.addTask(task);
            assert(task.count == 0);

            taskQueue.finish(false);
            assert(taskQueue.isAlive());

            try {
                taskQueue.join();
            } catch (InterruptedException ex) {
            }
            assert(task.count == 2);
        }

        // }}}
        // {{{ testTerminate

        /**
         * Test immediate termination of TaskQueue with low priority.
         */
        public void XXXtestTerminate()
        {
            taskQueue.setPriority(Thread.MIN_PRIORITY);

            TestTask task = new TestTask ();

            taskQueue.addTask(task);
            assert(task.count == 0);

            taskQueue.start();
            assert(task.count == 0);

            taskQueue.addTask(task);
            assert(task.count == 0);

            taskQueue.finish(true);
            assert(taskQueue.isAlive());

            try {
                taskQueue.join();
            } catch (InterruptedException ex) {
            }
            assert(task.count == 0);
        }

        // }}}

        // {{{ additional classes

        private class TestTask
            implements Runnable
        {
            public int count = 0;

            public TestTask ()
            {
            }

            public void run()
            {
                count++;
            }
        }

        // }}}
    }

    // }}}

}
