// {{{ copyright

/********************************************************************
 *
 * $Id: ThreadPool.jpp,v 1.4 2000/06/06 16:41:29 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.util;

// {{{ imports


import java.util.List;
import java.util.ArrayList;
import java.util.LinkedList;


import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;

// }}}

/**
 * A Thread pool manages a number of Threads to execute asynchronous calls.
 * It helps to avoid Thread creation overhead by keeping the Threads alive
 * and waiting for something to do. <p>
 *
 * The number of live Threads can be customized with the parameters to its
 * constructor. Threads are assigned first come first serve. The priority of
 * of the calling Thread does not make any difference. <p>
 *
 * Calls are executed by passing a Runnable to {@link #execute execute}. Its
 * run Method will be called from the next available Thread.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.4 $
 */
public class ThreadPool
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private final static Logger logger = new Logger (ThreadPool.class);

    /**
     * The Logger used for logging PoolThread methods.
     */
    private final static Logger ptLogger = new Logger (PoolThread.class);

    /**
     * Available Threads.
     */
    private List available = new ArrayList ();

    /**
     * Waiters for a Thread.
     */
    private LinkedList waiters = new LinkedList ();

    /**
     * Minimum number of threads.
     */
    private int minThreads;

    /**
     * Average number of threads.
     */
    private int avgThreads;

    /**
     * Maximum number of threads.
     */
    private int maxThreads;

    /**
     * Number of threads currently available.
     */
    private int currentThreads;

    /**
     * ThreadGroup used to create the threads in.
     */
    private ThreadGroup threadGroup;

    /**
     * Id counter for PoolThreads.
     */
    private static int count;

    // }}}

    // {{{ constructors

    /**
     * Create a new ThreadPool.
     *
     * @param   min     Initial number of threads to create.
     * @param   avg     Don't release threads unless there are more than avg
     *                  Threads around.
     * @param   max     Create at most that many Threads.
     */
    public ThreadPool (int min, int avg, int max)
    {
        this (min, avg, max, null);
    }

    /**
     * Create a new ThreadPool.
     *
     * @param   min     Initial number of threads to create.
     * @param   avg     Don't release threads unless there are more than avg
     *                  Threads around.
     * @param   max     Create at most that many Threads.
     * @param   group   Create all threads in this ThreadGroup
     */
    public ThreadPool (int min, int avg, int max, ThreadGroup group)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "ThreadPool(int,int,int,ThreadGroup)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "min: " + min + ", " +
                       "avg: " + avg + ", " +
                       "max: " + max + ", " +
                       "group: " + group);
        }

        minThreads = min;
        avgThreads = avg;
        maxThreads = max;
        threadGroup = group;

        if (minThreads > avgThreads
            || avgThreads > maxThreads
            || maxThreads <= 0) {
            throw new IllegalArgumentException
                ("Min, avg and max values don't make sense.");
        }

        // Create initial numbers of threads
        for (currentThreads = 0;
             currentThreads < minThreads;
             currentThreads++) {
            PoolThread pt = new PoolThread (threadGroup);
            pt.start();
            available.add(pt);
        }
    }

    // }}}

    // {{{ execute

    /**
     * Execute a runnable in the next available Thread. If there is no Thread
     * currently available and the maximum number of Threads is reached, the
     * calling Thread will be blocked, until a Thread is available.
     *
     * @param   run     The runnable whose run method will be called.
     */
    public void execute(Runnable run)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "execute(Runnable)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "run: " + run);
        }
        allocate().execute(run);
    }

    // }}}
    // {{{ allocate

    private synchronized PoolThread allocate()
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "allocate()", "");
        }
        if (available.size() > 0) {
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG, "allocate()",
                           "Thread available");
            }
            return (PoolThread) available.remove(0);
        }
        if (currentThreads < maxThreads) {
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG, "allocate()",
                           "Must create new Thread");
            }
            currentThreads++;
            PoolThread pt = new PoolThread (threadGroup);
            pt.start();
            return pt;
        }
        if (logger.level >= Log.DBG) {
            logger.log(Log.DBG, "allocate()",
                       "Waiting for Thread");
        }
        ResourceWaiter waiter = new ResourceWaiter (this);
        waiters.add(waiter);
        PoolThread pt = (PoolThread) waiter.waitFor(-1);
        if (logger.level >= Log.DBG) {
            logger.log(Log.DBG, "allocate()",
                       "got it: " + pt);
        }
        return pt;
    }

    // }}}
    // {{{ release

    private synchronized void release(PoolThread thread)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "release(PoolThread)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "thread: " + thread);
        }
        if (waiters.size() > 0) {
            ((ResourceWaiter) waiters.remove(0)).handResource(thread);
        } else if (currentThreads > avgThreads
                   && available.size() >= minThreads) {
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG, "release(PoolThread)",
                           "Terminating Thread " + thread);
            }
            thread.terminate();
            currentThreads--;
        } else {
            available.add(thread);
        }
    }

    // }}}

    // {{{ class PoolThread

    /**
     * A worker Thread used by the ThreadPool.
     */
    private class PoolThread extends Thread
    {
        // {{{ variables

        /**
         * The task to execute.
         */
        private Runnable runner;

        /**
         * Signal to quit the PoolThread.
         */
        private boolean quit;

        // }}}

        // {{{ constructor

        /**
         * Create a new PoolThread.
         *
         * @param       group   The ThreadGroup of which the PoolThread should
         *                      be a member.
         */
        public PoolThread (ThreadGroup group)
        {
            super(group, "PoolThread-" + count);

            if (ptLogger.level >= Log.MTD) {
                ptLogger.log(Log.MTD, "PoolThread()", "count: " + count++);
            }
            setDaemon(true);
        }

        // }}}
        // {{{ run

        /**
         * Wait for a tasks to become available and execute them.
         */
        public void run()
        {
            if (ptLogger.level >= Log.MTD) {
                ptLogger.log(Log.MTD, "run()", "");
            }
            Runnable runnable;
            while (true) {
                synchronized (this) {
                    while (runner == null && !quit) {
                        try {
                            this.wait();
                        } catch (InterruptedException ex) {
                        }
                    }
                    runnable = runner;
                    runner = null;
                    this.notifyAll();
                    if (quit) {
                        break;
                    }
                }
                try {
                    if (ptLogger.level >= Log.DBG) {
                        ptLogger.log(Log.DBG, "run()",
                                    "Executing " + runnable);
                    }
                    runnable.run();
                } catch (Throwable th) {
                    if (ptLogger.level >= Log.ERR) {
                        ptLogger.log("run()", th);
                    }
                }
                release(this);
            }
        }



        // }}}
        // {{{ execute

        /**
         * Execute one task.
         *
         * @param       runnable        The task to execute.
         */
        public synchronized void execute(Runnable runnable)
        {
            if (ptLogger.level >= Log.MTD) {
                ptLogger.log(Log.MTD, "execute(Runnable)",
                           ptLogger.level < Log.MTDDETAIL ? "" :
                           "runnable: " + runnable);
            }
            while (runner != null) {
                try {
                    this.wait();
                } catch (InterruptedException ex) {
                }
            }
            runner = runnable;
            this.notifyAll();
        }



        // }}}
        // {{{ terminate

        /**
         * Quit waiting for tasks and terminate the PoolThread.
         */
        public synchronized void terminate()
        {
            if (ptLogger.level >= Log.MTD) {
                ptLogger.log(Log.MTD, "terminate()", "");
            }
            quit = true;
            this.notifyAll();
        }

        // }}}
    }

    // }}}
}
