// {{{ copyright

/********************************************************************
 *
 * $Id: Visitor.jpp,v 1.7 2001/02/17 09:41:29 gs Exp $
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is qfs.de code.
 *
 * The Initial Developer of the Original Code is Gregor Schmid.
 * Portions created by Gregor Schmid are
 * Copyright (C) 1999 Quality First Software, Gregor Schmid.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 *******************************************************************/

// }}}

package de.qfs.lib.util;

// {{{ imports

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import java.util.Hashtable;

import de.qfs.lib.log.Log;
import de.qfs.lib.log.Logger;


import java.util.Collections;
import java.util.HashMap;
import java.util.Map;



import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;


// }}}

/**
 * This is a generic multi-method implementation of the Visitor pattern. It
 * is completely dynamic, requiring no compile time information. <p>
 *
 * A Visitor is any Object that implements any method of the signature: <br>
 * <code>public Object visit (&lt;someClass&gt;) throws ...</code><br>
 *
 * This method can be called indirectly via {@link #visit Visitor.visit}, eg:
 * <pre>
 * class DoSomethingVisitor
 * {
 *     // String visitor
 *     public Object visit (String string)
 *     {
 *         return doSomething(string);
 *     }
 *
 *     // Number visitor
 *     public Object visit (Number num)
 *     {
 *         return doSomething(num);
 *     }
 *
 *     // Fallback method
 *     public Object visit (Object object)
 *     {
 *         return null;
 *     }
 * }
 *
 * // visit a String
 * try {
 *     Object result = Visitor.visit(new DoSomethingVisitor (), someString);
 * } catch (Exception ex) {
 *     ...
 * }
 *
 * </pre>
 *
 * Method lookup is cached for improved speed.
 *
 * @author      Gregor Schmid
 * @version     $Revision: 1.7 $
 */
public abstract class Visitor
{
    // {{{ variables

    /**
     * The Logger used for logging.
     */
    private static Logger logger = new Logger (Visitor.class);

    /**
     * Lookup cache for methods.
     */
    private static Map cache = new HashMap ();

    /**
     * Static search key to avoid needless key object creation.
     */
    private static Key searchKey = new Key ();

    // }}}

    // {{{ visit

    /**
     * Call the visitor's visit method, that most closely matches the visited
     * object's class. The visit method is determined by first looking for one
     * that expects the Object's class as argument, then one that expects the
     * Object's superclass and so on.
     *
     * @param   visitor The visitor whose visit method is called.
     * @param   object  The object to visit.
     *
     * @return  The result of the visitor's matching visit method.
     *
     * @throws  InvocationTargetException       If the visitor's visit method
     *          declares and throws any checked exceptions, they will be
     *          wrapped in an InvocationTargetException. Undeclared
     *          RuntimeExceptions will be passed on directly.
     * @throws  NoSuchMethodException   If the visitor doesn't implement any
     *          matching visit method.
     */
    public static Object visit(Object visitor, Object object)
         throws InvocationTargetException, NoSuchMethodException
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "visit(Object,Object)",
                       logger.level < Log.MTDDETAIL ? "" :
                       "visitor: " + visitor + ", " +
                       "object: " + object);
        }

        Class vClass = visitor.getClass();
        Class oClass = object.getClass();

        Method visit = lookup(vClass, oClass);

        if (visit == null) {
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG, "visit(Object,Object)",
                           "Cache miss");
            }

            try {
                Class[] ca = new Class[] {oClass};
                while (visit == null && ca[0] != null) {
                    try {
                        visit = vClass.getMethod
                            ("visit", ca);
                    } catch (NoSuchMethodException ex) {
                        ca[0] = ca[0].getSuperclass();
                    }
                }
            } catch (IllegalArgumentException ex) {
                // Shouldn't happen
                if (logger.level >= Log.ERR) {
                    logger.log("visit(Object,Object)", ex);
                }
                return null;
            }
            if (visit == null) {
                throw new NoSuchMethodException
                    (vClass.getName() + ".visit(" + oClass.getName() + ")");
            } else {
                store(visit, vClass, oClass);
            }
        } else {
            if (logger.level >= Log.DBG) {
                logger.log(Log.DBG, "visit(Object,Object)",
                           "Cache hit, key: " + searchKey);
            }
        }

        try {
            return visit.invoke(visitor, new Object[] {object});
        } catch (InvocationTargetException ex) {
            Class[] declared = visit.getExceptionTypes();
            for (int i = 0; i < declared.length; i++) {
                if (declared[i].isInstance(ex.getTargetException())) {
                    // this Exception was expected
                    throw ex;
                }
            }
            // If it was not a declared Exception it should be a
            // RuntimeException or an Error
            if (ex.getTargetException() instanceof RuntimeException) {
                throw (RuntimeException) ex.getTargetException();
            }
            if (ex.getTargetException() instanceof Error) {
                throw (Error) ex.getTargetException();
            }
            // What else is there ?
            if (logger.level >= Log.ERR) {
                logger.log("visit(Object,Object)", ex,
                           "Unknown exception type");
            }
            return null;
        } catch (IllegalAccessException ex) {
            // Shouldn't happen
            if (logger.level >= Log.ERR) {
                logger.log("visit(Object,Object)", ex);
            }
            return null;
        } catch (IllegalArgumentException ex) {
            // Shouldn't happen
            if (logger.level >= Log.ERR) {
                logger.log("visit(Object,Object)", ex);
                if (logger.level >= Log.ERR) {
                    logger.log(Log.ERR, "visit(Object,Object)",
                               "Error occured with visitor: " + visitor +
                               "\nobject: " + object +
                               "\nmethod: " + visit +
                               "\nkey: " + searchKey);
                }
            }
            return null;
        }
    }

    // }}}
    // {{{ safeVisit

    /**
     * This method is identical to {@link #visit visit} except for Exception
     * handling. If the visitor doesn't have a matching visit method, an error
     * is logged and null is returned. It is also expected, but not verified,
     * that the visitor's visit method does not declare any exceptions. If it
     * does throw a declared exception anyway, it is logged as an error and
     * null is returned. RuntimeExceptions are passed on as usual.
     *
     * @param   visitor The visitor whose visit method is called.
     * @param   object  The object to visit.
     *
     * @return  The result of the visitor's matching visit method.
     */
    public static Object safeVisit(Object visitor, Object object)
    {
        if (logger.level >= Log.MTD) {
            logger.log(Log.MTD, "safevisit(Object,Object)", "");
        }

        try {
            return visit(visitor, object);
        } catch (NoSuchMethodException ex) {
            if (logger.level >= Log.ERR) {
                logger.log("safeVisit(Object,Object)", ex);
            }
            return null;
        } catch (InvocationTargetException ex) {
            if (logger.level >= Log.ERR) {
                logger.log("safeVisit(Object,Object)",
                           ex.getTargetException());
            }
            return null;
        }
    }

    // }}}

    // {{{ lookup

    /**
     * Lookup a method in the cache. <p>
     *
     * Used to synchronize access to the method cache.
     *
     * @param   vClass  The visitor's class.
     * @param   oClass  The visited object's class.
     *
     * @return  The method found or null.
     */
    private static synchronized Method lookup(Class vClass, Class oClass)
    {
        searchKey.visitorClass = vClass;
        searchKey.objectClass = oClass;
        return (Method) cache.get(searchKey);
    }

    // }}}
    // {{{ store

    /**
     * Store a method in the cache. <p>
     *
     * Used to synchronize access to the method cache.
     *
     * @param   mathod  The method to store.
     * @param   vClass  The visitor's class.
     * @param   oClass  The visited object's class.
     *
     */
    private static synchronized void store(Object value,
                                           Class vClass, Class oClass)
    {
        Key key = new Key(vClass, oClass);
        if (logger.level >= Log.DBG) {
            logger.log(Log.DBG, "store(Object,Class,Class)",
                       "Storing " + value + " for " + key);
        }

        cache.put(key, value);
    }

    // }}}

    // {{{ class Key

    /**
     * Struct that contains two classes and provides hashCode and equals
     * methods, so it can be used as a Hashtable key.
     */
    private static class Key
    {
        // {{{ variables

        /**
         * The visitor's class.
         */
        public Class visitorClass;

        /**
         * The visited object's class.
         */
        public Class objectClass;

        // }}}

        // {{{ constructors

        /**
         * Create a new key.
         */
        public Key()
        {
        }

        /**
         * Create a new key, initializing its values.
         *
         * @param       visitorClass    The visitor's class.
         * @param       objectClass     The visited object's class.
         */
        public Key(Class visitorClass, Class objectClass)
        {
            this.visitorClass = visitorClass;
            this.objectClass = objectClass;
        }

        // }}}

        // {{{ hashCode

        /**
         * Get the hash code for the key.
         *
         * @return      The Key's hash code.
         */
        public int hashCode()
        {
            return (visitorClass == null ? 0 : visitorClass.hashCode())
                + (objectClass == null ? 0 : objectClass.hashCode());
        }

        // }}}
        // {{{ equals

        /**
         * Compare the Key to another Object.
         *
         * @param       o       The Object to compare to.
         *
         * @return      True, if the Object is a Key and has equal memebers.
         */
        public boolean equals(Object o)
        {
            if (logger.level >= Log.MTD) {
                logger.log(Log.MTD, "equals(Object)",
                           logger.level < Log.MTDDETAIL ? "" :
                           "this: " + this +
                           "\no: " + o);
            }
            if (this == o) {
                if (logger.level >= Log.DBG) {
                    logger.dumpStack(Log.DBG, "equals(Object)", "");
                }
                return true;
            }
            if (! (o instanceof Key)) {
                return false;
            }
            Key key = (Key) o;

            return ((visitorClass == null && key.visitorClass == null)
                    || (visitorClass != null
                        && visitorClass.equals(key.visitorClass)))
                && ((objectClass == null && key.objectClass == null)
                    || (objectClass != null
                        && objectClass.equals(key.objectClass)));
        }

        // }}}

        // {{{ toString

        /**
         * Get a String representation for the Key.
         *
         * @return      The String representation of the Key.
         */
        public String toString()
        {
            return "Key@" + super.hashCode() +
                "[" + visitorClass.getName() +
                "," + objectClass.getName() +
                "," + hashCode() + "]";
        }

        // }}}
    }

    // }}}

    // {{{ class UnitTest

    /**
     * Test cases for the Visitor class.
     */
    public static class UnitTest extends TestCase
    {
        // {{{ variables

        /**
         * The visitor to test.
         */
        private TestVisitor visitor;

        // }}}

        // {{{ constructor

        /**
         * Create a new UnitTest.
         *
         * @param       name    The name of the test.
         */
        public UnitTest(String name)
        {
                super(name);
        }

        // }}}

        // {{{ setUp

        /**
         * Set up the test fixture.
         */
        protected void setUp()
        {
            //  Log.setOutputLevel(10);
            //  Log.setPreQueueLevel(10);
            //  Log.setQueueing(false);
            //  Logger.setDefaultLogLevel(10);
            visitor = new TestVisitor ();
        }

        // }}}
        // {{{ suite

        /**
         * Get the suite of tests.
         *
         * @return      The tests.
         */
        public static Test suite()
        {
                return new TestSuite(UnitTest.class);
        }

        // }}}

        // {{{ testLookup

        /**
         * Test method lookup.
         */
        public void testLookup()
        {
            try {
                String ret = (String) Visitor.visit(visitor, "a");
                assert(ret.equals("String: a"));
                ret = (String) Visitor.visit(visitor, "b");
                assert(ret.equals("String: b"));
            } catch (Exception ex) {
                if (logger.level >= Log.ERR) {
                    logger.log("testLookup()", ex);
                }
                assert(false);
            }
        }

        // }}}
        // {{{ testNoSuchMethod

        /**
         * Test failed method lookup.
         */
        public void testNoSuchMethod()
        {
            try {
                Visitor.visit(visitor, new Object ());
                assert(false);
            } catch (NoSuchMethodException ex) {
            } catch (InvocationTargetException ex) {
                if (logger.level >= Log.ERR) {
                    logger.log("testLookup()", ex);
                }
                assert(false);
            }
        }

        // }}}
        // {{{ testInheritance

        /**
         * Test method lookup for baseclass.
         */
        public void testInheritance()
        {
            try {
                String ret = (String) Visitor.visit(visitor, new Long (3));
                assert(ret.equals("Number: 3"));
                ret = (String) Visitor.visit(visitor, new Long (5));
                assert(ret.equals("Number: 5"));
                ret = (String) Visitor.visit(visitor, new Integer (3));
                assert(ret.equals("Integer: 3"));
                ret = (String) Visitor.visit(visitor, new Integer (5));
                assert(ret.equals("Integer: 5"));
            } catch (Exception ex) {
                if (logger.level >= Log.ERR) {
                    logger.log("testLookup()", ex);
                }
                assert(false);
            }
        }

        // }}}
        // {{{ testException

        /**
         * Test Exceptions in visit method.
         */
        public void testException()
        {
            try {
                Visitor.visit(visitor, new Boolean (true));
                assert(false);
            } catch (InvocationTargetException ex) {
                assert(ex.getTargetException() instanceof
                       IllegalArgumentException);
            } catch (NoSuchMethodException ex) {
                if (logger.level >= Log.ERR) {
                    logger.log("testLookup()", ex);
                }
                assert(false);
            }

            try {
                Visitor.visit(visitor, new Boolean (false));
                assert(false);
            } catch (NullPointerException ex) {
            } catch (Exception ex) {
                if (logger.level >= Log.ERR) {
                    logger.log("testLookup()", ex);
                }
                assert(false);
            }
        }

        // }}}

        // {{{ additional classes

        /**
         * Simple visitor for test cases.
         */
        private static class TestVisitor
        {
            public Object visit(String s)
            {
                return "String: " + s;
            }

            public Object visit(Number n)
            {
                return "Number: " + n;
            }

            public Object visit(Integer i)
            {
                return "Integer: " + i;
            }

            public Object visit(Boolean b)
                throws IllegalArgumentException
            {
                if (b.booleanValue()) {
                    throw new IllegalArgumentException ("Test");
                } else {
                    throw new NullPointerException ("Test");
                }
            }
        }

        // }}}
    }

    // }}}

}

